@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
  *  stress_reverse64
  *	generic fast-ish 64 bit reverse
  */
-static inline uint64_t ALWAYS_INLINE OPTIMIZE3 stress_reverse64(register uint64_t x)
+static inline uint64_t ALWAYS_INLINE stress_reverse64(register uint64_t x)
 {
 #if defined(HAVE_BUILTIN_BITREVERSE)
 	return __builtin_bitreverse64(x);
@@ -41,7 +41,7 @@ static inline uint64_t ALWAYS_INLINE OPTIMIZE3 stress_reverse64(register uint64_
  *  stress_reverse32
  *	generic fast-ish 32 bit reverse
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_reverse32(register uint32_t x)
+static inline uint32_t ALWAYS_INLINE stress_reverse32(register uint32_t x)
 {
 #if defined(HAVE_BUILTIN_BITREVERSE)
 	return __builtin_bitreverse32(x);
@@ -59,7 +59,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_reverse32(register uint32_
  *  stress_reverse16
  *	generic fast-ish 16 bit reverse
  */
-static inline uint16_t ALWAYS_INLINE OPTIMIZE3 stress_reverse16(register uint16_t x)
+static inline uint16_t ALWAYS_INLINE stress_reverse16(register uint16_t x)
 {
 #if defined(HAVE_BUILTIN_BITREVERSE)
 	return __builtin_bitreverse16(x);
@@ -76,7 +76,7 @@ static inline uint16_t ALWAYS_INLINE OPTIMIZE3 stress_reverse16(register uint16_
  *  stress_reverse8
  *	generic fast-ish 8 bit reverse
  */
-static inline uint8_t ALWAYS_INLINE OPTIMIZE3 stress_reverse8(register uint8_t x)
+static inline uint8_t ALWAYS_INLINE stress_reverse8(register uint8_t x)
 {
 #if defined(HAVE_BUILTIN_BITREVERSE)
 	return __builtin_bitreverse8(x);
@@ -92,7 +92,7 @@ static inline uint8_t ALWAYS_INLINE OPTIMIZE3 stress_reverse8(register uint8_t x
  *  stress_swap32()
  *	swap order of bytes of a uint32_t value
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_swap32(uint32_t val)
+static inline uint32_t ALWAYS_INLINE stress_swap32(uint32_t val)
 {
 #if defined(HAVE_BUILTIN_BSWAP32)
 	return __builtin_bswap32(val);
@@ -108,7 +108,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_swap32(uint32_t val)
  *  stress_bitreverse32()
  *	reverse bits in a uint32_t value
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_bitreverse32(const uint32_t val)
+static inline uint32_t ALWAYS_INLINE stress_bitreverse32(const uint32_t val)
 {
 #if defined(HAVE_BUILTIN_BITREVERSE)
 	return  __builtin_bitreverse32(val);
@@ -130,7 +130,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_bitreverse32(const uint32_
  *  stress_popcount64()
  *	population count (count number of 1 bits) in a uint64_t value
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_popcount64(const uint64_t val)
+static inline uint32_t ALWAYS_INLINE stress_popcount64(const uint64_t val)
 {
 #if defined(HAVE_BUILTIN_POPCOUNT) &&	\
     !defined(HAVE_COMPILER_PCC)
@@ -149,7 +149,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_popcount64(const uint64_t
  *  stress_popcount32()
  *	population count (count number of 1 bits) in a uint32_t value
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_popcount32(const uint32_t val)
+static inline uint32_t ALWAYS_INLINE stress_popcount32(const uint32_t val)
 {
 #if defined(HAVE_BUILTIN_POPCOUNT)
 	return  __builtin_popcount((unsigned int)val);
@@ -167,7 +167,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_popcount32(const uint32_t
  *  stress_parity32
  *	parity check in a uint32_t value
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_parity32(const uint32_t val)
+static inline uint32_t ALWAYS_INLINE stress_parity32(const uint32_t val)
 {
 #if defined(HAVE_BUILTIN_PARITY)
 	return  __builtin_parity((unsigned int)val);
@@ -187,7 +187,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_parity32(const uint32_t va
  *  stress_nextpwr2()
  *	round to next power of 2
  */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_nextpwr2(const uint32_t val)
+static inline uint32_t ALWAYS_INLINE stress_nextpwr2(const uint32_t val)
 {
 	register uint32_t v = val - 1;
 
