@@ -53,6 +53,12 @@
 #define	shim_memcmp(dst, src, n)	memcmp((dst), (src), (n))
 #endif
 
+#if defined(HAVE_BUILTIN_STRDUP)
+#define	shim_strdup(str)		__builtin_strdup((str))
+#else
+#define	shim_strdup(str)		strdup((str))
+#endif
+
 #if defined(HAVE_BUILTIN_CABS)
 #define shim_cabs(x)		__builtin_cabs((x))
 #else
@@ -604,7 +610,11 @@
 #if defined(HAVE_BUILTIN_TANL)
 #define shim_tanl(x)		__builtin_tanl((x))
 #else
+#if defined(HAVE_TANL)
 #define shim_tanl(x)		tanl((x))
+#else
+#define shim_tanl(x)		((long double)shim_tan((double)(x)))
+#endif
 #endif
 
 #if defined(HAVE_BUILTIN_TANHF)
@@ -622,7 +632,11 @@
 #if defined(HAVE_BUILTIN_TANHL)
 #define shim_tanhl(x)		__builtin_tanhl((x))
 #else
+#if defined(HAVE_TANHL)
 #define shim_tanhl(x)		tanhl((x))
+#else
+#define	shim_tanhl(x)		((long double)shim_tanh((double)(x)))
+#endif
 #endif
 
 #if defined(HAVE_BUILTIN_SQRT)
@@ -785,10 +799,20 @@
 #define shim_ynl(n, x)		ynl((n), (x))
 #endif
 
+#if defined(HAVE_BUILTIN_ROUND)
+#define shim_round(x)		__builtin_round((x))
+#else
+#define shim_round(x)		round((x))
+#endif
+
 #if defined(HAVE_BUILTIN_ROUNDL)
 #define shim_roundl(x)		__builtin_roundl((x))
 #else
+#if defined(HAVE_ROUNDL)
 #define shim_roundl(x)		roundl((x))
+#else
+#define shim_roundl(x)		((long double)shim_round((double)(x)))
+#endif
 #endif
 
 #if defined(HAVE_BUILTIN_STDC_ROTATE_LEFT)
