@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King
+ * Copyright (C) 2022-2025 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,8 @@
 #ifndef CORE_CAPABILITIES_H
 #define CORE_CAPABILITIES_H
 
+#include "stress-ng.h"
+
 #define SHIM_CAP_IS_ROOT		(-1)
 
 /* POSIX-draft defined capabilities */
@@ -263,4 +265,8 @@
 #define SHIM_CAP_BPF			SHIM_CAP_IS_ROOT
 #endif
 
+void stress_getset_capability(void);
+WARN_UNUSED bool stress_check_capability(const int capability);
+WARN_UNUSED int stress_drop_capabilities(const char *name);
+
 #endif
