@@ -17,7 +17,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-asm-x86.h"
 #include "core-config-check.h"
+#include "core-cpu.h"
+#include "core-put.h"
+#include "core-signal.h"
 
 #include <ctype.h>
 
@@ -25,6 +30,46 @@
 #include <termio.h>
 #endif
 
+#if defined(STRESS_ARCH_X86) &&		\
+    defined(HAVE_ASM_X86_LAHF)
+
+static volatile bool stress_config_sigill_trapped;
+
+static void stress_config_sigill_handler(int sig)
+{
+	(void)sig;
+
+	stress_config_sigill_trapped = true;
+}
+
+/*
+ *  stress_config_check_lahf_lm()
+ *	check if x86-64 has lahf_lm CPU flag set and if so check
+ *	if lahf opcode works without triggering a SIGILL as can
+ *	occur on the Apple Rosetta 2 emulator of x86-64
+ */
+static void stress_config_check_lahf_lm(void)
+{
+	if (stress_cpu_x86_has_lahf_lm()) {
+		uint8_t value;
+		struct sigaction old_action;
+
+		stress_config_sigill_trapped = false;
+
+		if (stress_sighandler(__func__, SIGILL, stress_config_sigill_handler, &old_action) < 0)
+			return;
+
+		value = stress_asm_lahf();
+		stress_uint8_put(value);
+
+		(void)sigaction(SIGILL, &old_action, NULL);
+
+		if (stress_config_sigill_trapped)
+			pr_warn("note: x86 processor CPUID lahf_lm bit set but the lahf opcode causes an illegal opcode trap\n");
+	}
+}
+#endif
+
 #if defined(__linux__)
 static int stress_config_check_cpu_filter(const struct dirent *d)
 {
@@ -67,46 +112,11 @@ void stress_config_check(void)
 	size_t shmall = 0, freemem = 0, totalmem = 0, freeswap = 0, totalswap = 0, freetotal;
 
 #if defined(__linux__)
-	{
-		char path[PATH_MAX];
-		const char *str;
-		bool is_snap = false;
-
-		str = getenv("SNAP_NAME");
-		if (str) {
-			char *lowstr = strdup(str);
-
-			if (lowstr) {
-				char *ptr;
-
-				for (ptr = lowstr; *ptr; ptr++)
-					*ptr = (char)tolower((int)*ptr);
-
-				if (strstr(lowstr, "stress"))
-					is_snap = true;
-				free(lowstr);
-			}
-		}
-		str = stress_get_proc_self_exe(path, sizeof(path));
-		if (str) {
-			if (strncmp("/snap/", path, 6) == 0)
-				is_snap = true;
-		}
-		if (is_snap) {
-			bool oom_avoid = true;
-
-			pr_warn("WARNING: stress-ng appears to be a snap and may not "
-				"run correctly inside this unsupported environment\n");
-			pr_warn("NO support will be provided for the snap based stress-ng\n");
-			pr_inf("note: forcibly enabling --oom-avoid for this environment\n");
-			stress_set_setting_global("oom-avoid", TYPE_ID_BOOL, &oom_avoid);
-		}
-	}
-
 	if (g_opt_flags & OPT_FLAGS_METRICS) {
 		static const char autogroup_path[] = "/proc/sys/kernel/sched_autogroup_enabled";
 		static const char cpu_path[] = "/sys/devices/system/cpu";
 		static const char boost_path[] = "/sys/devices/system/cpu/cpufreq/boost";
+		static const char turbo_path[] = "/sys/devices/system/cpu/intel_pstate/no_turbo";
 		uint64_t value;
 		struct dirent **namelist = NULL;
 		int n, i;
@@ -131,9 +141,16 @@ void stress_config_check(void)
 		if ((stress_config_read(boost_path, &value) != -1) && (value == 0)) {
 			pr_inf("note: boost is disabled and this may impact "
 				"top performance; setting %s to 1 may improve "
-				"performance.", boost_path);
+				"performance.\n", boost_path);
 		}
 
+		if ((stress_config_read(turbo_path, &value) != -1) && (value == 1)) {
+			pr_inf("note: turbo is disabled and this may impact "
+				"top performance; setting %s to 0 may improve "
+				"performance.\n", turbo_path);
+		}
+
+
 		n = scandir(cpu_path, &namelist, stress_config_check_cpu_filter, alphasort);
 		for (i = 0; i < n; i++) {
 			char filename[PATH_MAX];
@@ -162,4 +179,11 @@ void stress_config_check(void)
 		pr_inf("note: system has only %zu MB of free memory and swap, "
 			"recommend using --oom-avoid\n", freetotal / (size_t)MB);
 	}
+
+	/* Now CPU specific functional checks */
+#if defined(STRESS_ARCH_X86_64) &&	\
+    defined(HAVE_ASM_X86_LAHF)
+	stress_config_check_lahf_lm();
+#endif
+
 }
