@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -96,10 +96,10 @@ static const int madvise_options[] = {
 #endif
 
 /*
- *  stress_madvise_random()
+ *  stress_madvise_randomize()
  *	apply random madvise setting to a memory region
  */
-int stress_madvise_random(void *addr, const size_t length)
+int stress_madvise_randomize(void *addr, const size_t length)
 {
 #if defined(HAVE_MADVISE)
 	if (g_opt_flags & OPT_FLAGS_MMAP_MADVISE) {
@@ -115,14 +115,30 @@ int stress_madvise_random(void *addr, const size_t length)
 	return 0;
 }
 
+/*
+ *  stress_madvise_random()
+ *	apply MADV_RANDOM for page read order hint
+ */
+int stress_madvise_random(void *addr, const size_t length)
+{
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_RANDOM)
+	return madvise(addr, length, MADV_RANDOM);
+#else
+	(void)addr;
+	(void)length;
+	return 0;
+#endif
+}
+
 /*
  *  stress_madvise_mergeable()
  *	apply MADV_MERGEABLE for kernel same page merging
  */
 int stress_madvise_mergeable(void *addr, const size_t length)
 {
-#if defined(MADV_MERGEABLE) && \
-    defined(HAVE_MADVISE)
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_MERGEABLE)
 	return madvise(addr, length, MADV_MERGEABLE);
 #else
 	(void)addr;
@@ -131,11 +147,62 @@ int stress_madvise_mergeable(void *addr, const size_t length)
 #endif
 }
 
+/*
+ *  stress_madvise_collapse()
+ *	where possible collapse mapping into THP
+ */
+int stress_madvise_collapse(void *addr, size_t length)
+{
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_COLLAPSE)
+	return madvise(addr, length, MADV_COLLAPSE);
+#else
+	(void)addr;
+	(void)length;
+	return 0;
+#endif
+}
+
+/*
+ *  stress_madvise_willneed()
+ *	where possible fetch pages early
+ */
+int stress_madvise_willneed(void *addr, size_t length)
+{
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_WILLNEED)
+	return madvise(addr, length, MADV_WILLNEED);
+#else
+	(void)addr;
+	(void)length;
+	return 0;
+#endif
+}
+
+/*
+ *  stress_madvise_nohugepage()
+ *	apply MADV_NOHUGEPAGE to force as many PTEs as possible
+ */
+int stress_madvise_nohugepage(void *addr, const size_t length)
+{
+#if defined(HAVE_MADVISE) && \
+    defined(MADV_NOHUGEPAGE)
+	return madvise(addr, length, MADV_NOHUGEPAGE);
+#else
+	(void)addr;
+	(void)length;
+	return 0;
+#endif
+}
+
 /*
  *  stress_madvise_pid_all_pages()
  *	apply madvise advise to all pages in a progress
  */
-void stress_madvise_pid_all_pages(const pid_t pid, const int advise)
+void stress_madvise_pid_all_pages(
+	const pid_t pid,
+	const int *advice,
+	const size_t n_advice)
 {
 #if defined(HAVE_MADVISE) &&	\
     defined(__linux__)
@@ -150,8 +217,10 @@ void stress_madvise_pid_all_pages(const pid_t pid, const int advise)
 		return;
 	while (fgets(buf, sizeof(buf), fp)) {
 		void *start, *end, *offset;
-		int major, minor, n;
+		int n;
+		unsigned int major, minor;
 		uint64_t inode;
+		const size_t page_size = stress_get_page_size();
 		char prot[5];
 
 		n = sscanf(buf, "%p-%p %4s %p %x:%x %" PRIu64 " %4095s\n",
@@ -162,19 +231,28 @@ void stress_madvise_pid_all_pages(const pid_t pid, const int advise)
 		if (start >= end)
 			continue;	/* invalid address range */
 
-		VOID_RET(int, madvise(start, (size_t)((uint8_t *)end - (uint8_t *)start), advise));
+		if (n_advice == 1) {
+			VOID_RET(int, madvise(start, (size_t)((uint8_t *)end - (uint8_t *)start), advice[0]));
+		} else {
+			register uint8_t *ptr;
+
+			for (ptr = start; ptr < (uint8_t *)end; ptr += page_size) {
+				size_t idx = stress_mwc8modn((uint8_t)n_advice);
+				const int new_advice = advice[idx];
+
+				VOID_RET(int, madvise((void *)ptr, page_size, new_advice));
+			}
+		}
 
 		/*
 		 *  Readable protection? read pages
 		 */
 		if ((prot[0] == 'r') && (path[0] != '[')) {
-			const size_t page_size = stress_get_page_size();
+			register volatile uint8_t *vptr = (volatile uint8_t *)start;
 
-			volatile uint8_t *ptr = (volatile uint8_t *)start;
-
-			while (ptr < (uint8_t *)end) {
-				(*ptr);
-				ptr += page_size;
+			while (vptr < (uint8_t *)end) {
+				(*vptr);
+				vptr += page_size;
 			}
 		}
 	}
@@ -182,6 +260,7 @@ void stress_madvise_pid_all_pages(const pid_t pid, const int advise)
 	(void)fclose(fp);
 #else
 	(void)pid;
-	(void)advise;
+	(void)advice;
+	(void)n_advice;
 #endif
 }
