@@ -19,10 +19,12 @@
 #ifndef CORE_MADVISE_H
 #define CORE_MADVISE_H
 
+extern int stress_madvise_randomize(void *addr, const size_t length);
 extern int stress_madvise_random(void *addr, const size_t length);
 extern int stress_madvise_mergeable(void *addr, const size_t length);
 extern int stress_madvise_collapse(void *addr, size_t length);
+extern int stress_madvise_willneed(void *addr, const size_t length);
 extern int stress_madvise_nohugepage(void *addr, const size_t length);
-extern void stress_madvise_pid_all_pages(const pid_t pid, const int advise);
+extern void stress_madvise_pid_all_pages(const pid_t pid, const int *advice, const size_t n_advice);
 
 #endif
