@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,7 +46,9 @@ typedef struct {
 
 static const stress_domain_t domains[] = {
 	{ "ipv4",	AF_INET,	DOMAIN_INET },
+#if defined(AF_INET6)
 	{ "ipv6",	AF_INET6,	DOMAIN_INET6 },
+#endif
 	{ "unix",	AF_UNIX,	DOMAIN_UNIX },
 };
 
@@ -61,12 +63,12 @@ int stress_net_interface_exists(const char *interface, const int domain, struct
 	struct ifaddrs *ifaddr, *ifa;
 	int ret = -1;
 
-	if (!interface)
+	if (UNLIKELY(!interface))
 		return -1;
-	if (!addr)
+	if (UNLIKELY(!addr))
 		return -1;
 
-	if (getifaddrs(&ifaddr) < 0)
+	if (UNLIKELY(getifaddrs(&ifaddr) < 0))
 		return -1;
 	for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
 		if (!ifa->ifa_addr)
@@ -76,7 +78,7 @@ int stress_net_interface_exists(const char *interface, const int domain, struct
 		if (ifa->ifa_addr->sa_family != domain)
 			continue;
 		if (strcmp(ifa->ifa_name, interface) == 0) {
-			(void)memcpy(addr, ifa->ifa_addr, sizeof(*addr));
+			(void)shim_memcpy(addr, ifa->ifa_addr, sizeof(*addr));
 			ret = 0;
 			break;
 		}
@@ -177,7 +179,7 @@ int stress_set_sockaddr_if(
 	*len = 0;
 
 	/* omit ports 0..1023 */
-	if (sin_port < 1024)
+	if (UNLIKELY(sin_port < 1024))
 		sin_port += 1024;
 
 	switch (domain) {
@@ -240,7 +242,7 @@ int stress_set_sockaddr_if(
 		(void)shim_memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
 		(void)snprintf(addr.sun_path, sizeof(addr.sun_path),
-			"/tmp/stress-ng-%jd-%" PRIu32,
+			"/tmp/stress-ng-%" PRIdMAX "-%" PRIu32,
 			(intmax_t)pid, instance);
 		*sockaddr = (struct sockaddr *)&addr;
 		*len = sizeof(addr);
@@ -275,7 +277,7 @@ int stress_set_sockaddr(
  *  stress_set_sockaddr_port()
  *	setup just the socket address port
  */
-void HOT stress_set_sockaddr_port(
+void stress_set_sockaddr_port(
 	const int domain,
 	const int port,
 	struct sockaddr *sockaddr)
@@ -312,11 +314,11 @@ void HOT stress_set_sockaddr_port(
  */
 static bool PURE stress_net_port_range_ok(const int start_port, const int end_port)
 {
-	if ((start_port > end_port))
+	if (UNLIKELY((start_port > end_port)))
 		return false;
-	if ((start_port < 0) || (start_port >= 65536))
+	if (UNLIKELY((start_port < 0) || (start_port >= 65536)))
 		return false;
-	if ((end_port < 0) || (end_port >= 65536))
+	if (UNLIKELY((end_port < 0) || (end_port >= 65536)))
 		return false;
 	return true;
 }
@@ -330,10 +332,10 @@ int stress_net_reserve_ports(const int start_port, const int end_port)
 {
 	int i, port = -1;
 
-	if (!stress_net_port_range_ok(start_port, end_port))
+	if (UNLIKELY(!stress_net_port_range_ok(start_port, end_port)))
 		return -1;
 
-	if (stress_lock_acquire(g_shared->net_port_map.lock) < 0)
+	if (UNLIKELY(stress_lock_acquire(g_shared->net_port_map.lock) < 0))
 		return -1;
 
 	if (LIKELY(start_port == end_port)) {
@@ -378,10 +380,35 @@ int stress_net_reserve_ports(const int start_port, const int end_port)
  */
 void stress_net_release_ports(const int start_port, const int end_port)
 {
-	if (stress_net_port_range_ok(start_port, end_port)) {
+	if (LIKELY(stress_net_port_range_ok(start_port, end_port))) {
 		int i;
 
 		for (i = start_port; i <= end_port; i++)
 			STRESS_CLRBIT(g_shared->net_port_map.allocated, i);
 	}
 }
+
+/*
+ *  stress_ipv4_checksum()
+ *	ipv4 data checksum
+ */
+uint16_t PURE OPTIMIZE3 stress_ipv4_checksum(uint16_t *ptr, const size_t sz)
+{
+	register uint32_t sum = 0;
+	register size_t n = sz;
+
+	if (UNLIKELY(!ptr))
+		return 0;
+
+	while (n > 1) {
+		sum += *ptr++;
+		n -= 2;
+	}
+
+	if (n)
+		sum += *(uint8_t*)ptr;
+	sum = (sum >> 16) + (sum & 0xffff);
+	sum += (sum >> 16);
+
+	return (uint16_t)~sum;
+}
