@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King
+ * Copyright (C) 2022-2025 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
     defined(HAVE_BUILTIN_SUPPORTS) &&			\
     defined(HAVE_BUILTIN_NONTEMPORAL_LOAD)
 /* Clang non-temporal loads */
-static inline __uint128_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load128(__uint128_t *addr)
+static inline __uint128_t ALWAYS_INLINE stress_nt_load128(__uint128_t *addr)
 {
 	return __builtin_nontemporal_load(addr);
 }
@@ -42,7 +42,7 @@ static inline __uint128_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load128(__uint128_t
 #if defined(HAVE_BUILTIN_SUPPORTS) &&	\
     defined(HAVE_BUILTIN_NONTEMPORAL_LOAD)
 /* Clang non-temporal load */
-static inline uint64_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load64(uint64_t *addr)
+static inline uint64_t ALWAYS_INLINE stress_nt_load64(uint64_t *addr)
 {
 	return __builtin_nontemporal_load(addr);
 }
@@ -55,7 +55,7 @@ static inline uint64_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load64(uint64_t *addr)
 #if defined(HAVE_BUILTIN_SUPPORTS) &&			\
     defined(HAVE_BUILTIN_NONTEMPORAL_LOAD)
 /* Clang non-temporal load */
-static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load32(uint32_t *addr)
+static inline uint32_t ALWAYS_INLINE stress_nt_load32(uint32_t *addr)
 {
 	return __builtin_nontemporal_load(addr);
 }
@@ -68,7 +68,7 @@ static inline uint32_t ALWAYS_INLINE OPTIMIZE3 stress_nt_load32(uint32_t *addr)
 #if defined(HAVE_BUILTIN_SUPPORTS) &&			\
     defined(HAVE_BUILTIN_NONTEMPORAL_LOAD)
 /* Clang non-temporal load */
-static inline double ALWAYS_INLINE OPTIMIZE3 stress_nt_load_double(double *addr)
+static inline double ALWAYS_INLINE stress_nt_load_double(double *addr)
 {
 	return __builtin_nontemporal_load(addr);
 }
