@@ -253,6 +253,7 @@ const struct option stress_long_options[] = {
 	{ "far-branch",		1,	0,	OPT_far_branch },
 	{ "far-branch-flush",	0,	0,	OPT_far_branch_flush },
 	{ "far-branch-ops",	1,	0,	OPT_far_branch_ops },
+	{ "far-branch-pageout",	0,	0,	OPT_far_branch_pageout },
 	{ "far-branch-pages",	1,	0,	OPT_far_branch_pages },
 	{ "fault",		1,	0,	OPT_fault },
 	{ "fault-ops",		1,	0,	OPT_fault_ops },
@@ -411,6 +412,7 @@ const struct option stress_long_options[] = {
 	{ "ioport",		1,	0,	OPT_ioport },
 	{ "ioport-ops",		1,	0,	OPT_ioport_ops },
 	{ "ioport-opts",	1,	0,	OPT_ioport_opts },
+	{ "ioport-port",	1,	0,	OPT_ioport_port },
 	{ "ioprio",		1,	0,	OPT_ioprio },
 	{ "ioprio-ops",		1,	0,	OPT_ioprio_ops },
 	{ "iostat",		1,	0,	OPT_iostat },
@@ -478,6 +480,7 @@ const struct option stress_long_options[] = {
 	{ "llc-affinity-mlock",	0,	0,	OPT_llc_affinity_mlock },
 	{ "llc-affinity-numa",	0,	0,	OPT_llc_affinity_numa },
 	{ "llc-affinity-ops",	1,	0,	OPT_llc_affinity_ops },
+	{ "llc-affinity-size",	1,	0,	OPT_llc_affinity_size },
 	{ "loadavg",		1,	0,	OPT_loadavg },
 	{ "loadavg-ops",	1,	0,	OPT_loadavg_ops },
 	{ "loadavg-max",	1,	0,	OPT_loadavg_max },
@@ -1051,6 +1054,7 @@ const struct option stress_long_options[] = {
 	{ "spinmem-method",	1,	0,	OPT_spinmem_method },
 	{ "spinmem-numa",	0,	0,	OPT_spinmem_numa },
 	{ "spinmem-ops",	1,	0,	OPT_spinmem_ops },
+	{ "spinmem-yield",	0,	0,	OPT_spinmem_yield },
 	{ "splice",		1,	0,	OPT_splice },
 	{ "splice-bytes",	1,	0,	OPT_splice_bytes },
 	{ "splice-ops",		1,	0,	OPT_splice_ops },
@@ -1076,6 +1080,7 @@ const struct option stress_long_options[] = {
 	{ "stream-madvise",	1,	0,	OPT_stream_madvise },
 	{ "stream-mlock",	0,	0,	OPT_stream_mlock },
 	{ "stream-ops",		1,	0,	OPT_stream_ops },
+	{ "stream-prefetch",	0,	0,	OPT_stream_prefetch },
 	{ "stressor-time",	0,	0,	OPT_stressor_time },
 	{ "stressors",		0,	0,	OPT_stressors },
 	{ "swap",		1,	0,	OPT_swap },
