@@ -88,6 +88,7 @@
 #define OPT_FLAGS_C_STATES	 STRESS_BIT_ULL(57)	/* --c-states */
 #define OPT_FLAGS_STRESSOR_TIME	 STRESS_BIT_ULL(58)	/* --stressor-time */
 #define OPT_FLAGS_TASKSET_RANDOM STRESS_BIT_ULL(59)	/* --taskset-random */
+#define OPT_FLAGS_BUILDINFO	 STRESS_BIT_ULL(60)	/* --buildinfo */
 
 #define OPT_FLAGS_MINMAX_MASK		\
 	(OPT_FLAGS_MINIMIZE | OPT_FLAGS_MAXIMIZE)
@@ -234,6 +235,8 @@ typedef enum {
 	OPT_bubblesort_ops,
 	OPT_bubblesort_size,
 
+	OPT_buildinfo,
+
 	OPT_c_states,
 
 	OPT_class,
@@ -249,11 +252,13 @@ typedef enum {
 	OPT_cache_level,
 	OPT_cache_sfence,
 	OPT_cache_no_affinity,
+	OPT_cache_permute,
 	OPT_cache_prefetch,
 	OPT_cache_prefetchw,
 	OPT_cache_ways,
 
 	OPT_cachehammer,
+	OPT_cachehammer_numa,
 	OPT_cachehammer_ops,
 
 	OPT_cacheline,
@@ -384,6 +389,9 @@ typedef enum {
 	OPT_dynlib,
 	OPT_dynlib_ops,
 
+	OPT_easy_opcode,
+	OPT_easy_opcode_ops,
+
 	OPT_eigen,
 	OPT_eigen_ops,
 	OPT_eigen_method,
@@ -435,6 +443,7 @@ typedef enum {
 	OPT_far_branch,
 	OPT_far_branch_flush,
 	OPT_far_branch_ops,
+	OPT_far_branch_pageout,
 	OPT_far_branch_pages,
 
 	OPT_fault,
@@ -443,6 +452,9 @@ typedef enum {
 	OPT_fcntl,
 	OPT_fcntl_ops,
 
+	OPT_fd_abuse,
+	OPT_fd_abuse_ops,
+
 	OPT_fd_fork,
 	OPT_fd_fork_fds,
 	OPT_fd_fork_file,
@@ -473,6 +485,9 @@ typedef enum {
 	OPT_filename_ops,
 	OPT_filename_opts,
 
+	OPT_filerace,
+	OPT_filerace_ops,
+
 	OPT_flipflop,
 	OPT_flipflop_bits,
 	OPT_flipflop_ops,
@@ -631,6 +646,7 @@ typedef enum {
 	OPT_ioport,
 	OPT_ioport_ops,
 	OPT_ioport_opts,
+	OPT_ioport_port,
 
 	OPT_ionice_class,
 	OPT_ionice_level,
@@ -721,6 +737,7 @@ typedef enum {
 	OPT_llc_affinity_mlock,
 	OPT_llc_affinity_numa,
 	OPT_llc_affinity_ops,
+	OPT_llc_affinity_size,
 
 	OPT_loadavg,
 	OPT_loadavg_ops,
@@ -890,6 +907,13 @@ typedef enum {
 	OPT_mmapaddr_mlock,
 	OPT_mmapaddr_ops,
 
+	OPT_mmapcow,
+	OPT_mmapcow_fork,
+	OPT_mmapcow_free,
+	OPT_mmapcow_mlock,
+	OPT_mmapcow_numa,
+	OPT_mmapcow_ops,
+
 	OPT_mmapfiles,
 	OPT_mmapfiles_numa,
 	OPT_mmapfiles_ops,
@@ -1063,6 +1087,10 @@ typedef enum {
 	OPT_physpage_ops,
 	OPT_physpage_mtrr,
 
+	OPT_physmmap,
+	OPT_physmmap_ops,
+	OPT_physmmap_read,
+
 	OPT_pidfd,
 	OPT_pidfd_ops,
 
@@ -1460,8 +1488,10 @@ typedef enum {
 
 	OPT_spinmem,
 	OPT_spinmem_affinity,
-	OPT_spinmem_ops,
 	OPT_spinmem_method,
+	OPT_spinmem_numa,
+	OPT_spinmem_ops,
+	OPT_spinmem_yield,
 
 	OPT_splice,
 	OPT_splice_ops,
@@ -1495,6 +1525,7 @@ typedef enum {
 	OPT_stream_madvise,
 	OPT_stream_mlock,
 	OPT_stream_ops,
+	OPT_stream_prefetch,
 
 	OPT_stressor_time,
 
@@ -1615,6 +1646,9 @@ typedef enum {
 	OPT_udp_flood_domain,
 	OPT_udp_flood_if,
 
+	OPT_umask,
+	OPT_umask_ops,
+
 	OPT_umount,
 	OPT_umount_ops,
 
@@ -1677,6 +1711,7 @@ typedef enum {
 	OPT_vforkmany_vm_bytes,
 
 	OPT_vm_bytes,
+	OPT_vm_flush,
 	OPT_vm_hang,
 	OPT_vm_keep,
 	OPT_vm_populate,
@@ -1707,6 +1742,7 @@ typedef enum {
 	OPT_vma_ops,
 
 	OPT_vmstat,
+	OPT_vmstat_units,
 
 	OPT_vnni,
 	OPT_vnni_intrinsic,
