@@ -22,10 +22,20 @@
 #define STRESS_PRAGMA_(x) _Pragma (#x)
 #define STRESS_PRAGMA(x) STRESS_PRAGMA_(x)
 
+#if defined(HAVE_PRAGMA_PREFETCH) &&		\
+    defined(HAVE_COMPILER_GCC_OR_MUSL) &&  	\
+    defined(HAVE_PRAGMA)
+#define STRESS_PRAGMA_PREFETCH		_Pragma("GCC optimize (\"prefetch-loop-arrays\")")
+#define STRESS_PRAGMA_NOPREFETCH	_Pragma("GCC optimize (\"no-prefetch-loop-arrays\")")
+#else
+#define STRESS_PRAGMA_PREFETCH
+#define STRESS_PRAGMA_NOPREFETCH
+#endif
+
 #if defined(HAVE_PRAGMA_NO_HARD_DFP) &&		\
     defined(HAVE_COMPILER_GCC_OR_MUSL) &&	\
     defined(HAVE_PRAGMA)
-#define STRESS_PRAGMA_NO_HARD_DFP	 _Pragma("GCC target (\"no-hard-dfp\")")
+#define STRESS_PRAGMA_NO_HARD_DFP	_Pragma("GCC target (\"no-hard-dfp\")")
 #endif
 
 #if defined(HAVE_COMPILER_CLANG) &&	\
