@@ -96,6 +96,10 @@ UNEXPECTED
 #include <sys/random.h>
 #endif
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
 #if defined(HAVE_SYS_XATTR_H)
 #include <sys/xattr.h>
 #undef HAVE_ATTR_XATTR_H
@@ -357,6 +361,10 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 		if (mode & FALLOC_FL_COLLAPSE_RANGE)
 			return ret;
 #endif
+#if defined(FALLOC_FL_WRITE_ZEROES)
+		if (mode & FALLOC_FL_COLLAPSE_RANGE)
+			return ret;
+#endif
 #if defined(__NR_fallocate) &&	\
     defined(HAVE_SYSCALL)
 		ret = (int)syscall(__NR_fallocate, fd, 0, offset, len);
@@ -384,6 +392,10 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 #if defined(FALLOC_FL_COLLAPSE_RANGE)
 		if (mode & FALLOC_FL_COLLAPSE_RANGE)
 			return ret;
+#endif
+#if defined(FALLOC_FL_WRITE_ZEROES)
+		if (mode & FALLOC_FL_WRITE_ZEROES)
+			return ret;
 #endif
 		ret = syscall(__NR_fallocate, fd, 0, offset, len);
 		/* fallocate failed, try emulation mode */
@@ -410,7 +422,10 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 	if (mode & FALLOC_FL_COLLAPSE_RANGE)
 		return 0;
 #endif
-
+#if defined(FALLOC_FL_WRITE_ZEROES)
+	if (mode & FALLOC_FL_WRITE_ZEROES)
+		return 0;
+#endif
 	/*
 	 *  posix_fallocate returns 0 for success, > 0 as errno
 	 */
@@ -430,6 +445,10 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
 #if defined(FALLOC_FL_COLLAPSE_RANGE)
 	if (mode & FALLOC_FL_COLLAPSE_RANGE)
 		return 0;
+#endif
+#if defined(FALLOC_FL_WRITE_ZEROES)
+	if (mode & FALLOC_FL_WRITE_ZEROES)
+		return 0;
 #endif
 	return shim_emulate_fallocate(fd, offset, len);
 #endif
@@ -1412,6 +1431,17 @@ size_t shim_strlcat(char *dst, const char *src, size_t len)
 #endif
 }
 
+/*
+ *  shim_sync
+ *	wrapper of sync
+ */
+void shim_sync(void)
+{
+#if defined(HAVE_SYNC)
+	sync();
+#endif
+}
+
 /*
  *  shim_fsync
  *	wrapper for fsync
@@ -1947,9 +1977,9 @@ int shim_setxattrat(int dfd, const char *path, unsigned int at_flags,
 	return setxattr(dfd, path, name, at_flags, name, args, size);
 #elif defined(__NR_setxattrat) &&	\
       defined(HAVE_SYSCALL)
-	return (int)syscall(__NR_setxattrat, dfd, path, name, at_flags, name, args, size);
+	return (int)syscall(__NR_setxattrat, dfd, path, at_flags, name, args, size);
 #else
-	return (int)shim_enosys(0, dfd, path, name, at_flags, name, args, size);
+	return (int)shim_enosys(0, dfd, path, at_flags, name, args, size);
 #endif
 }
 
@@ -2317,7 +2347,7 @@ time_t shim_time(time_t *tloc)
  *	avoid libc calling gettimeofday via
  *	the VDSO and force the time system call
  */
-int shim_gettimeofday(struct timeval *tv, struct timezone *tz)
+int shim_gettimeofday(struct timeval *tv, shim_timezone_t *tz)
 {
 #if defined(__NR_gettimeofday) &&	\
     defined(HAVE_SYSCALL) &&		\
@@ -2707,12 +2737,15 @@ int shim_setdomainname(const char *name, size_t len)
 int shim_setgroups(int size, const gid_t *list)
 {
 #if defined(HAVE_GRP_H)
-#if defined(__linux__)
+#if defined(__linux__) &&	\
+    defined(HAVE_SETGROUPS)
 	/* Linux passes size as size_t */
 	return setgroups((size_t)size, list);
-#else
+#elif defined(HAVE_SETGROUPS)
 	/* POSIX variant, as used by BSD etc */
 	return setgroups(size, list);
+#else
+	return (int)shim_enosys(size, list);
 #endif
 #else
 	return (int)shim_enosys(size, list);
@@ -2978,3 +3011,60 @@ int shim_ppoll(
 #endif
 }
 
+/*
+ * shim_file_getattr()
+ *	shim wrapper for Linux 6.16 file_getattr
+ */
+int shim_file_getattr(
+	int dfd,
+	const char *filename,
+	struct shim_file_attr *ufattr,
+	size_t usize,
+	unsigned int at_flags)
+{
+#if defined(__NR_file_getattr)
+	return (int)syscall(__NR_file_getattr, dfd, filename, ufattr, usize, at_flags);
+#else
+	return shim_enosys(0, dfd, filename, ufattr, usize, at_flags);
+#endif
+}
+
+/*
+ * shim_file_setattr()
+ *	shim wrapper for Linux 6.16 file_setattr
+ */
+int shim_file_setattr(
+	int dfd,
+	const char *filename,
+	struct shim_file_attr *ufattr,
+	size_t usize,
+	unsigned int at_flags)
+{
+#if defined(__NR_file_getattr)
+	return (int)syscall(__NR_file_setattr, dfd, filename, ufattr, usize, at_flags);
+#else
+	return shim_enosys(0, dfd, filename, ufattr, usize, at_flags);
+#endif
+}
+
+/*
+ * shim_pause()
+ *	shim wrapper for pause()
+ */
+int shim_pause(void)
+{
+#if defined(HAVE_PAUSE)
+	return pause();
+#elif defined(HAVE_SYS_SELECT_H) &&	\
+      defined(HAVE_SELECT)
+	while (select(0, NULL, NULL, NULL, NULL) == 0)
+		;
+	errno = EINTR;
+	return -1;
+#else
+	while (sleep(~(unsigned int)0) == 0)
+		;
+	errno = EINTR;
+	return -1;
+#endif
+}
