@@ -286,6 +286,16 @@ typedef int64_t		shim_off64_t;
 #define SHIM_POSIX_MADV_DONTNEED POSIX_MADV_DONTNEED
 #endif
 
+/* timezone shim */
+#if defined(HAVE_TIMEZONE)
+typedef struct timezone shim_timezone_t;
+#else
+typedef struct shim_timezone {
+	int tz_minuteswest;	/* minute West of GMT */
+	int tz_dsttime;		/* DST correction */
+} shim_timezone_t;
+#endif
+
 /* clone3 clone args */
 struct shim_clone_args {
 	uint64_t flags;		/* Flags bit mask */
@@ -513,6 +523,14 @@ typedef struct shim_xattr_args {
         uint32_t flags;
 } shim_xattr_args;
 
+typedef struct shim_file_attr {
+	uint64_t fa_xflags;
+	uint32_t fa_extsize;
+	uint32_t fa_nextents;
+	uint32_t fa_projid;
+	uint32_t fa_cowextsize;
+} shim_file_attr_t;
+
 /*
  *  shim_unconstify_ptr()
  *      some older system calls require non-const void *
@@ -592,6 +610,7 @@ extern int shim_brk(void *addr);
 extern void *shim_sbrk(intptr_t increment);
 extern ssize_t shim_strscpy(char *dst, const char *src, size_t len);
 extern size_t shim_strlcat(char *dst, const char *src, size_t len);
+extern void shim_sync(void);
 extern int shim_fsync(int fd);
 extern int shim_fdatasync(int fd);
 extern int shim_pkey_alloc(unsigned int flags, unsigned int access_rights);
@@ -654,7 +673,7 @@ extern int shim_clock_gettime(clockid_t clk_id, struct timespec *tp);
 extern int shim_clock_settime(clockid_t clk_id, struct timespec *tp);
 extern int shim_nice(int inc);
 extern time_t shim_time(time_t *tloc);
-extern int shim_gettimeofday(struct timeval *tv, struct timezone *tz);
+extern int shim_gettimeofday(struct timeval *tv, shim_timezone_t *tz);
 extern int shim_close_range(unsigned int fd, unsigned int max_fd,
 	unsigned int flags);
 extern int shim_lookup_dcookie(uint64_t cookie, char *buffer, size_t len);
@@ -700,5 +719,9 @@ extern unsigned char shim_dirent_type(const char *path, const struct dirent *d);
 extern int shim_mseal(void *addr, size_t len, unsigned long int flags);
 extern int shim_ppoll(shim_pollfd_t *fds, shim_nfds_t nfds,
 	const struct timespec *tmo_p, const sigset_t *sigmask);
-
+extern int shim_file_getattr(int dfd, const char *filename,
+	struct shim_file_attr *ufattr, size_t usize, unsigned int at_flags);
+extern int shim_file_setattr(int dfd, const char *filename,
+	struct shim_file_attr *ufattr, size_t usize, unsigned int at_flags);
+extern int shim_pause(void);
 #endif
