@@ -56,8 +56,10 @@ static void MLOCKED_TEXT stress_pagein_handler(int signum)
 {
 	(void)signum;
 
-	if (jmp_env_set)
+	if (jmp_env_set) {
 		siglongjmp(jmp_env, 1);
+		stress_no_return();
+	}
 }
 
 /*
@@ -108,7 +110,7 @@ int stress_pagein_self(const char *name)
 		           " %5s %*x %*x:%*x %*d %1023s", &begin, &end, prot, tmppath) != 4)
 			continue;
 
-		/* Avoid VDSO etc.. */
+		/* Avoid vdso and vvar.. */
 		if (strncmp("[v", tmppath, 2) == 0)
 			continue;
 
