@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-put.h"
 #include "core-pragma.h"
@@ -109,6 +110,7 @@ static void NORETURN MLOCKED_TEXT stress_signal_handler(int signum)
 	 *  signal handler, so jmp back and re-try
 	 */
 	siglongjmp(jmpbuf, 1);
+	stress_no_return();
 }
 
 static int stress_bad_altstack_child(stress_args_t *args)
@@ -259,7 +261,7 @@ retry:
 			}
 			if (ret == 0)
 				stress_bad_altstack_force_fault(stack);
-			stress_bad_altstack_force_fault(g_shared->nullptr);
+			stress_bad_altstack_force_fault(g_shared->null_ptr);
 			goto retry;
 		case 8:
 #if defined(HAVE_VDSO_VIA_GETAUXVAL)
@@ -293,7 +295,7 @@ retry:
 		case 0:
 			/* Illegal unmapped stack */
 			(void)munmap(stack, stress_minsigstksz);
-			stress_bad_altstack_force_fault(g_shared->nullptr);
+			stress_bad_altstack_force_fault(g_shared->null_ptr);
 			break;
 		}
 	}
@@ -329,9 +331,10 @@ static int stress_bad_altstack(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS | map_stackflags, -1, 0);
 	if (stack == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zu byte signal handler stack, "
+		pr_inf_skip("%s: cannot mmap %zu byte signal handler stack%s, "
 			    "errno=%d (%s), skipping stressor\n",
-			args->name, (size_t)stress_minsigstksz, errno, strerror(errno));
+			args->name, (size_t)stress_minsigstksz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(stack, stress_minsigstksz, "altstack");
@@ -392,18 +395,19 @@ again:
 				goto again;
 			if (UNLIKELY(!stress_continue(args)))
 				return EXIT_SUCCESS;
-			pr_err("%s: fork failed: errno=%d: (%s)\n",
+			pr_err("%s: fork failed, errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		} else if (pid > 0) {
-			int status, ret;
+			int status;
+			pid_t ret;
 
 			/* Parent, wait for child */
 			ret = waitpid(pid, &status, 0);
 			if (ret < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 				(void)stress_kill_pid_wait(pid, NULL);
 			} else if (WIFSIGNALED(status)) {
 				/* If we got killed by OOM killer, re-start */
@@ -458,14 +462,14 @@ finish:
 
 const stressor_info_t stress_bad_altstack_info = {
 	.stressor = stress_bad_altstack,
-	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_bad_altstack_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sigaltstack()"
