@@ -42,7 +42,7 @@ static const stress_help_t help[] = {
 #define STRESS_BAD_IOCTL_CMD_RANDOM_INC	(2)
 #define STRESS_BAD_IOCTL_CMD_STRIDE	(3)
 
-static const char *stress_bad_ioctl_methods[] = {
+static const char * const stress_bad_ioctl_methods[] = {
 	"inc",
 	"random",
 	"random-inc",
@@ -115,7 +115,7 @@ static dev_ioctl_info_t *stress_bad_ioctl_dev_new(
 	node = (dev_ioctl_info_t *)calloc(1, sizeof(*node));
 	if (!node)
 		return NULL;
-	node->dev_path = strdup(dev_path);
+	node->dev_path = shim_strdup(dev_path);
 	if (!node->dev_path) {
 		free(node);
 		return NULL;
@@ -226,6 +226,7 @@ static void NORETURN MLOCKED_TEXT stress_segv_handler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 /*
@@ -574,14 +575,15 @@ again:
 			if (stress_redo_fork(args, errno))
 				goto again;
 		} else if (pid > 0) {
-			int status, wret;
+			int status;
+			pid_t wret;
 
 			/* Parent, wait for child */
 			wret = shim_waitpid(pid, &status, 0);
 			if (wret < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 				stress_force_killed_bogo(args);
 				(void)stress_kill_pid_wait(pid, NULL);
 			} else {
@@ -657,14 +659,14 @@ again:
 }
 const stressor_info_t stress_bad_ioctl_info = {
 	.stressor = stress_bad_ioctl,
-	.class = CLASS_DEV | CLASS_OS | CLASS_PATHOLOGICAL,
+	.classifier = CLASS_DEV | CLASS_OS | CLASS_PATHOLOGICAL,
 	.opts = opts,
 	.help = help
 };
 #else
 const stressor_info_t stress_bad_ioctl_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_DEV | CLASS_OS | CLASS_PATHOLOGICAL,
+	.classifier = CLASS_DEV | CLASS_OS | CLASS_PATHOLOGICAL,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without pthread and/or ioctl() _IOR macro or is not Linux"
