@@ -54,6 +54,15 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
+static const stress_opt_t opts[] = {
+	{ OPT_bigheap_bytes,  "bigheap-bytes",  TYPE_ID_SIZE_T_BYTES_VM, MIN_BIGHEAP_BYTES,  MAX_BIGHEAP_BYTES,  NULL },
+	{ OPT_bigheap_growth, "bigheap-growth", TYPE_ID_UINT64,          MIN_BIGHEAP_GROWTH, MAX_BIGHEAP_GROWTH, NULL },
+	{ OPT_bigheap_mlock,  "bigheap-mlock",  TYPE_ID_BOOL,            0,                  1,                  NULL },
+	END_OPT,
+};
+
+#if defined(HAVE_SIGLONGJMP)
+
 static sigjmp_buf jmp_env;
 static volatile int phase;
 static volatile void *fault_addr;
@@ -103,6 +112,7 @@ static void NORETURN MLOCKED_TEXT stress_bigheap_segvhandler(
 	sigcode = info->si_code;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #else
 static void NORETURN MLOCKED_TEXT stress_bigheap_segvhandler(int signum)
@@ -110,6 +120,7 @@ static void NORETURN MLOCKED_TEXT stress_bigheap_segvhandler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #endif
 
@@ -120,19 +131,20 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 	NOCLOBBER void *ptr = NULL;
 	NOCLOBBER const void *last_ptr = NULL;
 	NOCLOBBER uint8_t *last_ptr_end = NULL;
-	NOCLOBBER size_t size = 0;
+	NOCLOBBER size_t size = 0, stride;
 	NOCLOBBER double duration = 0.0, count = 0.0;
 	NOCLOBBER bool segv_reported = false;
 	const size_t page_size = args->page_size;
-	const size_t stride = page_size;
 	double rate;
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	const bool oom_avoid = !!(g_opt_flags & OPT_FLAGS_OOM_AVOID);
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 	bool bigheap_mlock = false;
 	struct sigaction action;
 	int ret;
 	NOCLOBBER int rc = EXIT_SUCCESS;
 
+	stride = (g_opt_flags & OPT_FLAGS_AGGRESSIVE) ? sizeof(uintptr_t) : page_size;
 	fault_addr = NULL;
 	signo = -1;
 	sigcode = -1;
@@ -141,7 +153,12 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 	(void)context;
 
 	(void)stress_get_setting("bigheap-mlock", &bigheap_mlock);
-	(void)stress_get_setting("bigheap-bytes", &bigheap_bytes);
+	if (!stress_get_setting("bigheap-bytes", &bigheap_bytes)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			bigheap_bytes = MAX_BIGHEAP_BYTES;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			bigheap_bytes = MIN_BIGHEAP_BYTES;
+	}
 	if (!stress_get_setting("bigheap-growth", &bigheap_growth)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			bigheap_growth = MAX_BIGHEAP_GROWTH;
@@ -226,9 +243,22 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 		if (old_ptr) {
 			phase = STRESS_BIGHEAP_REALLOC;
 			ptr = realloc(old_ptr, size);
+			if (g_opt_flags & OPT_FLAGS_AGGRESSIVE) {
+				if (LIKELY(ptr != NULL)) {
+					old_ptr = ptr;
+					size += 64;
+					ptr = realloc(old_ptr, size);
+					if (LIKELY(ptr != NULL))
+						stress_bogo_inc(args);
+				}
+			}
 		} else {
 			phase = STRESS_BIGHEAP_MALLOC;
-			ptr = malloc(size);
+			if (UNLIKELY(aggressive)) {
+				ptr = calloc(1, size);
+			} else {
+				ptr = malloc(size);
+			}
 		}
 		if (UNLIKELY(ptr == NULL)) {
 			phase = STRESS_BIGHEAP_OUT_OF_MEMORY;
@@ -310,17 +340,23 @@ static int stress_bigheap(stress_args_t *args)
 	return stress_oomable_child(args, NULL, stress_bigheap_child, STRESS_OOMABLE_NORMAL);
 }
 
-static const stress_opt_t opts[] = {
-	{ OPT_bigheap_bytes,  "bigheap-bytes",  TYPE_ID_SIZE_T_BYTES_VM, MIN_BIGHEAP_BYTES,  MAX_BIGHEAP_BYTES,  NULL },
-	{ OPT_bigheap_growth, "bigheap-growth", TYPE_ID_UINT64,          MIN_BIGHEAP_GROWTH, MAX_BIGHEAP_GROWTH, NULL },
-	{ OPT_bigheap_mlock,  "bigheap-mlock",  TYPE_ID_BOOL,            0,                  1,                  NULL },
-	END_OPT,
-};
-
 const stressor_info_t stress_bigheap_info = {
 	.stressor = stress_bigheap,
-	.class = CLASS_OS | CLASS_VM,
+	.classifier = CLASS_OS | CLASS_VM,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_bigheap_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_OS | CLASS_VM,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support",
+};
+
+#endif
