@@ -181,13 +181,34 @@ static int OPTIMIZE3 TARGET_CLONES stress_bitops_countbits(const char *name, uin
 			return EXIT_FAILURE;
 		}
 #endif
+
+		/* #6 Count bits triple mask method */
+		{
+			const uint32_t ones = ~(uint32_t)0;
+			const uint32_t mask1 = (ones / 3) << 1;
+			const uint32_t mask2 = ones / 5;
+			const uint32_t mask4 = ones / 17;
+
+			c2 = v - ((mask1 & v) >> 1);
+			c2 = (c2 & mask2) + ((c2 >> 2) & mask2);
+			c2 = (c2 + (c2 >> 4)) & mask4;
+			c2 += c2 >> 8;
+			c2 += c2 >> 16;
+			c2 &=  0xff;
+
+			if (UNLIKELY(c1 != c2)) {
+				pr_fail("%s: countbits triple mask failure, value 0x%" PRIx32 ", c1 = 0x%" PRIx32 ", c2 = 0x%" PRIx32 "\n",
+					name, v, c1, c2);
+				return EXIT_FAILURE;
+			}
+		}
 		v += dv;
 	}
 	stress_uint32_put(sum);
 #if defined(HAVE_BUILTIN_POPCOUNT)
-	*count += 5 * i;
+	*count += 6 * i;
 #else
-	*count += 4 * i;
+	*count += 5 * i;
 #endif
 	return EXIT_SUCCESS;
 }
@@ -1111,7 +1132,7 @@ static int stress_bitops(stress_args_t *args)
 
 	(void)stress_get_setting("bitops-method", &bitops_method);
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: using method '%s'\n", args->name, bitops_methods[bitops_method].name);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -1128,10 +1149,11 @@ static int stress_bitops(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 0, j = 0; i < SIZEOF_ARRAY(metrics); i++) {
-		char buf[32];
 		const double rate = metrics[i].duration > 0.0 ? metrics[i].count / metrics[i].duration : 0.0;
 
 		if (rate > 0.0) {
+			char buf[32];
+
 			(void)snprintf(buf, sizeof(buf), "%s mega-ops per second", bitops_methods[i].name);
 			stress_metrics_set(args, j, buf, rate / 1000000.0, STRESS_METRIC_GEOMETRIC_MEAN);
 			j++;
@@ -1147,7 +1169,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_bitops_info = {
 	.stressor = stress_bitops,
-	.class = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
