@@ -22,6 +22,7 @@
 #include "core-asm-riscv.h"
 #include "core-asm-ppc64.h"
 #include "core-affinity.h"
+#include "core-attribute.h"
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-numa.h"
@@ -47,8 +48,6 @@ typedef struct {
 	hammer_func_t hammer;
 } stress_cachehammer_func_t;
 
-static sigjmp_buf jmp_env;
-
 static const stress_help_t help[] = {
 	{ NULL,	"cachehammer N",	"start N CPU cache thrashing workers" },
 	{ NULL,	"cachehammer-numa",	"move pages to randomly chosen NUMA nodes" },
@@ -56,6 +55,15 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
+static const stress_opt_t opts[] = {
+	{ OPT_cachehammer_numa, "cachehammer-numa",  TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
+};
+
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+
 #if defined(HAVE_MSYNC)
 static int msync_flags[] = {
 #if defined(MS_SYNC)
@@ -76,7 +84,7 @@ static int msync_flags[] = {
 static char cachehammer_filename[PATH_MAX];
 static char cachehammer_path[PATH_MAX];
 
-static bool hammer_valid(void)
+static bool PURE hammer_valid(void)
 {
 	return true;
 }
@@ -1577,6 +1585,7 @@ static void NORETURN MLOCKED_TEXT stress_cache_sighandler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);         /* Ugly, bounce back */
+	stress_no_return();
 }
 
 /*
@@ -1869,7 +1878,7 @@ static int OPTIMIZE3 stress_cachehammer(stress_args_t *args)
 				addr2 = bad_page + (offset & page_mask);
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-				stress_cachehammer_numa(args, 50, &numa_count[3], addr1,
+				stress_cachehammer_numa(args, 50, &numa_count[4], addr1,
 							cachehammer_numa, numa_mask, numa_nodes);
 #endif
 				hammer(args, addr1, addr2, true, false);
@@ -1918,11 +1927,6 @@ unmap_local_buffer:
 	return ret;
 }
 
-static const stress_opt_t opts[] = {
-	{ OPT_cachehammer_numa, "cachehammer-numa",  TYPE_ID_BOOL, 0, 1, NULL },
-	END_OPT,
-};
-
 const stressor_info_t stress_cachehammer_info = {
 	.stressor = stress_cachehammer,
 	.init = stress_cachehammer_init,
@@ -1932,3 +1936,16 @@ const stressor_info_t stress_cachehammer_info = {
 	.opts = opts,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_cachehammer_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_CPU_CACHE,
+	.verify = VERIFY_ALWAYS,
+	.opts = opts,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
