@@ -57,7 +57,7 @@ typedef struct {
 	const char *value;
 } stress_cgroup_values_t;
 
-static void stress_cgroup_remove_nl(char *str)
+static inline void stress_cgroup_remove_nl(char *str)
 {
 	char *ptr;
 
@@ -178,7 +178,7 @@ static void stress_cgroup_umount(
 			return;
 		default:
 			/* Unexpected, so report it */
-			pr_inf("%s: umount failed %s: %d %s\n", args->name,
+			pr_inf("%s: umount failed %s, errno=%d %s\n", args->name,
 				path, errno, strerror(errno));
 			break;
 		}
@@ -529,17 +529,18 @@ again:
 				goto again;
 			if (UNLIKELY(!stress_continue(args)))
 				goto finish;
-			pr_err("%s: fork failed: errno=%d (%s)\n",
+			pr_err("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else if (pid > 0) {
-			int status, waitret;
+			int status;
+			pid_t waitret;
 
 			/* Parent, wait for child */
 			waitret = shim_waitpid(pid, &status, 0);
 			if (waitret < 0) {
 				if (errno != EINTR) {
-					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 					(void)stress_kill_pid(pid);
 				}
 				(void)shim_waitpid(pid, &status, 0);
@@ -576,7 +577,7 @@ finish:
 
 const stressor_info_t stress_cgroup_info = {
 	.stressor = stress_cgroup_mount,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.supported = stress_cgroup_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -584,7 +585,7 @@ const stressor_info_t stress_cgroup_info = {
 #else
 const stressor_info_t stress_cgroup_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.supported = stress_cgroup_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
