@@ -82,8 +82,10 @@ static void MLOCKED_TEXT stress_chattr_fault_handler(int signum)
 {
 	(void)signum;
 
-	if (do_jmp)
+	if (do_jmp) {
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
+	}
 }
 
 static char *stress_chattr_flags_str(const unsigned long int flags, char *str, const size_t str_len)
@@ -143,7 +145,7 @@ static int do_chattr(
 			    (errno != ENOTTY) &&
 			    (errno != EINVAL)) {
 				/* unexpected failure */
-				pr_fail("%s: ioctl SHIM_EXT2_IOC_GETFLAGS failed: errno=%d (%s)%s\n",
+				pr_fail("%s: ioctl SHIM_EXT2_IOC_GETFLAGS failed, errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno),
 					stress_get_fs_type(filename));
 				rc = EXIT_FAILURE;
@@ -172,7 +174,7 @@ static int do_chattr(
 					    (errno != EPERM) &&
 					    (errno != EINVAL)) {
 						/* unexpected failure */
-						pr_fail("%s: ioctl SHIM_EXT2_IOC_SETFLAGS (chattr zero flags) failed: errno=%d (%s)%s\n",
+						pr_fail("%s: ioctl SHIM_EXT2_IOC_SETFLAGS (chattr zero flags) failed, errno=%d (%s)%s\n",
 							args->name,
 							errno, strerror(errno),
 							stress_get_fs_type(filename));
@@ -207,7 +209,7 @@ static int do_chattr(
 				char flags_str[65];
 
 				stress_chattr_flags_str(flags, flags_str, sizeof(flags_str));
-				pr_fail("%s: ioctl SHIM_EXT2_IOC_SETFLAGS 0x%lx (chattr '%s') failed: errno=%d (%s)%s\n",
+				pr_fail("%s: ioctl SHIM_EXT2_IOC_SETFLAGS 0x%lx (chattr '%s') failed, errno=%d (%s)%s\n",
 					args->name, flags, flags_str,
 					errno, strerror(errno),
 					stress_get_fs_type(filename));
@@ -227,7 +229,7 @@ static int do_chattr(
 			    (errno != ENOTTY) &&
 			    (errno != EINVAL)) {
 				/* unexpected failure */
-				pr_fail("%s: ioctl SHIM_EXT2_IOC_GETFLAGS failed: errno=%d (%s)%s\n",
+				pr_fail("%s: ioctl SHIM_EXT2_IOC_GETFLAGS failed, errno=%d (%s)%s\n",
 					args->name,
 					errno, strerror(errno),
 					stress_get_fs_type(filename));
@@ -390,7 +392,7 @@ static int stress_chattr(stress_args_t *args)
 		}
 
 		if (fail == i) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: chattr not supported on filesystem, skipping stressor\n",
 					args->name);
 			rc = EXIT_NOT_IMPLEMENTED;
@@ -424,7 +426,7 @@ static int stress_chattr(stress_args_t *args)
 
 const stressor_info_t stress_chattr_info = {
 	.stressor = stress_chattr,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -433,7 +435,7 @@ const stressor_info_t stress_chattr_info = {
 
 const stressor_info_t stress_chattr_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without Linux chattr() support"
