@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -140,11 +140,10 @@ static const int close_range_flags[] = {
 
 /*
  *  stress_close_func()
- *	pthread that exits immediately
+ *	pthread that close file descriptors
  */
 static void *stress_close_func(void *arg)
 {
-	static void *nowt = NULL;
 	stress_pthread_args_t *pargs = (stress_pthread_args_t *)arg;
 	stress_args_t *args = pargs->args;
 
@@ -157,6 +156,8 @@ static void *stress_close_func(void *arg)
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 #endif
 
+	stress_random_small_sleep();
+
 	while (stress_continue(args)) {
 		const uint64_t delay =
 			max_delay_us ? stress_mwc64modn(max_delay_us) : 0;
@@ -167,7 +168,7 @@ static void *stress_close_func(void *arg)
 			fds[i] = dup2(fileno(stderr), i + FDS_START);
 		}
 
-		shim_usleep_interruptible(delay);
+		(void)shim_usleep_interruptible(delay);
 		if (fd != -1)
 			(void)close(fd);
 		if (dupfd != -1)
@@ -206,12 +207,12 @@ static void *stress_close_func(void *arg)
 		VOID_RET(int, shim_close_range(FDS_START + FDS_TO_DUP, FDS_START, ~0U));
 	}
 
-	return &nowt;
+	return &g_nowt;
 }
 
 /*
  *  stress_close()
- *	stress by creating pthreads
+ *	stress by closing file descriptors
  */
 static int stress_close(stress_args_t *args)
 {
@@ -365,7 +366,9 @@ static int stress_close(stress_args_t *args)
 			fd = open("/tmp/", O_RDONLY | O_DIRECTORY | O_CLOEXEC);
 			break;
 #endif
-#if defined(HAVE_LIB_RT)
+#if defined(HAVE_LIB_RT) &&	\
+    defined(HAVE_SHM_OPEN) &&	\
+    defined(HAVE_SHM_UNLINK)
 		case 12:
 			fd = shm_open(shm_name, O_CREAT | O_RDWR | O_TRUNC,
 				S_IRUSR | S_IWUSR);
@@ -437,7 +440,7 @@ static int stress_close(stress_args_t *args)
 			VOID_RET(int, shim_fstat(fd, &statbuf));
 
 			ret = dup(STDOUT_FILENO);
-			if (ret != -1)
+			if (LIKELY(ret != -1))
 				(void)close(ret);
 
 			t = stress_time_now();
@@ -448,13 +451,13 @@ static int stress_close(stress_args_t *args)
 				/*
 				 *  A close on a successfully closed fd should fail
 				 */
-				if (close(fd) == 0) {
+				if (UNLIKELY(close(fd) == 0)) {
 					pr_fail("%s: unexpectedly able to close the same file %d twice\n",
 						args->name, fd);
 					close_failure = true;
 				} else {
-					if ((errno != EBADF) &&
-					    (errno != EINTR)) {
+					if (UNLIKELY((errno != EBADF) &&
+						     (errno != EINTR))) {
 						pr_fail("%s: expected error on close failure, error=%d (%s)\n",
 							args->name,  errno, strerror(errno));
 						close_failure = true;
@@ -509,14 +512,14 @@ tidy:
 
 const stressor_info_t stress_close_info = {
 	.stressor = stress_close,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_close_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
