@@ -23,6 +23,7 @@
 #include "core-bitops.h"
 #include "core-builtin.h"
 #include "core-cpu.h"
+#include "core-net.h"
 #include "core-pragma.h"
 #include "core-put.h"
 #include "core-target-clones.h"
@@ -1572,7 +1573,7 @@ static inline long double complex PURE OPTIMIZE3 OPTIMIZE_FAST_MATH zeta(
  * stress_cpu_zeta()
  *	stress test Zeta(2.0)..Zeta(10.0)
  */
-static int stress_cpu_zeta(const char *name)
+static int OPTIMIZE3 OPTIMIZE_FAST_MATH stress_cpu_zeta(const char *name)
 {
 	long double precision = 0.00000001L;
 	int i;
@@ -3070,7 +3071,7 @@ static int OPTIMIZE3 stress_cpu(stress_args_t *args)
 			stress_cpu_counter_scale[i] = 1484.50 / stress_cpu_methods[i].bogo_op_rate;
 	}
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: using method '%s'\n", args->name, stress_cpu_methods[cpu_method].name);
 
 	/*
@@ -3088,13 +3089,16 @@ static int OPTIMIZE3 stress_cpu(stress_args_t *args)
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+
 	/*
 	 * Normal use case, 100% load, simple spinning on CPU
 	 */
 	if (cpu_load == 100) {
+		stress_cpu_disable_fp_subnormals();
 		do {
 			rc = stress_call_cpu_method(cpu_method, args, &counter);
 		} while ((rc == EXIT_SUCCESS) && stress_continue(args));
+		stress_cpu_enable_fp_subnormals();
 
 		stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 		return rc;
@@ -3105,6 +3109,7 @@ static int OPTIMIZE3 stress_cpu(stress_args_t *args)
 	 * not intended to be 100% accurate timing, it is good
 	 * enough for most purposes.
 	 */
+	stress_cpu_disable_fp_subnormals();
 	bias = 0.0;
 	do {
 		double delay_cpu_clock, t1_cpu_clock, t2_cpu_clock;
@@ -3183,8 +3188,9 @@ static int OPTIMIZE3 stress_cpu(stress_args_t *args)
 			bias = (t3_wall_clock - t2_wall_clock) - delay;
 		}
 	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
+	stress_cpu_enable_fp_subnormals();
 
-	if (stress_is_affinity_set() && (args->instance == 0)) {
+	if (stress_is_affinity_set() && (stress_instance_zero(args))) {
 		pr_inf("%s: CPU affinity probably set, this can affect CPU loading\n",
 			args->name);
 	}
@@ -3209,7 +3215,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_cpu_info = {
 	.stressor = stress_cpu,
-	.class = CLASS_CPU | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
