@@ -19,8 +19,12 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
+#include "core-prime.h"
 
+#if defined(HAVE_SYS_FILE_H)
 #include <sys/file.h>
+#endif
 
 #if defined(HAVE_SYS_SELECT_H)
 #include <sys/select.h>
@@ -353,7 +357,7 @@ static int stress_dentry(stress_args_t *args)
 			int fd;
 			double t;
 
-			if (!stress_continue(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto abort;
 
 			stress_temp_filename_args(args,
@@ -387,7 +391,7 @@ static int stress_dentry(stress_args_t *args)
 		}
 
 		stress_dentry_misc(dir_path);
-		sync();
+		shim_sync();
 
 		/*
 		 *  Now look up some bogus names to exercise
@@ -397,7 +401,7 @@ static int stress_dentry(stress_args_t *args)
 			const uint64_t gray_code = (i >> 1) ^ i;
 			double t;
 
-			if (!stress_continue(args))
+			if (UNLIKELY(!stress_continue(args)))
 				goto abort;
 
 			/* The following should succeed */
@@ -444,14 +448,14 @@ static int stress_dentry(stress_args_t *args)
 
 		stress_dentry_misc(dir_path);
 
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 abort:
 	stress_dentry_state(&nr_dentry2);
 	nr_dentries = nr_dentry2 - nr_dentry1;
-	if ((args->instance == 0) && (nr_dentries > 0)) {
+	if (stress_instance_zero(args) && (nr_dentries > 0)) {
 		pr_inf("%s: %" PRId64 " dentries allocated\n",
 			args->name, nr_dentries);
 	}
@@ -485,7 +489,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_dentry_info = {
 	.stressor = stress_dentry,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
