@@ -95,7 +95,7 @@ static inline int stress_dev_shm_child(
 				const uint32_t rnd = stress_mwc32();
 
 				stress_set_vma_anon_name(addr, (size_t)sz, "mmapped-dev-shm");
-				(void)stress_madvise_random(addr, (size_t)sz);
+				(void)stress_madvise_randomize(addr, (size_t)sz);
 				(void)stress_madvise_mergeable(addr, (size_t)sz);
 
 				/* Touch all pages with data */
@@ -147,20 +147,21 @@ again:
 				goto again;
 			if (!stress_continue(args))
 				goto finish;
-			pr_err("%s: fork failed: errno=%d: (%s)\n",
+			pr_err("%s: fork failed, errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			/* Nope, give up! */
 			(void)close(context->fd);
 			return EXIT_FAILURE;
 		} else if (pid > 0) {
 			/* Parent */
-			int ret, status = 0;
+			pid_t ret;
+			int status = 0;
 
 			ret = shim_waitpid(pid, &status, 0);
 			if (ret < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 				stress_force_killed_bogo(args);
 				(void)stress_kill_pid_wait(pid, &status);
 			}
@@ -206,12 +207,12 @@ static int stress_dev_shm(stress_args_t *args)
 	 */
 	if (access("/dev/shm", R_OK | W_OK) < 0) {
 		if (errno == ENOENT) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: /dev/shm does not exist, skipping test\n",
 					args->name);
 			return EXIT_NO_RESOURCE;
 		} else {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: cannot access /dev/shm, errno=%d (%s), skipping test\n",
 					args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
@@ -242,14 +243,14 @@ static int stress_dev_shm(stress_args_t *args)
 
 const stressor_info_t stress_dev_shm_info = {
 	.stressor = stress_dev_shm,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_dev_shm_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
