@@ -165,7 +165,7 @@ static void stress_efi_sysfs_fd(
 		ptr = mmap(NULL, len, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, fd, 0);
 		if (ptr != MAP_FAILED) {
-			stress_madvise_random(ptr, len);
+			stress_madvise_randomize(ptr, len);
 			stress_madvise_mergeable(ptr, len);
 			(void)munmap(ptr, len);
 		}
@@ -492,9 +492,10 @@ static int stress_efivar(stress_args_t *args)
 	efi_ignore = (bool *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (efi_ignore == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zd bytes of shared memory: "
+		pr_inf_skip("%s: cannot mmap %zu bytes of shared memory%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, sz, errno, strerror(errno));
+			args->name, sz, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(efi_ignore, sz, "efi-ignore-state");
@@ -509,17 +510,18 @@ again:
 			goto again;
 		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
-		pr_err("%s: fork failed: errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	} else if (pid > 0) {
-		int status, ret;
+		int status;
+		pid_t ret;
 
 		/* Parent, wait for child */
 		ret = shim_waitpid(pid, &status, 0);
 		if (ret < 0) {
 			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+					args->name, (intmax_t)pid, errno, strerror(errno));
 			stress_force_killed_bogo(args);
 			(void)stress_kill_pid_wait(pid, NULL);
 		} else if (WIFSIGNALED(status)) {
@@ -565,7 +567,7 @@ finish:
 const stressor_info_t stress_efivar_info = {
 	.stressor = stress_efivar,
 	.supported = stress_efivar_supported,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -580,7 +582,7 @@ static int stress_efivar_supported(const char *name)
 const stressor_info_t stress_efivar_info = {
 	.stressor = stress_unimplemented,
 	.supported = stress_efivar_supported,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux with EFI variable filesystem"
