@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-capabilities.h"
 #include "core-cpu.h"
 #include "core-killpid.h"
 #include "core-out-of-memory.h"
@@ -3900,6 +3901,7 @@ static void NORETURN MLOCKED_TEXT stress_sig_handler(int signum)
 		_exit(1);
 	}
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 /*
