@@ -593,7 +593,8 @@ static int stress_fanotify(stress_args_t *args)
 		ret = posix_memalign(&buffer, BUFFER_SIZE, BUFFER_SIZE);
 		if ((ret != 0) || (buffer == NULL)) {
 			pr_err("%s: posix_memalign: cannot allocate 4K "
-				"aligned buffer\n", args->name);
+				"aligned buffer%s\n", args->name,
+				stress_get_memfree_str());
 			rc = EXIT_NO_RESOURCE;
 			goto tidy;
 		}
@@ -747,7 +748,7 @@ tidy:
 		(void)stress_kill_pid_wait(pid, NULL);
 	(void)shim_unlink(filename);
 	(void)shim_unlink(filename2);
-	(void)sync();
+	shim_sync();
 	(void)stress_temp_dir_rm_args(args);
 	stress_mount_free(mnts, n_mnts);
 
@@ -757,14 +758,14 @@ tidy:
 const stressor_info_t stress_fanotify_info = {
 	.stressor = stress_fanotify,
 	.supported = stress_fanotify_supported,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_fanotify_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/fanotify.h"
