@@ -20,16 +20,18 @@
 #include "stress-ng.h"
 #include "core-put.h"
 
-static sigjmp_buf jmp_env;
-static volatile bool do_jmp = true;
-static volatile int die_signum = -1;
-
 static const stress_help_t help[] = {
 	{ NULL,	"fault N",	"start N workers producing page faults" },
 	{ NULL,	"fault-ops N",	"stop after N page fault bogo operations" },
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+static volatile bool do_jmp = true;
+static volatile int die_signum = -1;
+
 /*
  *  stress_segvhandler()
  *	SEGV handler
@@ -38,8 +40,10 @@ static void MLOCKED_TEXT stress_segvhandler(int signum)
 {
 	die_signum = signum;
 
-	if (do_jmp)
+	if (do_jmp) {
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
+	}
 }
 
 /*
@@ -295,3 +299,14 @@ const stressor_info_t stress_fault_info = {
 	.classifier = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_fault_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
