@@ -142,7 +142,8 @@ static void stress_fiemap_ioctl(
 
 		fiemap = (struct fiemap *)calloc(1, sizeof(*fiemap));
 		if (UNLIKELY(!fiemap)) {
-			pr_err("Out of memory allocating fiemap\n");
+			pr_err("%s: out of memory allocating fiemap%s\n",
+				args->name, stress_get_memfree_str());
 			break;
 		}
 		fiemap->fm_length = ~0UL;
@@ -150,7 +151,7 @@ static void stress_fiemap_ioctl(
 		/* Find out how many extents there are */
 		if (UNLIKELY(ioctl(fd, FS_IOC_FIEMAP, fiemap) < 0)) {
 			if (errno == EOPNOTSUPP) {
-				if (args->instance == 0)
+				if (stress_instance_zero(args))
 					pr_inf_skip("%s: ioctl FS_IOC_FIEMAP not supported on the file system, skipping stressor\n",
 						args->name);
 				free(fiemap);
@@ -162,7 +163,7 @@ static void stress_fiemap_ioctl(
 			break;
 		}
 		(void)shim_sched_yield();
-		if (!stress_continue(args)) {
+		if (UNLIKELY(!stress_continue(args))) {
 			free(fiemap);
 			break;
 		}
@@ -175,8 +176,9 @@ static void stress_fiemap_ioctl(
 		tmp = (struct fiemap *)realloc(fiemap,
 			sizeof(*fiemap) + extents_size);
 		if (UNLIKELY(!tmp)) {
-			pr_fail("%s: ioctl FS_IOC_FIEMAP failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: realloc failed%s, errno=%d (%s)\n",
+				args->name, stress_get_memfree_str(),
+				errno, strerror(errno));
 			free(fiemap);
 			break;
 		}
@@ -244,7 +246,7 @@ static int stress_fiemap(stress_args_t *args)
 	int ret, fd, rc = EXIT_FAILURE;
 	char filename[PATH_MAX];
 	size_t n;
-	uint64_t fiemap_bytes = DEFAULT_FIEMAP_SIZE;
+	uint64_t fiemap_bytes, fiemap_bytes_total = DEFAULT_FIEMAP_SIZE;
 	struct fiemap fiemap;
 	const char *fs_type;
 #if defined(O_SYNC)
@@ -259,22 +261,39 @@ static int stress_fiemap(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	s_pids = stress_s_pids_mmap(MAX_FIEMAP_PROCS);
+	s_pids = stress_sync_s_pids_mmap(MAX_FIEMAP_PROCS);
 	if (s_pids == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, MAX_FIEMAP_PROCS);
 		stress_lock_destroy(counter_lock);
 		return EXIT_NO_RESOURCE;
 	}
 
-	if (!stress_get_setting("fiemap-bytes", &fiemap_bytes)) {
+	if (!stress_get_setting("fiemap-bytes", &fiemap_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			fiemap_bytes = MAXIMIZED_FILE_SIZE;
+			fiemap_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			fiemap_bytes = MIN_FIEMAP_SIZE;
+			fiemap_bytes_total = MIN_FIEMAP_SIZE;
 	}
-	fiemap_bytes /= args->num_instances;
-	if (fiemap_bytes < MIN_FIEMAP_SIZE)
+	if (fiemap_bytes_total < MIN_FIEMAP_SIZE) {
+		fiemap_bytes_total = MIN_FIEMAP_SIZE;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --fiemap-bytes too small, using %" PRIu64 " instead\n",
+				args->name, fiemap_bytes_total);
+	}
+	if (fiemap_bytes_total > MAX_FIEMAP_SIZE) {
+		fiemap_bytes_total = MAX_FIEMAP_SIZE;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --fiemap-bytes too large, using %" PRIu64 " instead\n",
+				args->name, fiemap_bytes_total);
+	}
+
+	fiemap_bytes = fiemap_bytes_total / args->instances;
+	if (fiemap_bytes < MIN_FIEMAP_SIZE) {
 		fiemap_bytes = MIN_FIEMAP_SIZE;
+		fiemap_bytes_total = fiemap_bytes * args->instance;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, fiemap_bytes, fiemap_bytes_total);
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
@@ -298,19 +317,18 @@ static int stress_fiemap(stress_args_t *args)
 	if (ioctl(fd, FS_IOC_FIEMAP, &fiemap) < 0) {
 		errno = EOPNOTSUPP;
 		if (errno == EOPNOTSUPP) {
-			if (args->instance == 0)
-				pr_inf_skip("%s: FS_IOC_FIEMAP not supported on the file system, skipping stressor%s\n",
+			if (stress_instance_zero(args))
+				pr_inf_skip("%s: ioctl FS_IOC_FIEMAP not supported on the file system, skipping stressor%s\n",
 					args->name, fs_type);
 			rc = EXIT_NOT_IMPLEMENTED;
 			goto close_clean;
 		}
 	}
 
-
 	for (n = 0; n < MAX_FIEMAP_PROCS; n++) {
 		stress_sync_start_init(&s_pids[n]);
 
-		if (!stress_continue(args)) {
+		if (UNLIKELY(!stress_continue(args))) {
 			rc = EXIT_SUCCESS;
 			goto reap;
 		}
@@ -339,7 +357,7 @@ dir_clean:
 clean:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)stress_s_pids_munmap(s_pids, MAX_FIEMAP_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, MAX_FIEMAP_PROCS);
 	stress_lock_destroy(counter_lock);
 
 	return rc;
@@ -347,7 +365,7 @@ clean:
 
 const stressor_info_t stress_fiemap_info = {
 	.stressor = stress_fiemap,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -355,7 +373,7 @@ const stressor_info_t stress_fiemap_info = {
 #else
 const stressor_info_t stress_fiemap_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
