@@ -447,8 +447,8 @@ static int stress_file_ioctl(stress_args_t *args)
 #if defined(FICLONERANGE)
 		{
 			struct file_clone_range fcr;
-			const off_t sz = 4096 * (stress_mwc8() & 0x3);
-			const off_t offset = (stress_mwc8() * 4096) & (file_sz - 1);
+			const off_t sz = (off_t)4096 * (stress_mwc8() & 0x3);
+			const off_t offset = ((off_t)4096 * stress_mwc8()) & (file_sz - 1);
 
 			(void)shim_memset(&fcr, 0, sizeof(fcr));
 			fcr.src_fd = fd;
@@ -582,7 +582,7 @@ static int stress_file_ioctl(stress_args_t *args)
 			r.l_len = (int64_t)file_sz * 2;
 			VOID_RET(int, ioctl(fd, FS_IOC_RESVSP, &r));
 
-			if (lseek(fd, (off_t)0, SEEK_SET) != (off_t)-1) {
+			if (lseek(fd, (off_t)0, SEEK_SET) >= 0) {
 				(void)shim_memset(&r, 0, sizeof(r));
 				r.l_whence = SEEK_CUR;
 				r.l_start = (int64_t)0;
@@ -612,7 +612,7 @@ static int stress_file_ioctl(stress_args_t *args)
 
 			VOID_RET(int, ioctl(fd, FS_IOC_RESVSP64, &r));
 
-			if (lseek(fd, (off_t)0, SEEK_SET) != (off_t)-1) {
+			if (lseek(fd, (off_t)0, SEEK_SET) >= 0) {
 				(void)shim_memset(&r, 0, sizeof(r));
 				r.l_whence = SEEK_CUR;
 				r.l_start = (int64_t)0;
@@ -722,7 +722,7 @@ static int stress_file_ioctl(stress_args_t *args)
 #endif
 
 		if (!exercised) {	/* cppcheck-suppress knownConditionTrueFalse */
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf("%s: no available file ioctls to exercise\n",
 					args->name);
 			rc = EXIT_NOT_IMPLEMENTED;
@@ -748,7 +748,7 @@ tidy:
 
 const stressor_info_t stress_file_ioctl_info = {
 	.stressor = stress_file_ioctl,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
