@@ -31,7 +31,7 @@
 #define STRESS_FILENAME_EXT	(2)	/* EXT* filesystems */
 
 /* mapping of opts text to STRESS_FILE_NAME_* values */
-static const char *filename_opts[] = {
+static const char * const filename_opts[] = {
 	"probe",	/* STRESS_FILENAME_PROBE */
 	"posix",	/* STRESS_FILENAME_POSIX */
 	"ext",		/* STRESS_FILENAME_EXT */
@@ -89,7 +89,14 @@ static int stress_filename_probe_length(
 			return -1;
 		}
 		(void)close(fd);
-		(void)shim_unlink(filename);
+		if (shim_unlink(filename)) {
+			pr_err("%s: unlink() failed when probing "
+				"for filename length, "
+				"errno=%d (%s)\n",
+				args->name, errno, strerror(errno));
+			*sz_max = 0;
+			return -1;
+		}
 		max = i;
 	}
 	*sz_max = max + 1;
@@ -153,7 +160,14 @@ static int stress_filename_probe(
 			}
 		} else {
 			(void)close(fd);
-			(void)shim_unlink(filename);
+			if (shim_unlink(filename)) {
+				pr_err("%s: unlink() failed when probing "
+					"for allowed filename characters, "
+					"errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				*chars_allowed = 0;
+				return -errno;
+			}
 			allowed[j] = (char)i;
 			j++;
 		}
@@ -204,7 +218,11 @@ static void stress_filename_generate(
  *  stress_filename_tidy()
  *	clean up residual files
  */
-static void stress_filename_tidy(const char *path)
+static void stress_filename_tidy(
+	stress_args_t *args,
+	const char *path,
+	int *rc
+)
 {
 	DIR *dir;
 
@@ -219,7 +237,12 @@ static void stress_filename_tidy(const char *path)
 				continue;
 			(void)stress_mk_filename(filename, sizeof(filename),
 				path, d->d_name);
-			(void)shim_unlink(filename);
+			if (shim_unlink(filename)) {
+				pr_fail("%s: unlink() failed when tidying, "
+					"errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				*rc = EXIT_FAILURE;
+			}
 		}
 		(void)closedir(dir);
 	}
@@ -248,6 +271,162 @@ static void stress_filename_generate_random(
 	filename[i] = '\0';
 }
 
+/*
+ *  stress_filename_generate_random_utf8()
+ *	generate a utf8 filename, may be legal or not
+ *	for the filename charset being supported
+ */
+static void stress_filename_generate_random_utf8(
+	char *filename,
+	const size_t sz_max)
+{
+	size_t i = 0, j = 0;
+
+	while (i < sz_max) {
+		const size_t residual = STRESS_MINIMUM(sz_max - i, 4);
+		const size_t len = stress_mwc8modn(residual) + 1;
+
+		switch (len) {
+		default:
+		case 1:
+			filename[i++] = stress_mwc8modn(127) + 1;
+			break;
+		case 2:
+			filename[i++] = 0xc0 | (stress_mwc8() & 0x1f);
+			j = i;
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			break;
+		case 3:
+			filename[i++] = 0xe0 | (stress_mwc8() & 0x0f);
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			j = i;
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			break;
+		case 4:
+			filename[i++] = 0xf0 | (stress_mwc8() & 0x07);
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			j = i;
+			filename[i++] = 0x80 | (stress_mwc8() & 0x3f);
+			break;
+		}
+	}
+	filename[i] = '\0';
+	/*
+	 *  occassionally truncate valid utf8 filename to create
+	 *  invalid utf8 strings, c.f.:
+	 *  https://sourceware.org/pipermail/cygwin/2024-September/256451.html
+	 */
+	if (j && stress_mwc8() < 16)
+		filename[j] = '\0';
+
+}
+
+/*
+ *  stress_filename_readdir()
+ *	the test directory should only contain files ., .. and the
+ *	test file. Hence we can sanity check the test file from what
+ *	readdir returns, the stat should be the same and the there
+ *	should be just one regular file in the test directory
+ */
+static int stress_filename_readdir(
+	stress_args_t *args,
+	const char *pathname,
+	const char *filename)
+{
+	DIR *dir;
+	struct dirent *d;
+	static int count = 0;
+	struct stat sb_file;
+	int readdir_count = 0, rc = 0;
+	static bool filename_differs = false;
+
+	/*
+	 *  Only perform expensive readdir test occasionally
+	 */
+	if (count <= 0) {
+		count = stress_mwc8() + 128;
+	} else {
+		count--;
+		return 0;
+	}
+
+	(void)shim_memset(&sb_file, 0, sizeof(sb_file));
+	if (stat(filename, &sb_file) < 0) {
+		pr_fail("%s: failed: cannot stat file, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		return -1;
+	}
+
+	dir = opendir(pathname);
+	if (!dir) {
+		pr_fail("%s: failed: cannot opendir directory '%s'\n",
+			args->name, pathname);
+		return -1;
+	}
+
+	while ((d = readdir(dir)) != NULL) {
+		char fullname[PATH_MAX];
+		struct stat sb_readdir_file;
+		int name_cmp;
+
+		/* ignore dot files */
+		if (stress_is_dot_filename(d->d_name))
+			continue;
+
+		readdir_count++;
+
+		(void)snprintf(fullname, sizeof(fullname), "%s/%s", pathname, d->d_name);
+		name_cmp = strcmp(fullname, filename);
+
+		/*
+		 *  if the name differs, stat fails or stat'd files
+		 *  differ (not the same file) then fail
+		 */
+		(void)shim_memset(&sb_readdir_file, 0, sizeof(sb_readdir_file));
+		if (name_cmp) {
+			if (stat(fullname, &sb_readdir_file) < 0) {
+				pr_fail("%s: failed: cannot stat readdir'd file, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				rc = -1;
+				continue;
+			}
+			/* stat'd files differ? */
+			if (memcmp(&sb_file, &sb_readdir_file, sizeof(sb_file))) {
+				pr_fail("%s: failed: difference between stat'd file and readdir'd file, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				rc = -1;
+				continue;
+			}
+			/*
+			 * stat'd files the same but name is different, report once,
+			 * this is not an error per-se, because bijective en/decoding between
+			 * API and filesystem is not always possible. This is in particular
+			 * the case on Cygwin due to a non-trivial conversion between current
+			 * codepage and UTF-16.
+			 */
+			if (!filename_differs) {
+				filename_differs = true;
+				pr_inf("%s: note: created filename different from one read by readdir()\n",
+					args->name);
+			}
+		}
+	}
+	/*
+	 *  excluding . and .. there should be just one test file in
+	 *  the directory, so check this
+	 */
+	if (readdir_count != 1) {
+		pr_fail("%s: failed: found %d files in test directory, expecting 1\n",
+			args->name, readdir_count);
+		rc = -1;
+	}
+	(void)closedir(dir);
+
+	return rc;
+}
+
+
 /*
  *  stress_filename_test()
  *	create a file, and check if it fails.
@@ -256,6 +435,7 @@ static void stress_filename_generate_random(
  */
 static void stress_filename_test(
 	stress_args_t *args,
+	const char *pathname,
 	const char *filename,
 	const size_t sz_max,
 	const bool should_pass,
@@ -291,7 +471,61 @@ static void stress_filename_test(
 		/* exercise dcache lookup of existent filename */
 		VOID_RET(int, shim_stat(filename, &buf));
 
-		(void)shim_unlink(filename);
+		if (should_pass)
+			if (stress_filename_readdir(args, pathname, filename) < 0)
+				*rc = EXIT_FAILURE;
+
+		if (shim_unlink(filename)) {
+			pr_fail("%s: unlink() failed on file of length "
+				"%zu bytes, errno=%d (%s)\n",
+				args->name, sz_max, errno, strerror(errno));
+			*rc = EXIT_FAILURE;
+			return;
+		}
+	}
+
+	/* exercise dcache lookup of non-existent filename */
+	ret = shim_stat(filename, &buf);
+	if (ret == 0) {
+		pr_fail("%s: stat succeeded on non-existent unlinked file\n",
+			args->name);
+		*rc = EXIT_FAILURE;
+	}
+}
+
+/*
+ *  stress_filename_test_utf8()
+ *      exercise utf8 filename, may or may not fail
+ */
+static void stress_filename_test_utf8(
+	stress_args_t *args,
+	const char *filename,
+	const size_t sz_max,
+	const pid_t pid,
+	int *rc)
+{
+	int fd;
+	int ret;
+	struct stat buf;
+
+	/* exercise dcache lookup of non-existent filename */
+	VOID_RET(int, shim_stat(filename, &buf));
+
+	if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0)
+		return;
+
+	stress_read_fdinfo(pid, fd);
+	(void)close(fd);
+
+	/* exercise dcache lookup of existent filename */
+	VOID_RET(int, shim_stat(filename, &buf));
+
+	if (shim_unlink(filename)) {
+		pr_fail("%s: unlink() failed on file of length "
+			"%zu bytes, errno=%d (%s)\n",
+			args->name, sz_max, errno, strerror(errno));
+		*rc = EXIT_FAILURE;
+		return;
 	}
 
 	/* exercise dcache lookup of non-existent filename */
@@ -396,7 +630,7 @@ static int stress_filename(stress_args_t *args)
 		break;
 	}
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: filesystem allows %zu unique "
 			"characters in a %zu character long filename\n",
 			args->name, chars_allowed, sz_max);
@@ -484,64 +718,77 @@ again:
 
 			/* Should succeed */
 			stress_filename_generate(ptr, 1, ch);
-			stress_filename_test(args, filename, 1, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, 1, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, 1, chars_allowed);
-			stress_filename_test(args, filename, 1, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, 1, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max, ch);
-			stress_filename_test(args, filename, sz_max, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, sz_max, chars_allowed);
-			stress_filename_test(args, filename, sz_max, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz_max - 1, ch);
-			stress_filename_test(args, filename, sz_max - 1, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max - 1, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, sz_max - 1, chars_allowed);
-			stress_filename_test(args, filename, sz_max - 1, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max - 1, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Should fail */
 			stress_filename_generate(ptr, sz_max + 1, ch);
-			stress_filename_test(args, filename, sz_max + 1, false, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max + 1, false, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, sz_max + 1, chars_allowed);
-			stress_filename_test(args, filename, sz_max + 1, false, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz_max + 1, false, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, sz, ch);
-			stress_filename_test(args, filename, sz, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, sz, chars_allowed);
-			stress_filename_test(args, filename, sz, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, sz, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			/* Should succeed */
 			stress_filename_generate(ptr, rnd_sz, ch);
-			stress_filename_test(args, filename, rnd_sz, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, rnd_sz, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 			stress_filename_generate_random(ptr, rnd_sz, chars_allowed);
-			stress_filename_test(args, filename, rnd_sz, true, mypid, &rc);
+			stress_filename_test(args, pathname, filename, rnd_sz, true, mypid, &rc);
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
+			/* May succeed or fail */
+			stress_filename_generate_random_utf8(ptr, sz_max - 1);
+			stress_filename_test_utf8(args, filename, sz_max - 1, pid, &rc);
+			if (UNLIKELY(!stress_continue(args)))
+				break;
+
+			if (filename_opt == STRESS_FILENAME_EXT) {
+				stress_filename_generate_random_utf8(ptr, rnd_sz);
+				stress_filename_test_utf8(args, filename, rnd_sz, pid, &rc);
+				if (UNLIKELY(!stress_continue(args)))
+					break;
+			}
+
 #if defined(HAVE_PATHCONF)
 #if defined(_PC_NAME_MAX)
 			VOID_RET(long int, pathconf(pathname, _PC_NAME_MAX));
@@ -564,7 +811,7 @@ again:
 
 tidy_dir:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)stress_filename_tidy(pathname);
+	stress_filename_tidy(args, pathname, &rc);
 
 	return rc;
 }
@@ -581,7 +828,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_filename_info = {
 	.stressor = stress_filename,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
