@@ -19,6 +19,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-lock.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-resources.h"
 
@@ -157,8 +158,18 @@ static int stress_forkheavy_child(stress_args_t *args, void *context)
 	if (min_mem_free < MIN_MEM_FREE)
 		min_mem_free = MIN_MEM_FREE;
 
-	(void)stress_get_setting("forkheavy-allocs", &forkheavy_allocs);
-	(void)stress_get_setting("forkheavy-procs", &forkheavy_procs);
+	if (!stress_get_setting("forkheavy-allocs", &forkheavy_allocs)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			forkheavy_allocs = MAX_FORKHEAVY_ALLOCS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			forkheavy_allocs = MAX_FORKHEAVY_ALLOCS;
+	}
+	if (!stress_get_setting("forkheavy-procs", &forkheavy_procs)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			forkheavy_procs = MAX_FORKHEAVY_PROCS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			forkheavy_procs = MIN_FORKHEAVY_PROCS;
+	}
 	(void)stress_get_setting("forkheavy-mlock", &forkheavy_mlock);
 
 #if defined(MCL_FUTURE)
@@ -248,7 +259,7 @@ static int stress_forkheavy(stress_args_t *args)
 	forkheavy_args.num_resources = DEFAULT_FORKHEAVY_ALLOCS;
 	forkheavy_args.resources = (stress_resources_t *)malloc(forkheavy_args.num_resources * sizeof(*forkheavy_args.resources));
 	if (!forkheavy_args.resources) {
-		pr_inf_skip("%s: cannot allocate %zd resource structures, skipping stressor\n",
+		pr_inf_skip("%s: cannot allocate %zu resource structures, skipping stressor\n",
 			args->name, forkheavy_args.num_resources);
 		return EXIT_NO_RESOURCE;
 	}
@@ -258,8 +269,8 @@ static int stress_forkheavy(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (metrics == MAP_FAILED) {
-		pr_inf_skip("%s: failed to memory map %zd bytes, skipping stressor\n",
-			args->name, sizeof(*metrics));
+		pr_inf_skip("%s: failed to memory map %zu bytes%s, skipping stressor\n",
+			args->name, sizeof(*metrics), stress_get_memfree_str());
 		free(forkheavy_args.resources);
 		return EXIT_NO_RESOURCE;
 	}
@@ -294,7 +305,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_forkheavy_info = {
 	.stressor = stress_forkheavy,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
