@@ -29,7 +29,7 @@ static const stress_help_t help[] = {
 #define STRESS_GOTO_BACKWARD	(1)
 #define STRESS_GOTO_RANDOM	(2)
 
-static const char *stress_goto_directions[] = {
+static const char * const stress_goto_directions[] = {
 	"forward",	/* STRESS_GOTO_FORWARD */
 	"backward",	/* STRESS_GOTO_BACKWARD */
 	"random",	/* STRESS_GOTO_RANDOM */
@@ -441,7 +441,7 @@ L0x000:
 
 const stressor_info_t stress_goto_info = {
 	.stressor = stress_goto,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -449,7 +449,7 @@ const stressor_info_t stress_goto_info = {
 #else
 const stressor_info_t stress_goto_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
