@@ -18,14 +18,18 @@
  *
  */
 #include "stress-ng.h"
+#include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-sort.h"
 
 #define MIN_HEAPSORT_SIZE	(1 * KB)
 #define MAX_HEAPSORT_SIZE	(4 * MB)
 #define DEFAULT_HEAPSORT_SIZE	(256 * KB)
 
+#if defined(HAVE_SIGLONGJMP)
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"heapsort N",	   	"start N workers heap sorting 32 bit random integers" },
@@ -140,6 +144,7 @@ static const stress_opt_t opts[] = {
 	END_OPT,
 };
 
+#if defined(HAVE_SIGLONGJMP)
 /*
  *  stress_heapsort_handler()
  *	SIGALRM generic handler
@@ -151,8 +156,10 @@ static void MLOCKED_TEXT stress_heapsort_handler(int signum)
 	if (do_jmp) {
 		do_jmp = false;
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
 	}
 }
+#endif
 
 /*
  *  stress_heapsort()
@@ -162,18 +169,20 @@ static int stress_heapsort(stress_args_t *args)
 {
 	uint64_t heapsort_size = DEFAULT_HEAPSORT_SIZE;
 	int32_t *data, *ptr;
-	size_t n, i, heapsort_method = 0;
-	struct sigaction old_action;
-	int ret;
+	size_t n, i, data_size, heapsort_method = 0;
 	double rate;
 	NOCLOBBER int rc = EXIT_SUCCESS;
 	NOCLOBBER double duration = 0.0, count = 0.0, sorted = 0.0;
 	heapsort_func_t heapsort_func;
+#if defined(HAVE_SIGLONGJMP)
+	struct sigaction old_action;
+	int ret;
+#endif
 
 	(void)stress_get_setting("heapsort-method", &heapsort_method);
 
 	heapsort_func = stress_heapsort_methods[heapsort_method].heapsort_func;
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_inf("%s: using method '%s'\n",
 			args->name, stress_heapsort_methods[heapsort_method].name);
 
@@ -184,14 +193,21 @@ static int stress_heapsort(stress_args_t *args)
 			heapsort_size = MIN_HEAPSORT_SIZE;
 	}
 	n = (size_t)heapsort_size;
+	data_size = n * sizeof(*data);
 
-	if ((data = (int32_t *)calloc(n, sizeof(*data))) == NULL) {
-		pr_inf_skip("%s: failed to allocate %zu integers, skipping stressor\n",
-			args->name, n);
+	data = (int32_t *)stress_mmap_populate(NULL, data_size,
+				PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (data == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zu 32 bit integers%s, "
+			"errno=%d (%s), skipping stressor\n", args->name, n,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	(void)stress_madvise_collapse(data, data_size);
+	stress_set_vma_anon_name(data, data_size, "heapsort-data");
 
-
+#if defined(HAVE_SIGLONGJMP)
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
 		/*
@@ -200,11 +216,11 @@ static int stress_heapsort(stress_args_t *args)
 		(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 		goto tidy;
 	}
-
 	if (stress_sighandler(args->name, SIGALRM, stress_heapsort_handler, &old_action) < 0) {
 		free(data);
 		return EXIT_FAILURE;
 	}
+#endif
 
 	stress_sort_data_int32_init(data, n);
 
@@ -221,7 +237,7 @@ static int stress_heapsort(stress_args_t *args)
 		stress_sort_compare_reset();
 		t = stress_time_now();
 		if (heapsort_func(data, n, sizeof(*data), stress_sort_cmp_fwd_int32) < 0) {
-			pr_fail("%s: heapsort of random data failed: %d (%s)\n",
+			pr_fail("%s: heapsort of random data failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		} else {
@@ -247,7 +263,7 @@ static int stress_heapsort(stress_args_t *args)
 		stress_sort_compare_reset();
 		t = stress_time_now();
 		if (heapsort_func(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
-			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
+			pr_fail("%s: reversed heapsort of random data failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		} else {
@@ -276,7 +292,7 @@ static int stress_heapsort(stress_args_t *args)
 		stress_sort_compare_reset();
 		t = stress_time_now();
 		if (heapsort_func(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
-			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
+			pr_fail("%s: reversed heapsort of random data failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		} else {
@@ -301,9 +317,11 @@ static int stress_heapsort(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
+#if defined(HAVE_SIGLONGJMP)
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
+#endif
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "heapsort comparisons per sec",
@@ -311,14 +329,14 @@ tidy:
 	stress_metrics_set(args, 1, "heapsort comparisons per item",
 		count / sorted, STRESS_METRIC_HARMONIC_MEAN);
 
-	free(data);
+	(void)munmap((void *)data, data_size);
 
 	return rc;
 }
 
 const stressor_info_t stress_heapsort_info = {
 	.stressor = stress_heapsort,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SORT,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SORT,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
