@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-pthread.h"
+#include "core-shim.h"
 
 #include <sys/ioctl.h>
 
@@ -49,6 +50,7 @@ static const stress_help_t help[] = {
 typedef struct {
 	int dir_fd;
 	int file_fd;
+	char *file_name;
 } stress_data_t;
 
 static void *inode_flags_counter_lock;
@@ -106,6 +108,30 @@ static const int inode_flags[] = {
 #endif
 };
 
+static const int attr_flags[] = {
+#if defined(FS_SYNC_FL)
+	FS_SYNC_FL,
+#endif
+#if defined(FS_IMMUTABLE_FL)
+	FS_IMMUTABLE_FL,
+#endif
+#if defined(FS_APPEND_FL)
+	FS_APPEND_FL,
+#endif
+#if defined(FS_NODUMP_FL)
+	FS_NODUMP_FL,
+#endif
+#if defined(FS_NOATIME_FL)
+	FS_NOATIME_FL,
+#endif
+#if defined(FS_DAX_FL)
+	FS_DAX_FL,
+#endif
+#if defined(FS_PROJINHERIT_FL)
+	FS_PROJINHERIT_FL,
+#endif
+};
+
 /*
  *  stress_inode_flags_ioctl()
  *	try and toggle an inode flag on/off
@@ -159,9 +185,21 @@ static int stress_inode_flags_stressor(
 	const stress_data_t *data)
 {
 	size_t idx = 0;
+	char *file_name;
+#if defined(AT_EMPTY_PATH)
+	char *two_pages;
+
+	two_pages = calloc(2, args->page_size);
+#endif
+
+	/* Find basename */
+	file_name = strrchr(data->file_name, '/');
+	if (file_name)
+		file_name++;
 
 	while (LIKELY(keep_running && stress_continue(args))) {
 		size_t i;
+		struct shim_file_attr ufattr;
 
 		/* Work through all inode flag permutations */
 		stress_inode_flags_ioctl(args, data->dir_fd, inode_flag_perms[idx]);
@@ -183,7 +221,57 @@ static int stress_inode_flags_stressor(
 		stress_inode_flags_ioctl_sane(data->file_fd);
 		shim_fsync(data->file_fd);
 		VOID_RET(bool, stress_bogo_inc_lock(args, inode_flags_counter_lock, 1));
+
+#if defined(AT_EMPTY_PATH)
+		if (shim_file_getattr(data->dir_fd, NULL, &ufattr, sizeof(ufattr), AT_EMPTY_PATH) == 0)
+			VOID_RET(int, shim_file_setattr(data->dir_fd, NULL, &ufattr, sizeof(ufattr), AT_EMPTY_PATH));
+#endif
+		if (file_name) {
+			/* Valid filename, exercise file_{get|set}attr() */
+			if (shim_file_getattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0) == 0)
+				VOID_RET(int, shim_file_setattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0));
+
+			if (shim_file_getattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0) == 0) {
+				struct shim_file_attr ufattr_save;
+
+				for (i = 0; LIKELY(stress_continue(args) && (i < SIZEOF_ARRAY(attr_flags))); i++) {
+					(void)shim_memcpy(&ufattr_save, &ufattr, sizeof(ufattr_save));
+
+					ufattr.fa_xflags |= attr_flags[i];
+					VOID_RET(int, shim_file_setattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0));
+
+					(void)shim_memcpy(&ufattr, &ufattr_save, sizeof(ufattr_save));
+					VOID_RET(int, shim_file_setattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0));
+				}
+				for (i = 0; LIKELY(stress_continue(args) && (i < SIZEOF_ARRAY(attr_flags))); i++) {
+					(void)shim_memcpy(&ufattr_save, &ufattr, sizeof(ufattr_save));
+
+					if (ufattr.fa_xflags & attr_flags[i]) {
+						ufattr.fa_xflags &= ~attr_flags[i];
+						VOID_RET(int, shim_file_setattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0));
+
+						(void)shim_memcpy(&ufattr, &ufattr_save, sizeof(ufattr_save));
+						VOID_RET(int, shim_file_setattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr), 0));
+					}
+				}
+			}
+
+			/* Invalid size */
+			VOID_RET(int, shim_file_getattr(data->dir_fd, file_name, &ufattr, sizeof(ufattr) >> 1, 0));
+		}
+		/* Invalid filename */
+		VOID_RET(int, shim_file_getattr(data->dir_fd, "", &ufattr, sizeof(ufattr), 0));
+
+#if defined(AT_EMPTY_PATH)
+		/* Invalid size, E2BIG */
+		VOID_RET(int, shim_file_getattr(data->dir_fd, NULL, (struct shim_file_attr *)two_pages,
+			args->page_size * 2, AT_EMPTY_PATH));
+#endif
 	}
+#if defined(AT_EMPTY_PATH)
+	if (two_pages)
+		free(two_pages);
+#endif
 
 	return 0;
 }
@@ -251,6 +339,8 @@ static int stress_inode_flags(stress_args_t *args)
 	(void)shim_strscpy(tmp, file_name, sizeof(tmp));
 	dir_name = dirname(tmp);
 
+	data.file_name = file_name;
+
 	data.dir_fd = open(dir_name, O_RDONLY | O_DIRECTORY);
 	if (data.dir_fd < 0) {
 		pr_err("%s: cannot open %s, errno=%d (%s)\n",
