@@ -19,11 +19,10 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "io-uring.h"
 
-#define SQE_SET_OPTIMIZE	(1)
-
 #if defined(HAVE_LINUX_IO_URING_H)
 #include <linux/io_uring.h>
 #endif
@@ -34,6 +33,9 @@
 #include <attr/xattr.h>
 #endif
 
+#define MIN_IO_URING_ENTRIES	(1)
+#define MAX_IO_URING_ENTRIES	(16384)
+
 static const stress_help_t help[] = {
 	{ NULL,	"io-uring N",		"start N workers that issue io-uring I/O requests" },
 	{ NULL, "io-uring-entries N",	"specify number if io-uring ring entries" },
@@ -43,7 +45,7 @@ static const stress_help_t help[] = {
 };
 
 static const stress_opt_t opts[] = {
-	{ OPT_io_uring_entries, "io-uring-entries", TYPE_ID_UINT32, 1, 16384, NULL },
+	{ OPT_io_uring_entries, "io-uring-entries", TYPE_ID_UINT32, MIN_IO_URING_ENTRIES, MAX_IO_URING_ENTRIES, NULL },
 	{ OPT_io_uring_rand,    "io-uring-rand",    TYPE_ID_BOOL,   0, 1, NULL },
 	END_OPT,
 };
@@ -69,7 +71,8 @@ static const stress_opt_t opts[] = {
      defined(HAVE_IORING_OP_STATX) || 	\
      defined(HAVE_IORING_OP_SETXATTR) || \
      defined(HAVE_IORING_OP_GETXATTR) || \
-     defined(HAVE_IORING_OP_SYNC_FILE_RANGE))
+     defined(HAVE_IORING_OP_SYNC_FILE_RANGE) ||	\
+     defined(HAVE_IORING_OP_FTRUNCATE))
 
 /*
  *  io uring file info
@@ -260,9 +263,10 @@ static int stress_setup_io_uring(
 		MAP_SHARED | MAP_POPULATE,
 		submit->io_uring_fd, IORING_OFF_SQ_RING);
 	if (submit->sq_mmap == MAP_FAILED) {
-		pr_inf_skip("%s: could not mmap submission queue buffer, "
+		pr_inf_skip("%s: could not mmap submission queue buffer%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -274,9 +278,10 @@ static int stress_setup_io_uring(
 				MAP_SHARED | MAP_POPULATE,
 				submit->io_uring_fd, IORING_OFF_CQ_RING);
 		if (submit->cq_mmap == MAP_FAILED) {
-			pr_inf_skip("%s: could not mmap completion queue buffer, "
+			pr_inf_skip("%s: could not mmap completion queue buffer%s, "
 				"errno=%d (%s), skipping stressor\n",
-				args->name, errno, strerror(errno));
+				args->name, stress_get_memfree_str(),
+				errno, strerror(errno));
 			(void)munmap(submit->sq_mmap, submit->cq_size);
 			return EXIT_NO_RESOURCE;
 		}
@@ -295,9 +300,10 @@ static int stress_setup_io_uring(
 			PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE,
 			submit->io_uring_fd, IORING_OFF_SQES);
 	if (submit->sqes_mmap == MAP_FAILED) {
-		pr_inf_skip("%s: count not mmap submission queue buffer, "
+		pr_inf_skip("%s: count not mmap submission queue buffer%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		if (submit->cq_mmap != submit->sq_mmap)
 			(void)munmap(submit->cq_mmap, submit->cq_size);
 		(void)munmap(submit->sq_mmap, submit->sq_size);
@@ -437,9 +443,7 @@ static int stress_io_uring_submit(
 	stress_asm_mb();
 	idx = tail & *submit->sq_ring.ring_mask;
 	sqe = &submit->sqes_mmap[idx];
-#if !defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 
 	setup_func(io_uring_file, sqe, extra_data);
 	/* Save opcode for later completion error reporting */
@@ -485,7 +489,7 @@ retry:
  *  stress_io_uring_cancel_setup()
  *	setup cancel submit over io_uring
  */
-static void stress_io_uring_async_cancel_setup(
+static void OPTIMIZE3 stress_io_uring_async_cancel_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -495,19 +499,11 @@ static void stress_io_uring_async_cancel_setup(
 	const struct io_uring_sqe *sqe_to_cancel =
 		(const struct io_uring_sqe *)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = sqe_to_cancel->fd;
 	sqe->flags = 2;
 	sqe->opcode = IORING_OP_ASYNC_CANCEL;
 	sqe->addr = sqe_to_cancel->addr;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->off = 0;
-	sqe->len = 0;
-	sqe->splice_fd_in = 0;
-#endif
 }
 
 /*
@@ -553,21 +549,15 @@ static void stress_io_uring_cancel_rdwr(
  *  stress_io_uring_readv_setup()
  *	setup readv submit over io_uring
  */
-static void stress_io_uring_readv_setup(
+static void OPTIMIZE3 stress_io_uring_readv_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->opcode = IORING_OP_READV;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs;
 	sqe->len = io_uring_file->blocks;
@@ -581,21 +571,15 @@ static void stress_io_uring_readv_setup(
  *  stress_io_uring_writev_setup()
  *	setup writev submit over io_uring
  */
-static void stress_io_uring_writev_setup(
+static void OPTIMIZE3 stress_io_uring_writev_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->opcode = IORING_OP_WRITEV;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs;
 	sqe->len = io_uring_file->blocks;
@@ -609,21 +593,15 @@ static void stress_io_uring_writev_setup(
  *  stress_io_uring_read_setup()
  *	setup read submit over io_uring
  */
-static void stress_io_uring_read_setup(
+static void OPTIMIZE3 stress_io_uring_read_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->opcode = IORING_OP_READ;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
 	sqe->len = io_uring_file->iovecs[0].iov_len;
@@ -637,21 +615,15 @@ static void stress_io_uring_read_setup(
  *  stress_io_uring_write_setup()
  *	setup write submit over io_uring
  */
-static void stress_io_uring_write_setup(
+static void OPTIMIZE3 stress_io_uring_write_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->opcode = IORING_OP_WRITE;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
 	sqe->len = io_uring_file->iovecs[0].iov_len;
@@ -665,26 +637,16 @@ static void stress_io_uring_write_setup(
  *  stress_io_uring_fsync_setup()
  *	setup fsync submit over io_uring
  */
-static void stress_io_uring_fsync_setup(
+static void OPTIMIZE3 stress_io_uring_fsync_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FSYNC;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->len = 0;
-	sqe->off = 0;
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-	sqe->rw_flags = 0;
-#endif
 }
 #endif
 
@@ -693,7 +655,7 @@ static void stress_io_uring_fsync_setup(
  *  stress_io_uring_nop_setup()
  *	setup nop submit over io_uring
  */
-static void stress_io_uring_nop_setup(
+static void OPTIMIZE3 stress_io_uring_nop_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -701,9 +663,7 @@ static void stress_io_uring_nop_setup(
 	(void)io_uring_file;
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->opcode = IORING_OP_NOP;
 }
 #endif
@@ -713,78 +673,48 @@ static void stress_io_uring_nop_setup(
  *  stress_io_uring_fallocate_setup()
  *	setup fallocate submit over io_uring
  */
-static void stress_io_uring_fallocate_setup(
+static void OPTIMIZE3 stress_io_uring_fallocate_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FALLOCATE;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->off = 0;			/* offset */
-#endif
 	sqe->addr = stress_mwc16();	/* length */
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->len = 0;			/* mode */
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-	sqe->rw_flags = 0;
-#endif
 }
 #endif
 
 #if defined(HAVE_IORING_OP_FADVISE)
 /*
- *  stress_io_uring_fadvise_setup ()
+ *  stress_io_uring_fadvise_setup()
  *	setup fadvise submit over io_uring
  */
-static void stress_io_uring_fadvise_setup(
+static void OPTIMIZE3 stress_io_uring_fadvise_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_FADVISE;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->off = 0;			/* offset */
-#endif
 	sqe->len = io_uring_rand ?  stress_mwc16(): 1024;
 #if defined(POSIX_FADV_NORMAL)
 	sqe->fadvise_advice = POSIX_FADV_NORMAL;
-#else
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->fadvise_advice = 0;
-#endif
-#endif
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-	sqe->addr = 0;
 #endif
 }
 #endif
 
 #if defined(HAVE_IORING_OP_CLOSE)
 /*
- *  stress_io_uring_close_setup ()
+ *  stress_io_uring_close_setup()
  *	setup close submit over io_uring
  */
-static void stress_io_uring_close_setup(
+static void OPTIMIZE3 stress_io_uring_close_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -792,21 +722,10 @@ static void stress_io_uring_close_setup(
 	(void)io_uring_file;
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	/* don't worry about bad fd if dup fails */
 	sqe->fd = dup(io_uring_file->fd_dup);
 	sqe->opcode = IORING_OP_CLOSE;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->ioprio = 0;
-	sqe->off = 0;
-	sqe->addr = 0;
-	sqe->len = 0;
-	sqe->rw_flags = 0;
-	sqe->buf_index = 0;
-#endif
 }
 #endif
 
@@ -815,33 +734,20 @@ static void stress_io_uring_close_setup(
  *  stress_io_uring_madvise_setup ()
  *	setup madvise submit over io_uring
  */
-static void stress_io_uring_madvise_setup(
+static void OPTIMIZE3 stress_io_uring_madvise_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
 	sqe->opcode = IORING_OP_MADVISE;
 	sqe->addr = (uintptr_t)io_uring_file->iovecs[0].iov_base;
 	sqe->len = 4096;
 #if defined(MADV_NORMAL)
 	sqe->fadvise_advice = MADV_NORMAL;
-#else
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->fadvise_advice = 0;
-#endif
-#endif
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-	sqe->off = 0;
 #endif
 }
 #endif
@@ -851,7 +757,7 @@ static void stress_io_uring_madvise_setup(
  *  stress_io_uring_statx_setup()
  *	setup statx submit over io_uring
  */
-static void stress_io_uring_statx_setup(
+static void OPTIMIZE3 stress_io_uring_statx_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -864,20 +770,12 @@ static void stress_io_uring_statx_setup(
 	if (io_uring_file->fd_at >= 0) {
 		static shim_statx_t statxbuf;
 
-#if defined(SEQ_LATE_MEMSET)
 		(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 		sqe->opcode = IORING_OP_STATX;
 		sqe->fd = io_uring_file->fd_at;
 		sqe->addr = (uintptr_t)"";
 		sqe->addr2 = (uintptr_t)&statxbuf;
 		sqe->statx_flags = AT_EMPTY_PATH;
-#if defined(SQE_SET_OPTIMIZE)
-		/* memset to zero already, so no need for following */
-		sqe->ioprio = 0;
-		sqe->buf_index = 0;
-		sqe->flags = 0;
-#endif
 		sqe->len = STATX_SIZE;
 	}
 #endif
@@ -889,26 +787,17 @@ static void stress_io_uring_statx_setup(
  *  stress_io_uring_sync_file_range_setup()
  *	setup sync_file_range submit over io_uring
  */
-static void stress_io_uring_sync_file_range_setup(
+static void OPTIMIZE3 stress_io_uring_sync_file_range_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
 {
 	(void)extra_info;
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->fd = io_uring_file->fd;
 	sqe->off = stress_mwc16() & ~511UL;
 	sqe->len = stress_mwc32() & ~511UL;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-	sqe->addr = 0;
-	sqe->ioprio = 0;
-	sqe->buf_index = 0;
-#endif
 }
 #endif
 
@@ -918,7 +807,7 @@ static void stress_io_uring_sync_file_range_setup(
  *  stress_io_uring_setxattr_setup()
  *	setup setxattr submit over io_uring
  */
-static void stress_io_uring_setxattr_setup(
+static void OPTIMIZE3 stress_io_uring_setxattr_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -928,27 +817,11 @@ static void stress_io_uring_setxattr_setup(
 
 	static char attr_value[] = "ioring-xattr-data";
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->opcode = IORING_OP_SETXATTR;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->fd = 0;
-#endif
 	sqe->off = (uintptr_t)attr_value;
 	sqe->len = sizeof(attr_value);
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->addr = (uintptr_t)"user.var_test";
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->ioprio = 0;
-	sqe->rw_flags = 0;
-	sqe->buf_index = 0;
-#endif
 	sqe->addr3 = (uintptr_t)io_uring_file->filename;
         sqe->xattr_flags = XATTR_CREATE;
 }
@@ -961,7 +834,7 @@ static void stress_io_uring_setxattr_setup(
  *  stress_io_uring_getxattr_setup()
  *	setup getxattr submit over io_uring
  */
-static void stress_io_uring_getxattr_setup(
+static void OPTIMIZE3 stress_io_uring_getxattr_setup(
 	const stress_io_uring_file_t *io_uring_file,
 	struct io_uring_sqe *sqe,
 	const void *extra_info)
@@ -971,36 +844,37 @@ static void stress_io_uring_getxattr_setup(
 
 	static char attr_value[128];
 
-#if defined(SEQ_LATE_MEMSET)
 	(void)shim_memset(sqe, 0, sizeof(*sqe));
-#endif
 	sqe->opcode = IORING_OP_GETXATTR;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->fd = 0;
-#endif
 	sqe->off = (uintptr_t)attr_value;
 	sqe->len = sizeof(attr_value);
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->flags = 0;
-#endif
 	sqe->addr = (uintptr_t)"user.var_test";
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-	sqe->ioprio = 0;
-	sqe->rw_flags = 0;
-	sqe->buf_index = 0;
-#endif
 	sqe->addr3 = (uintptr_t)io_uring_file->filename;
-#if defined(SQE_SET_OPTIMIZE)
-	/* memset to zero already, so no need for following */
-        sqe->xattr_flags = 0;
-#endif
 }
 #endif
 
 
+#if defined(HAVE_IORING_OP_FTRUNCATE)
+/*
+ *  stress_io_uring_ftruncate_setup ()
+ *	setup ftruncate submit over io_uring
+ */
+static void OPTIMIZE3 stress_io_uring_ftruncate_setup(
+	const stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe,
+	const void *extra_info)
+{
+	(void)io_uring_file;
+	(void)extra_info;
+
+	(void)shim_memset(sqe, 0, sizeof(*sqe));
+	/* don't worry about bad fd if dup fails */
+	sqe->fd = io_uring_file->fd;
+	sqe->opcode = IORING_OP_FTRUNCATE;
+	sqe->off = stress_mwc16() & ~511UL;
+}
+#endif
+
 /*
  *  We have some duplications here because we want to perform more than
  *  one of these io-urion ops per round before we do a completion so we
@@ -1051,6 +925,9 @@ static const stress_io_uring_setup_info_t stress_io_uring_setups[] = {
     defined(XATTR_CREATE)
 	{ IORING_OP_GETXATTR,	"IORING_OP_GETXATTR",	stress_io_uring_getxattr_setup },
 #endif
+#if defined(HAVE_IORING_OP_FTRUNCATE)
+	{ IORING_OP_FTRUNCATE,	"IORING_OP_FTRUNCATE",	stress_io_uring_ftruncate_setup },
+#endif
 };
 
 /*
@@ -1102,7 +979,12 @@ static int stress_io_uring_child(stress_args_t *args, void *context)
 
 	io_uring_rand = false;
 
-	(void)stress_get_setting("io-uring-entries", &io_uring_entries);
+	if (!stress_get_setting("io-uring-entries", &io_uring_entries)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			io_uring_entries = MAX_IO_URING_ENTRIES;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			io_uring_entries = MIN_IO_URING_ENTRIES;
+	}
 	(void)stress_get_setting("io-uring-rand", &io_uring_rand);
 
 	(void)shim_memset(&submit, 0, sizeof(submit));
@@ -1135,8 +1017,9 @@ static int stress_io_uring_child(stress_args_t *args, void *context)
 				MAP_SHARED | MAP_POPULATE | MAP_ANONYMOUS, -1, 0);
 		if (io_uring_file.iovecs[i].iov_base == MAP_FAILED) {
 			io_uring_file.iovecs[i].iov_base = NULL;
-			pr_inf_skip("%s: cannot mmap allocate iovec iov_base, errno=%d (%s), "
+			pr_inf_skip("%s: cannot mmap allocate iovec iov_base%s, errno=%d (%s), "
 				"skipping stressor\n", args->name,
+				stress_get_memfree_str(),
 				errno, strerror(errno));
 			stress_io_uring_unmap_iovecs(&io_uring_file);
 			return EXIT_NO_RESOURCE;
@@ -1240,7 +1123,7 @@ static int stress_io_uring(stress_args_t *args)
 
 const stressor_info_t stress_io_uring_info = {
 	.stressor = stress_io_uring,
-	.class = CLASS_IO | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -1248,7 +1131,7 @@ const stressor_info_t stress_io_uring_info = {
 #else
 const stressor_info_t stress_io_uring_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_IO | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
