@@ -50,7 +50,7 @@ static void *counter_lock;
  *  stress_iomix_rnd_offset()
  *	generate a random offset between 0..max-1
  */
-static off_t stress_iomix_rnd_offset(const off_t max)
+static inline off_t stress_iomix_rnd_offset(const off_t max)
 {
 	return (off_t)stress_mwc64modn((uint64_t)max);
 }
@@ -118,7 +118,7 @@ static void stress_iomix_fsync_min_1Hz(const int fd)
 			(void)shim_fdatasync(fd);
 			break;
 		case 2:
-			(void)sync();
+			shim_sync();
 			break;
 		}
 	}
@@ -561,8 +561,11 @@ static void stress_iomix_rd_wr_mmap(
 				stress_uint8_put(buffer[0]);
 
 				stress_rndbuf(mmaps[i], page_size);
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 				(void)shim_msync(mmaps[i], page_size,
 					stress_mwc1() ? MS_ASYNC : MS_SYNC);
+#endif
 			}
 		}
 		(void)shim_usleep(100000);
@@ -854,21 +857,21 @@ static void stress_iomix_drop_caches(
 	(void)iomix_bytes;
 
 	do {
-		(void)sync();
+		shim_sync();
 		if (stress_system_write("/proc/sys/vm/drop_caches", "1", 1) < 0)
-			(void)pause();
+			(void)shim_pause();
 		(void)sleep(5);
 		if (UNLIKELY(!stress_continue(args)))
 			return;
-		(void)sync();
+		shim_sync();
 		if (stress_system_write("/proc/sys/vm/drop_caches", "2", 1) < 0)
-			(void)pause();
+			(void)shim_pause();
 		(void)sleep(5);
 		if (UNLIKELY(!stress_continue(args)))
 			return;
-		(void)sync();
+		shim_sync();
 		if (stress_system_write("/proc/sys/vm/drop_caches", "3", 1) < 0)
-			(void)pause();
+			(void)shim_pause();
 		(void)sleep(5);
 	} while (stress_bogo_inc_lock(args, counter_lock, true));
 }
@@ -1028,7 +1031,29 @@ static void stress_iomix_cachestat(
 }
 #endif
 
-static stress_iomix_func iomix_funcs[] = {
+#if defined(HAVE_READAHEAD)
+static void stress_iomix_readahead(
+	stress_args_t *args,
+	const int fd,
+	const char *fs_type,
+	const off_t iomix_bytes)
+{
+	(void)fs_type;
+
+	do {
+		const off_t offset = stress_iomix_rnd_offset(iomix_bytes);
+		const size_t len = 512 * stress_mwc8modn(16);
+
+		VOID_RET(int, readahead(fd, offset, len));
+
+		(void)shim_usleep(stress_mwc32modn(2000000));
+		if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
+			return;
+	} while (stress_bogo_inc_lock(args, counter_lock, false));
+}
+#endif
+
+static const stress_iomix_func iomix_funcs[] = {
 	stress_iomix_wr_seq_bursts,
 	stress_iomix_wr_rnd_bursts,
 	stress_iomix_wr_seq_slow,
@@ -1062,6 +1087,9 @@ static stress_iomix_func iomix_funcs[] = {
     defined(__NR_cachestat)
 	stress_iomix_cachestat,
 #endif
+#if defined(HAVE_READAHEAD)
+	stress_iomix_readahead,
+#endif
 };
 
 #define MAX_IOMIX_PROCS	(SIZEOF_ARRAY(iomix_funcs))
@@ -1074,7 +1102,7 @@ static int stress_iomix(stress_args_t *args)
 {
 	int fd, ret;
 	char filename[PATH_MAX];
-	off_t iomix_bytes = DEFAULT_IOMIX_BYTES;
+	off_t iomix_bytes, iomix_bytes_total = DEFAULT_IOMIX_BYTES;
 	const size_t page_size = args->page_size;
 	size_t i;
 	stress_pid_t *s_pids, *s_pids_head = NULL;
@@ -1085,9 +1113,10 @@ static int stress_iomix(stress_args_t *args)
 	if (stress_sigchld_set_handler(args) < 0)
 		return EXIT_NO_RESOURCE;
 
-	s_pids = stress_s_pids_mmap(MAX_IOMIX_PROCS);
+	s_pids = stress_sync_s_pids_mmap(MAX_IOMIX_PROCS);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu PIDs, skipping stressor\n", args->name, MAX_IOMIX_PROCS);
+		pr_inf_skip("%s: failed to mmap %zu PIDs%s, skipping stressor\n",
+			args->name, MAX_IOMIX_PROCS, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -1102,17 +1131,23 @@ static int stress_iomix(stress_args_t *args)
 		goto tidy_s_pids;
 	}
 
-	if (!stress_get_setting("iomix-bytes", &iomix_bytes)) {
+	if (!stress_get_setting("iomix-bytes", &iomix_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			iomix_bytes = MAXIMIZED_FILE_SIZE;
+			iomix_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			iomix_bytes = MIN_IOMIX_BYTES;
+			iomix_bytes_total = MIN_IOMIX_BYTES;
 	}
-	iomix_bytes /= args->num_instances;
-	if (iomix_bytes < (off_t)MIN_IOMIX_BYTES)
+	iomix_bytes = iomix_bytes_total / args->instances;
+	if (iomix_bytes < (off_t)MIN_IOMIX_BYTES) {
 		iomix_bytes = (off_t)MIN_IOMIX_BYTES;
-	if (iomix_bytes < (off_t)page_size)
+		iomix_bytes_total = iomix_bytes * args->instance;
+	}
+	if (iomix_bytes < (off_t)page_size) {
 		iomix_bytes = (off_t)page_size;
+		iomix_bytes_total = iomix_bytes * args->instance;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, iomix_bytes, iomix_bytes_total);
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
@@ -1191,7 +1226,7 @@ static int stress_iomix(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		pause();
+		(void)shim_pause();
 	} while (stress_bogo_inc_lock(args, counter_lock, false));
 
 	ret = EXIT_SUCCESS;
@@ -1205,7 +1240,7 @@ lock_destroy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)stress_lock_destroy(counter_lock);
 tidy_s_pids:
-	(void)stress_s_pids_munmap(s_pids, MAX_IOMIX_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, MAX_IOMIX_PROCS);
 
 	return ret;
 }
@@ -1217,7 +1252,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_iomix_info = {
 	.stressor = stress_iomix,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
