@@ -26,6 +26,9 @@
 #include <intel-ipsec-mb.h>
 #endif
 
+#define MIN_IPSEC_MB_JOBS		(1)
+#define MAX_IPSEC_MB_JOBS		(65536)
+
 #if IMB_VERSION_NUM <= 0x3700
 #define IMB_MGR				MB_MGR
 #define IMB_JOB				JOB_AES_HMAC
@@ -150,7 +153,7 @@ static uint64_t stress_ipsec_mb_features(stress_args_t *args, const IMB_MGR *p_m
 {
 	const uint64_t features = p_mgr->features;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		char str[256] = "";
 		size_t i;
 
@@ -702,7 +705,7 @@ static void stress_ipsec_all(
 	const size_t data_len,
 	const int jobs);
 
-static stress_ipsec_funcs_t stress_ipsec_funcs[] = {
+static const stress_ipsec_funcs_t stress_ipsec_funcs[] = {
 	{ stress_ipsec_all,		"all",		},
 	{ stress_ipsec_cmac,		"cmac",		},
 	{ stress_ipsec_ctr,		"ctr",		},
@@ -764,7 +767,7 @@ static const char *stress_ipsec_mb_method(const size_t i)
 
 /*
  *  stress_ipsec_mb()
- *      stress Intel ipsec_mb instruction
+ *      stress Intel ipsec_mb library functionality
  */
 static int stress_ipsec_mb(stress_args_t *args)
 {
@@ -777,11 +780,16 @@ static int stress_ipsec_mb(stress_args_t *args)
 	size_t ipsec_mb_method = 0;
 	int ipsec_mb_jobs = 128;
 
-	(void)stress_get_setting("ipsec-mb-jobs", &ipsec_mb_jobs);
+	if (!stress_get_setting("ipsec-mb-jobs", &ipsec_mb_jobs)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			ipsec_mb_jobs = MAX_IPSEC_MB_JOBS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			ipsec_mb_jobs = MIN_IPSEC_MB_JOBS;
+	}
 	(void)stress_get_setting("ipsec-mb-method", &ipsec_mb_method);
 
 	if (imb_get_version() < IMB_VERSION(0, 51, 0)) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: version %s of Intel IPSec MB library is too low, skipping\n",
 				args->name, imb_get_version_str());
 		return EXIT_NOT_IMPLEMENTED;
@@ -789,7 +797,7 @@ static int stress_ipsec_mb(stress_args_t *args)
 
 	mb_mgr = alloc_mb_mgr(0);
 	if (!mb_mgr) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: failed to setup Intel IPSec MB library, skipping\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
@@ -804,7 +812,7 @@ static int stress_ipsec_mb(stress_args_t *args)
 		mb_features[i].stats.duration = 0.0;
 	}
 	if (!got_features) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: not enough CPU features to support Intel IPSec MB library, skipping\n", args->name);
 		free_mb_mgr(mb_mgr);
 		return EXIT_NOT_IMPLEMENTED;
@@ -814,7 +822,7 @@ static int stress_ipsec_mb(stress_args_t *args)
 		const char *feature_name = mb_features[ipsec_mb_feature].name;
 
 		if (!mb_features[ipsec_mb_feature].supported) {
-			if (args->instance == 0) {
+			if (stress_instance_zero(args)) {
 
 				pr_inf_skip("%s: requested ipsec-mb-feature feature '%s' is not supported, skipping\n",
 					args->name, feature_name);
@@ -825,7 +833,7 @@ static int stress_ipsec_mb(stress_args_t *args)
 		for (i = 0; i < SIZEOF_ARRAY(mb_features); i++) {
 			mb_features[i].supported = (i == ipsec_mb_feature);
 		}
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: using just feature '%s'\n", args->name, feature_name);
 	}
 
@@ -881,7 +889,7 @@ const stressor_info_t stress_ipsec_mb_info = {
 	.stressor = stress_ipsec_mb,
 	.supported = stress_ipsec_mb_supported,
 	.opts = opts,
-	.class = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
 	.help = help
 };
 #else
@@ -896,7 +904,7 @@ static int stress_ipsec_mb_supported(const char *name)
 
 static const stress_opt_t opts[] = {
 	{ OPT_ipsec_mb_feature, "ipsec-mb-feature", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
-	{ OPT_ipsec_mb_jobs,    "ipsec-mb-jobs",    TYPE_ID_INT, 1, 65536, NULL },
+	{ OPT_ipsec_mb_jobs,    "ipsec-mb-jobs",    TYPE_ID_INT, MIN_IPSEC_MB_JOBS, MAX_IPSEC_MB_JOBS, NULL },
 	{ OPT_ipsec_mb_method,  "ipsec-mb-method",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
 	END_OPT,
 };
@@ -905,7 +913,7 @@ const stressor_info_t stress_ipsec_mb_info = {
 	.stressor = stress_unimplemented,
 	.supported = stress_ipsec_mb_supported,
 	.opts = opts,
-	.class = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE,
 	.help = help,
 	.unimplemented_reason = "built on non-x86-64 without IPSec MB library"
 };
