@@ -22,11 +22,15 @@
 #include "core-capabilities.h"
 #include "core-cpu.h"
 #include "core-cpu-cache.h"
+#include "core-mmap.h"
 #include "core-numa.h"
 #include "core-target-clones.h"
 
 #include <sched.h>
 
+#define MIN_LLC_SIZE	(4 * KB)
+#define MAX_LLC_SIZE	(4 * GB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"llc-affinity N",	"start N workers exercising low level cache over all CPUs" },
 	{ NULL,	"llc-affinity-mlock",	"attempt to mlock pages into memory" },
@@ -39,6 +43,7 @@ static const stress_opt_t opts[] = {
 	{ OPT_llc_affinity_clflush, "llc-affinity-clflush", TYPE_ID_BOOL, 0, 1, NULL },
 	{ OPT_llc_affinity_mlock,   "llc-affinity-mlock",   TYPE_ID_BOOL, 0, 1, NULL },
 	{ OPT_llc_affinity_numa,    "llc-affinity-numa",    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_llc_affinity_size,    "llc-affinity-size",    TYPE_ID_SIZE_T_BYTES_VM, MIN_LLC_SIZE, MAX_LLC_SIZE, NULL },
 	END_OPT,
 };
 
@@ -78,7 +83,8 @@ static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_64(
 	*duration += (t2 - t1);
 }
 
-#if defined(HAVE_ASM_PPC64_DCBST)
+#if defined(STRESS_ARCH_PPC64) &&	\
+    defined(HAVE_ASM_PPC64_DCBST)
 static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_64_ppc64_dcbst(
 	uint64_t *buf,
 	const uint64_t *buf_end,
@@ -109,6 +115,38 @@ static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_64_ppc64_dcbst(
 }
 #endif
 
+#if defined(STRESS_ARCH_PPC) &&	\
+    defined(HAVE_ASM_PPC_DCBST)
+static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_64_ppc_dcbst(
+	uint64_t *buf,
+	const uint64_t *buf_end,
+	double *duration,
+	const size_t cache_line_size)
+{
+	double t1, t2;
+	static uint64_t val = 0;
+	register uint64_t *ptr;
+
+	(void)cache_line_size;
+
+	t1 = stress_time_now();
+	for (ptr = buf; ptr < buf_end; ptr += 8, val++) {
+		*(ptr + 0) = val;
+		*(ptr + 1) = val;
+		*(ptr + 2) = val;
+		*(ptr + 3) = val;
+		*(ptr + 4) = val;
+		*(ptr + 5) = val;
+		*(ptr + 6) = val;
+		*(ptr + 7) = val;
+		stress_asm_ppc_dcbst((void *)ptr);
+	}
+	t2 = stress_time_now();
+
+	*duration += (t2 - t1);
+}
+#endif
+
 #if defined(HAVE_ASM_X86_CLFLUSH)
 static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_64_x86_clfsh(
 	uint64_t *buf,
@@ -194,7 +232,8 @@ static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_n(
 	*duration += (t2 - t1);
 }
 
-#if defined(HAVE_ASM_PPC64_DCBST)
+#if defined(STRESS_ARCH_PPC64) &&	\
+    defined(HAVE_ASM_PPC64_DCBST)
 static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_n_ppc64_dcbst(
 	uint64_t *buf,
 	const uint64_t *buf_end,
@@ -220,6 +259,33 @@ static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_n_ppc64_dcbst(
 }
 #endif
 
+#if defined(STRESS_ARCH_PPC) &&	\
+    defined(HAVE_ASM_PPC_DCBST)
+static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_n_ppc_dcbst(
+	uint64_t *buf,
+	const uint64_t *buf_end,
+	double *duration,
+	const size_t cache_line_size)
+{
+	double t1, t2;
+	static uint64_t val = 0;
+	register uint64_t *ptr;
+	const size_t n = cache_line_size / sizeof(uint64_t);
+
+	t1 = stress_time_now();
+	for (ptr = buf; ptr < buf_end; ptr += n, val++) {
+		register uint64_t *cptr, *cptr_end;
+
+		for (cptr = ptr, cptr_end = ptr + n; cptr < cptr_end; cptr++)
+			*cptr = val;
+		stress_asm_ppc_dcbst((void *)ptr);
+	}
+	t2 = stress_time_now();
+
+	*duration += (t2 - t1);
+}
+#endif
+
 #if defined(HAVE_ASM_X86_CLFLUSH)
 static void TARGET_CLONES OPTIMIZE3 stress_llc_write_cache_line_n_x86_clfsh(
 	uint64_t *buf,
@@ -332,7 +398,7 @@ static int stress_llc_affinity(stress_args_t *args)
 	const uint32_t n_cpus = stress_get_usable_cpus(&cpus, true);
 	const size_t page_size = args->page_size;
 	uint32_t cpu_idx = args->instance;
-	size_t llc_size, cache_line_size, mmap_sz;
+	size_t llc_affinity_size = 0, cache_line_size = 64, mmap_sz;
 	uint64_t *buf, *buf_end;
 	uint64_t affinity_changes = 0;
 	double write_duration, read_duration, rate, writes, reads, t_start, duration;
@@ -341,9 +407,10 @@ static int stress_llc_affinity(stress_args_t *args)
 	bool llc_affinity_clflush = false;
 	bool llc_affinity_numa = false;
 	char *clflush_op = NULL;
-	const int numa_nodes = stress_numa_nodes();
+	const int n_numa_nodes = stress_numa_nodes();
 #if defined(HAVE_LINUX_MEMPOLICY_H)
         stress_numa_mask_t *numa_mask = NULL;
+        stress_numa_mask_t *numa_nodes = NULL;
 #endif
 
 	stress_catch_sigill();
@@ -351,26 +418,31 @@ static int stress_llc_affinity(stress_args_t *args)
 	(void)stress_get_setting("llc-affinity-clflush", &llc_affinity_clflush);
 	(void)stress_get_setting("llc-affinity-mlock", &llc_affinity_mlock);
 	(void)stress_get_setting("llc-affinity-numa", &llc_affinity_numa);
+	(void)stress_get_setting("llc-affinity-size", &llc_affinity_size);
 
-	stress_cpu_cache_get_llc_size(&llc_size, &cache_line_size);
-	if (llc_size == 0) {
-		pr_inf_skip("%s: cannot determine cache details, skipping stressor\n",
-			args->name);
-		stress_free_usable_cpus(&cpus);
-		return EXIT_NO_RESOURCE;
+	if (llc_affinity_size == 0) {
+		stress_cpu_cache_get_llc_size(&llc_affinity_size, &cache_line_size);
+		if (llc_affinity_size == 0) {
+			pr_inf_skip("%s: cannot determine cache details, skipping stressor\n",
+				args->name);
+			pr_inf("%s: alternatively use --llc-affinity-size to specify LLC cache size\n",
+				args->name);
+			stress_free_usable_cpus(&cpus);
+			return EXIT_NO_RESOURCE;
+		}
 	}
-        llc_size *= numa_nodes;
+        llc_affinity_size *= n_numa_nodes;
         if (!args->instance) {
-		if (numa_nodes > 1)  {
-			pr_inf("%s: scaling LLC cache size by number of numa nodes %d to %zdK\n",
-				args->name, numa_nodes, llc_size / 1024);
+		if (n_numa_nodes > 1)  {
+			pr_inf("%s: scaling lower level cache size by number of numa nodes %d to %zdK\n",
+				args->name, n_numa_nodes, llc_affinity_size / 1024);
 		} else  {
-			pr_inf("%s: using LLC cache size of %zuK\n",
-				args->name, llc_size / 1024);
+			pr_inf("%s: using lower level cache size of %zuK\n",
+				args->name, llc_affinity_size / 1024);
 		}
 	}
 
-	mmap_sz = STRESS_MAXIMUM(n_cpus * page_size, llc_size);
+	mmap_sz = STRESS_MAXIMUM(n_cpus * page_size, llc_affinity_size);
 
 	/*
 	 *  Allocate a LLC sized buffer to exercise
@@ -378,7 +450,9 @@ static int stress_llc_affinity(stress_args_t *args)
 	buf = (uint64_t *)stress_mmap_populate(NULL, mmap_sz, PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
-		pr_fail("%s: mmap'd region of %zu bytes failed\n", args->name, mmap_sz);
+		pr_inf_skip("%s: failed to mmap region of %zu bytes%s, errno=%d (%s), "
+			"skipping stressor\n", args->name, mmap_sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		stress_free_usable_cpus(&cpus);
 		return EXIT_NO_RESOURCE;
 	}
@@ -386,31 +460,20 @@ static int stress_llc_affinity(stress_args_t *args)
 
 	if (llc_affinity_numa) {
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-		if (stress_numa_nodes() > 1) {
-			numa_mask = stress_numa_mask_alloc();
-			if (!numa_mask) {
-				pr_inf("%s: cannot allocate NUMA mask, disabling --llc-affinity-numa\n",
-					args->name);
-				llc_affinity_numa = false;
-			}
-		} else {
-			if (args->instance == 0) {
-				pr_inf("%s: only 1 NUMA node available, disabling --llc-affinity-numa\n",
-					args->name);
-				llc_affinity_numa = false;
-			}
-		}
+		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask, "--llc-affinity-numa", &llc_affinity_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --llc-affinity-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		llc_affinity_numa = false;
 #endif
 	}
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-	if (llc_affinity_numa && numa_mask) {
-		stress_numa_randomize_pages(numa_mask, buf, page_size, mmap_sz);
+	if (llc_affinity_numa && numa_mask && numa_nodes) {
+		if (stress_numa_mask_nodes_get(numa_nodes) > 0)
+			stress_numa_randomize_pages(args, numa_nodes, numa_mask, buf, mmap_sz, page_size);
 		stress_numa_mask_free(numa_mask);
+		stress_numa_mask_free(numa_nodes);
 	}
 #endif
 	if (llc_affinity_mlock)
@@ -427,12 +490,20 @@ static int stress_llc_affinity(stress_args_t *args)
 	read_duration = 0.0;
 
 	if (cache_line_size == 64) {
-#if defined(HAVE_ASM_PPC64_DCBST)
+#if defined(STRESS_ARCH_PPC64) && 	\
+    defined(HAVE_ASM_PPC64_DCBST)
 		if (llc_affinity_clflush) {
 			write_func = stress_llc_write_cache_line_64_ppc64_dcbst;
 			clflush_op = "dcbst";
 		} else
 #endif
+#if defined(STRESS_ARCH_PPC) &&	\
+    defined(HAVE_ASM_PPC_DCBST)
+		if (llc_affinity_clflush) {
+			write_func = stress_llc_write_cache_line_64_ppc_dcbst;
+			clflush_op = "dcbst";
+		} else
+#endif
 #if defined(HAVE_ASM_X86_CLFLUSHOPT)
 		if (llc_affinity_clflush && stress_cpu_x86_has_clflushopt()) {
 			write_func = stress_llc_write_cache_line_64_x86_clfshopt;
@@ -448,12 +519,20 @@ static int stress_llc_affinity(stress_args_t *args)
 		write_func = stress_llc_write_cache_line_64;
 		read_func = stress_llc_read_cache_line_64;
 	} else {
-#if defined(HAVE_ASM_PPC64_DCBST)
+#if defined(STRESS_ARCH_PPC64) && 	\
+    defined(HAVE_ASM_PPC64_DCBST)
 		if (llc_affinity_clflush) {
 			write_func = stress_llc_write_cache_line_n_ppc64_dcbst;
 			clflush_op = "dcbst";
 		} else
 #endif
+#if defined(STRESS_ARCH_PPC) &&	\
+    defined(HAVE_ASM_PPC_DCBST)
+		if (llc_affinity_clflush) {
+			write_func = stress_llc_write_cache_line_n_ppc_dcbst;
+			clflush_op = "dcbst";
+		} else
+#endif
 #if defined(HAVE_ASM_X86_CLFLUSHOPT)
 		if (llc_affinity_clflush && stress_cpu_x86_has_clflushopt()) {
 			write_func = stress_llc_write_cache_line_n_x86_clfshopt;
@@ -470,7 +549,7 @@ static int stress_llc_affinity(stress_args_t *args)
 		read_func = stress_llc_read_cache_line_n;
 	}
 
-	if (clflush_op && (args->instance == 0))
+	if (clflush_op && (stress_instance_zero(args)))
 		pr_inf("%s: using %s cache flushing op-code\n", args->name, clflush_op);
 
 	t_start = stress_time_now();
@@ -521,7 +600,7 @@ static int stress_llc_affinity(stress_args_t *args)
 
 const stressor_info_t stress_llc_affinity_info = {
 	.stressor = stress_llc_affinity,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -531,7 +610,7 @@ const stressor_info_t stress_llc_affinity_info = {
 
 const stressor_info_t stress_llc_affinity_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
