@@ -197,7 +197,8 @@ static int stress_locka_contention(
 
 		locka_info = stress_locka_info_new();
 		if (UNLIKELY(!locka_info)) {
-			pr_err("%s: calloc failed, out of memory\n", args->name);
+			pr_err("%s: calloc failed, out of memory%s\n",
+				args->name, stress_get_memfree_str());
 			return -1;
 		}
 		locka_info->offset = offset;
@@ -321,14 +322,14 @@ tidy:
 }
 const stressor_info_t stress_locka_info = {
 	.stressor = stress_locka,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_locka_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without fcntl() F_GETLK, F_SETLK, F_SETLKW, F_WRLCK or F_UNLCK commands"
