@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 
 #include <sys/ioctl.h>
 
@@ -287,7 +288,7 @@ static int stress_loop(stress_args_t *args)
 		 *  Sync is required to avoid loop_set_block_size
 		 *  warning messages
 		 */
-		(void)sync();
+		shim_sync();
 
 #if defined(LOOP_SET_BLOCK_SIZE)
 		/*
