@@ -17,6 +17,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-mmap.h"
 
 #if defined(HAVE_LINUX_LSM_H)
 #include <linux/lsm.h>
@@ -98,7 +100,10 @@ static int stress_lsm(stress_args_t *args)
 
 	buf = stress_mmap_populate(NULL, buf_size, PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zu byte sized buffer, skipping stressor\n", args->name, buf_size);
+		pr_inf_skip("%s: cannot mmap %zu byte sized buffer%s, errno=%d (%s),"
+			"skipping stressor\n",
+			args->name, buf_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(buf, buf_size, "lsm-data");
@@ -153,6 +158,7 @@ static int stress_lsm(stress_args_t *args)
 		for (j = 0; j < SIZEOF_ARRAY(attr); j++) {
 			struct lsm_ctx *ctx = (struct lsm_ctx *)buf;
 			struct lsm_ctx *ctx_end = (struct lsm_ctx *)((uintptr_t)buf + buf_size);
+			struct lsm_ctx tmp_ctx ALIGNED(8);
 
 			size = buf_size;
 			t = stress_time_now();
@@ -206,11 +212,13 @@ static int stress_lsm(stress_args_t *args)
 			 *  exercise invalid ctx_len, see Linux commits
 			 *  a04a1198088a and d8bdd795d383
 			 */
-			ctx->id = LSM_ID_APPARMOR;
-			ctx->flags = 0;
-			ctx->len = sizeof(*ctx);
-			ctx->ctx_len = -sizeof(*ctx);
-			VOID_RET(int, shim_lsm_set_self_attr(LSM_ATTR_CURRENT, ctx, sizeof(*ctx), 0));
+			shim_memcpy(&tmp_ctx, ctx, sizeof(tmp_ctx));
+
+			tmp_ctx.id = LSM_ID_APPARMOR;
+			tmp_ctx.flags = 0;
+			tmp_ctx.len = sizeof(tmp_ctx);
+			tmp_ctx.ctx_len = -sizeof(tmp_ctx);
+			VOID_RET(int, shim_lsm_set_self_attr(LSM_ATTR_CURRENT, &tmp_ctx, sizeof(tmp_ctx), 0));
 		}
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
@@ -237,13 +245,13 @@ err:
 
 const stressor_info_t stress_lsm_info = {
 	.stressor = stress_lsm,
-	.class = CLASS_OS | CLASS_SECURITY,
+	.classifier = CLASS_OS | CLASS_SECURITY,
 	.help = help
 };
 #else
 const stressor_info_t stress_lsm_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS | CLASS_SECURITY,
+	.classifier = CLASS_OS | CLASS_SECURITY,
 	.help = help,
 	.unimplemented_reason = "built without linux/lsm.h or lsm_list_modules or lsm_get_self_attr system calls"
 };
