@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-madvise.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
 
@@ -206,6 +207,7 @@ static void NORETURN MLOCKED_TEXT stress_sigbus_handler(int signum)
 	sigbus_count++;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 #if defined(MADV_FREE)
@@ -390,6 +392,16 @@ static void *stress_madvise_pages(void *arg)
 #endif
 		(void)shim_msync(ptr, page_size, MS_ASYNC);
 	}
+#if defined(MADV_PAGEOUT) && defined(MS_SYNC)
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE) {
+		(void)shim_madvise(buf, sz, MADV_PAGEOUT);
+		(void)shim_msync(buf, sz, MS_SYNC);
+#if defined(MADV_POPULATE_WRITE)
+		(void)shim_madvise(buf, sz, MADV_POPULATE_WRITE);
+		(void)shim_msync(buf, sz, MS_SYNC);
+#endif
+	}
+#endif
 
 	/*
 	 *  Exercise a highly likely bad advice option
@@ -517,8 +529,9 @@ static int stress_madvise(stress_args_t *args)
 	page = (char *)stress_mmap_populate(NULL, page_size, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate %zd byte page, skipping stressor\n",
-			args->name, page_size);
+		pr_inf_skip("%s: failed to mmap %zu byte page%s, errno=%d (%s), skipping stressor\n",
+			args->name, page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(page, page_size, "data-page");
@@ -609,7 +622,7 @@ static int stress_madvise(stress_args_t *args)
 		}
 
 		(void)shim_memset(buf, 0xff, sz);
-		(void)stress_madvise_random(buf, sz);
+		(void)stress_madvise_randomize(buf, sz);
 		(void)stress_mincore_touch_pages(buf, sz);
 		stress_process_madvise(pid, buf, sz);
 
@@ -732,14 +745,14 @@ madv_free_out:
 
 const stressor_info_t stress_madvise_info = {
 	.stressor = stress_madvise,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
 #else
 const stressor_info_t stress_madvise_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
