@@ -19,6 +19,8 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 #include "core-put.h"
 #include "core-target-clones.h"
@@ -30,7 +32,7 @@
 static const stress_help_t help[] = {
 	{ NULL,	"matrix N",		"start N workers exercising matrix operations" },
 	{ NULL,	"matrix-method M",	"specify matrix stress method M, default is all" },
-	{ NULL,	"matrix-ops N",		"stop after N maxtrix bogo operations" },
+	{ NULL,	"matrix-ops N",		"stop after N matrix bogo operations" },
 	{ NULL,	"matrix-size N",	"specify the size of the N x N matrix" },
 	{ NULL,	"matrix-yx",		"matrix operation is y by x instead of x by y" },
 	{ NULL,	NULL,			NULL }
@@ -41,7 +43,7 @@ static const stress_help_t help[] = {
 typedef float	stress_matrix_type_t;
 
 /*
- *  the matrix stress test has different classes of maxtrix stressor
+ *  the matrix stress test has different classes of matrix stressor
  */
 typedef void (*stress_matrix_func_t)(
 	const size_t n,
@@ -866,34 +868,47 @@ static inline int stress_matrix_exercise(
 	a = (matrix_ptr_t)stress_mmap_populate(NULL, matrix_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (a == MAP_FAILED) {
-		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
+		pr_fail("%s: matrix allocation failed, out of memory%s, errno=%d (%s)\n",
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		goto tidy_ret;
 	}
+	(void)stress_madvise_collapse(a, matrix_mmap_size);
 	stress_set_vma_anon_name(a, matrix_mmap_size, "matrix-a");
 
 	b = (matrix_ptr_t)stress_mmap_populate(NULL, matrix_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (b == MAP_FAILED) {
-		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
+		pr_fail("%s: matrix allocation failed, out of memory%s, errno=%d (%s)\n",
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		goto tidy_a;
 	}
+	(void)stress_madvise_collapse(b, matrix_mmap_size);
 	stress_set_vma_anon_name(b, matrix_mmap_size, "matrix-b");
 
 	r = (matrix_ptr_t)stress_mmap_populate(NULL, matrix_mmap_size,
 		PROT_READ | PROT_WRITE, flags, -1, 0);
 	if (r == MAP_FAILED) {
-		pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
+		pr_fail("%s: matrix allocation failed, out of memory%s, errno=%d (%s)\n",
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		goto tidy_b;
 	}
+	(void)stress_madvise_collapse(r, matrix_mmap_size);
 	stress_set_vma_anon_name(r, matrix_mmap_size, "matrix-r");
 
 	if (verify) {
 		s = (matrix_ptr_t)stress_mmap_populate(NULL, matrix_mmap_size,
 			PROT_READ | PROT_WRITE, flags, -1, 0);
 		if (s == MAP_FAILED) {
-			pr_fail("%s: matrix allocation failed, out of memory\n", args->name);
+			pr_fail("%s: matrix allocation failed, out of memory%s, errno=%d (%s)\n",
+				args->name, stress_get_memfree_str(),
+				errno, strerror(errno));
 			goto tidy_r;
 		}
+		(void)stress_madvise_collapse(s, matrix_mmap_size);
+		stress_set_vma_anon_name(r, matrix_mmap_size, "matrix-s");
 	}
 
 	/*
@@ -983,7 +998,7 @@ static int stress_matrix(stress_args_t *args)
 	(void)stress_get_setting("matrix-method", &matrix_method);
 	(void)stress_get_setting("matrix-yx", &matrix_yx);
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: using method '%s' (%s)\n", args->name, matrix_methods[matrix_method].name,
 			matrix_yx ? "y by x" : "x by y");
 
@@ -1019,7 +1034,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_matrix_info = {
 	.stressor = stress_matrix,
-	.class = CLASS_CPU | CLASS_FP | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_FP | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -1036,7 +1051,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_matrix_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_FP | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_FP | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
