@@ -23,6 +23,7 @@
 #include "core-asm-x86.h"
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
+#include "core-mmap.h"
 #include "core-numa.h"
 #include "core-pthread.h"
 
@@ -72,7 +73,7 @@ static int page_write_sync(const int fd, const size_t page_size)
 			return (int)rc;
 		n += (size_t)rc;
 	}
-	(void)sync();
+	shim_sync();
 
 	return 0;
 }
@@ -511,8 +512,9 @@ static int stress_mcontend(stress_args_t *args)
 	data[0] = stress_mmap_populate(NULL, page_size, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE, fd, 0);
 	if (data[0] == MAP_FAILED) {
-		pr_inf("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf("%s: mmap of %zu bytes failed%s, errno=%d (%s)\n",
+			args->name, page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
 #if defined(HAVE_SCHED_SETAFFINITY)
@@ -523,8 +525,9 @@ static int stress_mcontend(stress_args_t *args)
 	data[1] = stress_mmap_populate(NULL, page_size , PROT_READ | PROT_WRITE,
 			MAP_PRIVATE, fd, 0);
 	if (data[1] == MAP_FAILED) {
-		pr_inf("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf("%s: mmap of %zu bytes failed%s, errno=%d (%s)\n",
+			args->name, page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		(void)munmap(data[0], page_size);
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
@@ -540,7 +543,7 @@ static int stress_mcontend(stress_args_t *args)
 		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask,
 						"--mcontend-numa", &mcontend_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --mcontend-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		mcontend_numa = false;
@@ -600,14 +603,14 @@ static int stress_mcontend(stress_args_t *args)
 
 const stressor_info_t stress_mcontend_info = {
 	.stressor = stress_mcontend,
-	.class = CLASS_MEMORY,
+	.classifier = CLASS_MEMORY,
 	.opts = opts,
 	.help = help
 };
 #else
 const stressor_info_t stress_mcontend_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_MEMORY,
+	.classifier = CLASS_MEMORY,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
