@@ -227,8 +227,10 @@ static void stress_segv_handler(int signum)
 
 	segv_count++;
 
-	if (do_jmp)
+	if (do_jmp) {
 		siglongjmp(jmp_env, 1);         /* Ugly, bounce back */
+		stress_no_return();
+	}
 }
 
 /*
@@ -256,7 +258,7 @@ static int stress_memhotplug(stress_args_t *args)
 
 	dir = opendir(sys_memory_path);
 	if (!dir) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: %s not accessible, skipping stressor\n",
 				args->name, sys_memory_path);
 		return EXIT_NOT_IMPLEMENTED;
@@ -269,7 +271,7 @@ static int stress_memhotplug(stress_args_t *args)
 			n++;
 	}
 	if (n == 0) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: no hotplug memory entries found, skipping stressor\n",
 				args->name);
 		(void)closedir(dir);
@@ -279,8 +281,8 @@ static int stress_memhotplug(stress_args_t *args)
 
 	mem_info = (stress_mem_info_t *)calloc(n, sizeof(*mem_info));
 	if (!mem_info) {
-		pr_inf_skip("%s: failed to allocate %zd mem_info structs, skipping stressor\n",
-			args->name, n);
+		pr_inf_skip("%s: failed to allocate %zu mem_info structs%s, skipping stressor\n",
+			args->name, n, stress_get_memfree_str());
 		(void)closedir(dir);
 		return EXIT_NO_RESOURCE;
 	}
@@ -289,14 +291,14 @@ static int stress_memhotplug(stress_args_t *args)
 	while ((max < n) && ((d = readdir(dir)) != NULL)) {
 		if ((strncmp(d->d_name, "memory", 6) == 0) &&
 		     stress_memhotplug_removable(d->d_name)) {
-			mem_info[max].name = strdup(d->d_name);
+			mem_info[max].name = shim_strdup(d->d_name);
 			mem_info[max].timeout = false;
 			max++;
 		}
 	}
 	(void)closedir(dir);
 
-	pr_dbg("%s: found %zd removable hotplug memory regions\n",
+	pr_dbg("%s: found %zu removable hotplug memory regions\n",
 		args->name, max);
 
 	metrics.offline_duration = 0.0;
@@ -362,7 +364,7 @@ finish:
 
 const stressor_info_t stress_memhotplug_info = {
 	.stressor = stress_memhotplug,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.supported = stress_memhotplug_supported,
 	.help = help
@@ -370,7 +372,7 @@ const stressor_info_t stress_memhotplug_info = {
 #else
 const stressor_info_t stress_memhotplug_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
