@@ -23,12 +23,14 @@
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-nt-load.h"
 #include "core-nt-store.h"
 #include "core-numa.h"
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
 #include "core-pragma.h"
+#include "core-prime.h"
 #include "core-target-clones.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H) &&	\
@@ -876,8 +878,8 @@ static int stress_memthrash_child(stress_args_t *args, void *ctxt)
 
 	pthread_info = (stress_pthread_info_t *)calloc(max_threads, sizeof(*pthread_info));
 	if (!pthread_info) {
-		pr_inf_skip("%s: failed to allocate pthread information array, skipping stressor\n",
-			args->name);
+		pr_inf_skip("%s: failed to allocate pthread information array%s, skipping stressor\n",
+			args->name, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -889,8 +891,10 @@ mmap_retry:
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (mem == MAP_FAILED) {
 		if (UNLIKELY(!stress_continue_flag())) {
-			pr_dbg("%s: mmap failed: %d %s\n",
-				args->name, errno, strerror(errno));
+			pr_dbg("%s: mmap of %zu bytes failed%s, errno=%d (%s)\n",
+				args->name, (size_t)MEM_SIZE,
+				stress_get_memfree_str(),
+				errno, strerror(errno));
 			free(pthread_info);
 			return EXIT_NO_RESOURCE;
 		}
@@ -901,6 +905,7 @@ mmap_retry:
 	}
 	stress_set_vma_anon_name(mem, MEM_SIZE, "memthrash-data");
 	(void)stress_madvise_mergeable(mem, MEM_SIZE);
+	(void)stress_madvise_nohugepage(mem, MEM_SIZE);
 
 	for (i = 0; i < max_threads; i++) {
 		pthread_info[i].ret = pthread_create(&pthread_info[i].pthread,
@@ -921,7 +926,7 @@ mmap_retry:
 			goto reap;
 	}
 	/* Wait for SIGALRM or SIGINT/SIGHUP etc */
-	(void)pause();
+	(void)shim_pause();
 
 reap:
 	thread_terminate = true;
@@ -959,12 +964,12 @@ static int stress_memthrash(stress_args_t *args)
 
 	context.args = args;
 	context.total_cpus = (uint32_t)stress_get_processors_online();
-	context.max_threads = stress_memthrash_max(args->num_instances, context.total_cpus);
+	context.max_threads = stress_memthrash_max(args->instances, context.total_cpus);
 #if defined(HAVE_MEMTHRASH_NUMA)
 	{
 		context.numa_mask = stress_numa_mask_alloc();
 		if (!context.numa_mask) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf("%s: no NUMA nodes or maximum NUMA nodes, ignoring numa memthrash method\n", args->name);
 		}
 	}
@@ -973,22 +978,25 @@ static int stress_memthrash(stress_args_t *args)
 	(void)stress_get_setting("memthrash-method", &memthrash_method);
 	context.memthrash_method = &memthrash_methods[memthrash_method];
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("%s: using method '%s'\n", args->name, context.memthrash_method->name);
 		pr_inf("%s: starting %" PRIu32 " thread%s on each of the %"
 			PRIu32 " stressors on a %" PRIu32 " CPU system\n",
 			args->name, context.max_threads, plural(context.max_threads),
-			args->num_instances, context.total_cpus);
-		if (context.max_threads * args->num_instances > context.total_cpus) {
+			args->instances, context.total_cpus);
+		if (context.max_threads * args->instances > context.total_cpus) {
 			pr_inf("%s: this is not an optimal choice of stressors, "
 				"try %" PRIu32 " instead\n",
 			args->name,
-			stress_memthash_optimal(args->num_instances, context.total_cpus));
+			stress_memthash_optimal(args->instances, context.total_cpus));
 		}
 	}
 
 	(void)sigfillset(&set);
 
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, MEM_SIZE, MEM_SIZE * args->instances);
+
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -1016,7 +1024,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_memthrash,
-	.class = CLASS_MEMORY,
+	.classifier = CLASS_MEMORY,
 	.opts = opts,
 	.help = help
 };
@@ -1029,7 +1037,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_memthrash_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_MEMORY,
+	.classifier = CLASS_MEMORY,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
