@@ -23,9 +23,10 @@
 #include "core-builtin.h"
 #include "core-cpu.h"
 #include "core-madvise.h"
-#include "core-target-clones.h"
-#include "core-numa.h"
+#include "core-mmap.h"
 #include "core-nt-store.h"
+#include "core-numa.h"
+#include "core-target-clones.h"
 
 #include <time.h>
 
@@ -91,6 +92,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
 static sigjmp_buf jmp_env;
 static int handled_signum = -1;
 #if defined(HAVE_TIMER_FUNCTIONALITY)
@@ -1204,6 +1207,7 @@ static MLOCKED_TEXT NORETURN void stress_misaligned_handler(int signum)
 	handled_signum = signum;
 	stress_misligned_disable();
 	siglongjmp(jmp_env, STRESS_MISALIGNED_ERROR);
+	stress_no_return();
 }
 
 #if defined(HAVE_TIMER_FUNCTIONALITY)
@@ -1225,6 +1229,7 @@ static MLOCKED_TEXT void stress_misaligned_timer_handler(int signum)
 
 	stress_misaligned_reset_timer();
 	siglongjmp(jmp_env, STRESS_MISALIGNED_TIMED_OUT);
+	stress_no_return();
 }
 #endif
 
@@ -1471,3 +1476,28 @@ const stressor_info_t stress_misaligned_info = {
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+static const char *stress_misaligned_method(const size_t i)
+{
+	(void)i;
+
+	return NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_misaligned_method, "misaligned-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_misaligned_method },
+	END_OPT,
+};
+
+const stressor_info_t stress_misaligned_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_CPU_CACHE | CLASS_MEMORY,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
