@@ -32,7 +32,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if defined(__linux__)
+#if defined(__linux__) &&	\
+    defined(HAVE_MKNOD)
 
 typedef struct {
 	const mode_t	mode;
@@ -274,7 +275,7 @@ static int stress_mknod(stress_args_t *args)
 		stress_mknod_tidy(args, i);
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		(void)sync();
+		shim_sync();
 	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -289,14 +290,14 @@ static int stress_mknod(stress_args_t *args)
 
 const stressor_info_t stress_mknod_info = {
 	.stressor = stress_mknod,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_mknod_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
