@@ -59,6 +59,42 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		     NULL }
 };
 
+static void stress_mmap_stressful(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
+{
+	(void)opt_name;
+	*type_id = TYPE_ID_SIZE_T;
+	*(size_t *)value = 0;
+
+	(void)stress_set_setting_true("mmap", "mmap-mergeable", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-mprotect", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-file", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-odirect", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-madvise", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-mlock", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-numa", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-slow-munmap", opt_arg);
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_mmap_async,       "mmap-async",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_bytes,       "mmap-bytes",       TYPE_ID_SIZE_T_BYTES_VM, MIN_MMAP_BYTES, MAX_MMAP_BYTES, NULL },
+	{ OPT_mmap_file,        "mmap-file",        TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_madvise,     "mmap-madvise",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mergeable,   "mmap-mergeable",   TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mlock,       "mmap-mlock",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mmap2,       "mmap-mmap2",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mprotect,    "mmap-mprotect",    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_numa,	"mmap-numa",	    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_odirect,     "mmap-odirect",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_osync,       "mmap-osync",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_slow_munmap,	"mmap-slow-munmap", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_stressful,   "mmap-stressful",   TYPE_ID_CALLBACK, 0, 0, stress_mmap_stressful },
+	{ OPT_mmap_write_check, "mmap-write-check", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
+};
+
+#if defined(HAVE_SIGLONGJMP)
+
 typedef void * (*mmap_func_t)(void *addr, size_t length, int prot, int flags, int fd, off_t offset);
 
 typedef struct {
@@ -160,7 +196,8 @@ static const int mmap_flags[] = {
 #if defined(MAP_NORESERVE)
 	MAP_NORESERVE,
 #endif
-#if defined(MAP_STACK)
+#if defined(MAP_STACK) &&	\
+    !defined(__FreeBSD__)
 	MAP_STACK,
 #endif
 #if defined(MAP_EXECUTABLE)
@@ -195,6 +232,10 @@ static const int mmap_flags[] = {
 #if defined(MAP_PREFAULT_READ)
 	MAP_PREFAULT_READ,
 #endif
+/* FreeBSD */
+#if defined(MAP_ALIGNED_SUPER)
+	MAP_ALIGNED_SUPER,
+#endif
 /* NetBSD */
 #if defined(MAP_WIRED)
 	MAP_WIRED,
@@ -226,8 +267,10 @@ static void MLOCKED_TEXT stress_mmap_sighandler(int signum)
 {
 	(void)signum;
 
-	if (jmp_env_set)
+	if (jmp_env_set) {
 		siglongjmp(jmp_env, 1);
+		stress_no_return();
+	}
 }
 
 /*
@@ -457,7 +500,10 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 	const int fd = context->fd;
 	NOCLOBBER int no_mem_retries = 0;
 	const int bad_fd = stress_get_bad_fd();
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 	const int ms_flags = context->mmap_async ? MS_ASYNC : MS_SYNC;
+#endif
 	uint8_t *mapped, **mappings;
 	size_t *idx;
 	void *hint;
@@ -482,8 +528,10 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (mappings == MAP_FAILED) {
-		pr_dbg("%s: cannot allocate mappings buffer, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_dbg("%s: cannot allocate %zu byte mappings buffer%s, errno=%d (%s)\n",
+			args->name, pages * sizeof(*mappings),
+			stress_get_memfree_str(),
+			errno, strerror(errno));
 		(void)munmap((void *)mapped, pages * sizeof(*mapped));
 		return EXIT_NO_RESOURCE;
 	}
@@ -495,8 +543,9 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (idx == MAP_FAILED) {
-		pr_dbg("%s: cannot allocate idx buffer, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_dbg("%s: cannot allocate %zu byte idx buffer%s, errno=%d (%s)\n",
+			args->name, pages * sizeof(*idx),
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)mappings, pages * sizeof(*mappings));
 		(void)munmap((void *)mapped, pages * sizeof(*mapped));
 		return EXIT_NO_RESOURCE;
@@ -594,13 +643,16 @@ retry:
 		no_mem_retries = 0;
 		if (mmap_file) {
 			(void)shim_memset(buf, 0xff, sz);
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 			(void)shim_msync((void *)buf, sz, ms_flags);
+#endif
 		}
 		if (context->mmap_madvise)
-			(void)stress_madvise_random(buf, sz);
+			(void)stress_madvise_randomize(buf, sz);
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 		if (context->mmap_numa)
-			stress_numa_randomize_pages(args, context->numa_nodes, context->numa_mask, buf, page_size, sz);
+			stress_numa_randomize_pages(args, context->numa_nodes, context->numa_mask, buf, sz, page_size);
 #endif
 		if (context->mmap_mergeable)
 			(void)stress_madvise_mergeable(buf, sz);
@@ -661,7 +713,7 @@ retry:
 				}
 #endif
 				if (context->mmap_madvise)
-					(void)stress_madvise_random(mappings[page], page_size);
+					(void)stress_madvise_randomize(mappings[page], page_size);
 				stress_mmap_mprotect(args->name, mappings[page],
 					page_size, page_size, context->mmap_mprotect);
 			}
@@ -711,7 +763,7 @@ retry:
 						(void)shim_mlock(mappings[page], page_size);
 					(void)stress_mincore_touch_pages(mappings[page], page_size);
 					if (context->mmap_madvise)
-						(void)stress_madvise_random(mappings[page], page_size);
+						(void)stress_madvise_randomize(mappings[page], page_size);
 					if (context->mmap_mergeable)
 						(void)stress_madvise_mergeable(mappings[page], page_size);
 					stress_mmap_mprotect(args->name, mappings[page],
@@ -728,7 +780,10 @@ retry:
 					}
 					if (mmap_file) {
 						(void)shim_memset(mappings[page], (int)n, page_size);
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 						(void)shim_msync((void *)mappings[page], page_size, ms_flags);
+#endif
 #if defined(FALLOC_FL_KEEP_SIZE) &&	\
     defined(FALLOC_FL_PUNCH_HOLE)
 						(void)shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
@@ -915,7 +970,7 @@ static int stress_mmap(stress_args_t *args)
 	bool mmap_mmap2 = false;
 	int ret, all_flags;
 	stress_mmap_context_t context;
-	size_t i;
+	size_t i, mmap_total;
 
 	jmp_env_set = false;
 
@@ -929,6 +984,8 @@ static int stress_mmap(stress_args_t *args)
 	context.mmap_mlock = false;
 	context.mmap_mprotect = false;
 	context.mmap_numa = false;
+	context.mmap_slow_munmap = false;
+	context.mmap_write_check = false;
 	context.flags = MAP_PRIVATE | MAP_ANONYMOUS;
 #if defined(MAP_POPULATE)
 	context.flags |= MAP_POPULATE;
@@ -970,24 +1027,31 @@ static int stress_mmap(stress_args_t *args)
     defined(__NR_mmap2)
 		context.mmap = (mmap_func_t)mmap2_try;
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: using mmap instead of mmap2 as it is not available\n",
 				args->name);
 #endif
 	}
 
-	if (!stress_get_setting("mmap-bytes", &context.mmap_bytes)) {
+	mmap_total = DEFAULT_MMAP_BYTES;
+	if (!stress_get_setting("mmap-bytes", &mmap_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			context.mmap_bytes = MAX_32;
+			mmap_total = MAX_32;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			context.mmap_bytes = MIN_MMAP_BYTES;
+			mmap_total = MIN_MMAP_BYTES;
 	}
-	context.mmap_bytes /= args->instances;
-	if (context.mmap_bytes < MIN_MMAP_BYTES)
+	context.mmap_bytes = mmap_total / args->instances;
+	if (context.mmap_bytes < MIN_MMAP_BYTES) {
 		context.mmap_bytes = MIN_MMAP_BYTES;
-	if (context.mmap_bytes < page_size)
+		mmap_total = context.mmap_bytes * args->instances;
+	}
+	if (context.mmap_bytes < page_size) {
 		context.mmap_bytes = page_size;
+		mmap_total = context.mmap_bytes * args->instances;
+	}
 	context.sz = context.mmap_bytes & ~(page_size - 1);
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, context.mmap_bytes, mmap_total);
 
 	if (context.mmap_file) {
 		int file_flags = O_CREAT | O_RDWR;
@@ -1066,7 +1130,7 @@ redo:
 						&context.numa_mask, "--mmap-numa",
 						&context.mmap_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --mmap-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		context.mmap_numa = false;
@@ -1099,44 +1163,23 @@ redo:
 	return ret;
 }
 
-static void stress_mmap_stressful(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
-{
-	(void)opt_name;
-	*type_id = TYPE_ID_SIZE_T;
-	*(size_t *)value = 0;
-
-	(void)stress_set_setting_true("mmap", "mmap-mergeable", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-mprotect", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-file", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-odirect", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-madvise", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-mlock", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-numa", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-slow-munmap", opt_arg);
-}
-
-static const stress_opt_t opts[] = {
-	{ OPT_mmap_async,       "mmap-async",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_bytes,       "mmap-bytes",       TYPE_ID_SIZE_T_BYTES_VM, MIN_MMAP_BYTES, MAX_MMAP_BYTES, NULL },
-	{ OPT_mmap_file,        "mmap-file",        TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_madvise,     "mmap-madvise",     TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mergeable,   "mmap-mergeable",   TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mlock,       "mmap-mlock",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mmap2,       "mmap-mmap2",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mprotect,    "mmap-mprotect",    TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_numa,	"mmap-numa",	    TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_odirect,     "mmap-odirect",     TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_osync,       "mmap-osync",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_slow_munmap,	"mmap-slow-munmap", TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_stressful,   "mmap-stressful",   TYPE_ID_CALLBACK, 0, 0, stress_mmap_stressful },
-	{ OPT_mmap_write_check, "mmap-write-check", TYPE_ID_BOOL, 0, 1, NULL },
-	END_OPT,
-};
-
 const stressor_info_t stress_mmap_info = {
 	.stressor = stress_mmap,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_mmap_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_VM | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
