@@ -171,7 +171,7 @@ static int stress_mmapfixed_child(stress_args_t *args, void *context)
 #endif
 		if (info->mmapfixed_mlock)
 			(void)shim_mlock(buf, sz);
-		(void)stress_madvise_random(buf, sz);
+		(void)stress_madvise_randomize(buf, sz);
 #if defined(HAVE_MREMAP) &&	\
     NEED_GLIBC(2,4,0) && 	\
     defined(MREMAP_FIXED) &&	\
@@ -202,7 +202,7 @@ static int stress_mmapfixed_child(stress_args_t *args, void *context)
 #endif
 			if (info->mmapfixed_mlock)
 				(void)shim_mlock(buf, sz);
-			(void)stress_madvise_random(buf, sz);
+			(void)stress_madvise_randomize(buf, sz);
 
 			for (mask = ~(uintptr_t)0; mask > page_size; mask >>= 1) {
 				const uintptr_t rndaddr = rndaddr_base & mask;
@@ -239,7 +239,7 @@ static int stress_mmapfixed_child(stress_args_t *args, void *context)
 #endif
 					if (info->mmapfixed_mlock)
 						(void)shim_mlock(buf, sz);
-					(void)stress_madvise_random(buf, sz);
+					(void)stress_madvise_randomize(buf, sz);
 				}
 			}
 		}
@@ -283,7 +283,7 @@ static int stress_mmapfixed(stress_args_t *args)
 						&info.numa_mask, "--mmapfixed-numa",
 						&info.mmapfixed_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --mmapfixed-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		info.mmapfixed_numa = false;
@@ -309,7 +309,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_mmapfixed_info = {
 	.stressor = stress_mmapfixed,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
