@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 
 #define MIN_MMAPFORK_BYTES     (4 * KB)
 #define MAX_MMAPFORK_BYTES     (MAX_MEM_LIMIT)
@@ -120,6 +121,7 @@ static int stress_mmapfork(stress_args_t *args)
 	const size_t wipe_size = args->page_size;
 	bool wipe_ok = false;
 #endif
+	bool report_size = (stress_instance_zero(args));
 
 #if defined(MADV_WIPEONFORK)
 	/*
@@ -135,6 +137,7 @@ static int stress_mmapfork(stress_args_t *args)
 	}
 #endif
 
+
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -156,6 +159,7 @@ static int stress_mmapfork(stress_args_t *args)
 			if (s_pids[i].pid == 0) {
 				/* Child */
 				const pid_t ppid = getppid();
+				size_t len_total;
 
 				stress_parent_died_alarm();
 				(void)sched_settings_apply(true);
@@ -170,8 +174,6 @@ static int stress_mmapfork(stress_args_t *args)
 					_exit(MMAPFORK_FAILURE);
 				}
 
-				len = ((size_t)info.freeram / (args->num_instances * MAX_PIDS)) / 2;
-
 #if defined(MADV_WIPEONFORK)
 				if (wipe_ok && (wipe_ptr != MAP_FAILED) &&
 				    stress_memory_is_not_zero(wipe_ptr, wipe_size)) {
@@ -180,14 +182,24 @@ static int stress_mmapfork(stress_args_t *args)
 					_exit(MMAPFORK_FAILURE);
 				}
 #endif
-
-				if (!stress_get_setting("mmapfork-bytes", &len)) {
+				len_total = (size_t)info.freeram;
+				if (!stress_get_setting("mmapfork-bytes", &len_total)) {
 					if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-						len = MIN_MMAPFORK_BYTES;
+						len_total = MIN_MMAPFORK_BYTES;
+					if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+						len_total = MAX_32;
 				}
+				if (len_total < MIN_MMAPFORK_BYTES)
+					len_total = MIN_MMAPFORK_BYTES;
+				if (len_total < args->page_size)
+					len_total = args->page_size;
 
-				if (len < MIN_MMAPFORK_BYTES)
-					len = MIN_MMAPFORK_BYTES;
+				len = (len_total / (args->instances * MAX_PIDS)) / 2;
+				if (len < args->page_size)
+					len = args->page_size;
+				if ((i == 0) && report_size) {
+					stress_usage_bytes(args, len, len_total);
+				}
 
 				segv_ret = MMAPFORK_SEGV_MMAP;
 				ptr = stress_mmap_populate(NULL, len, PROT_READ | PROT_WRITE,
@@ -218,6 +230,7 @@ static int stress_mmapfork(stress_args_t *args)
 				}
 				_exit(EXIT_SUCCESS);
 			}
+			report_size = false;
 		}
 
 		/*
@@ -231,8 +244,8 @@ static int stress_mmapfork(stress_args_t *args)
 
 			if (shim_waitpid(s_pids[i].pid, &status, 0) < 0) {
 				if (UNLIKELY(errno != EINTR)) {
-					pr_err("%s: waitpid errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_err("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)s_pids[i].pid, errno, strerror(errno));
 				} else {
 					/* Probably an SIGARLM, force reap */
 					goto reap;
@@ -288,7 +301,7 @@ reap:
 
 const stressor_info_t stress_mmapfork_info = {
 	.stressor = stress_mmapfork,
-	.class = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -296,7 +309,7 @@ const stressor_info_t stress_mmapfork_info = {
 #else
 const stressor_info_t stress_mmapfork_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.opts = opts,
 	.help = help,
