@@ -75,6 +75,7 @@ typedef struct {
 	int fd;
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	stress_numa_mask_t *numa_mask;
+	stress_numa_mask_t *numa_nodes;
 #endif
 } stress_mmaphuge_context_t;
 
@@ -132,7 +133,7 @@ static int stress_mmaphuge_child(stress_args_t *args, void *v_context)
 				bufs[i].sz = sz;
 				/* If we're mapping onto a file, try it first */
 				if (context->mmaphuge_file) {
-					const off_t offset = 4096 * stress_mwc8modn(16);
+					const off_t offset = (off_t)4096 * stress_mwc8modn(16);
 
 					if (sz + offset < context->sz) {
 						buf = (uint8_t *)mmap(NULL, sz,
@@ -163,7 +164,7 @@ static int stress_mmaphuge_child(stress_args_t *args, void *v_context)
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 					if (context->mmaphuge_numa)
-						stress_numa_randomize_pages(context->numa_mask, buf, page_size, sz);
+						stress_numa_randomize_pages(args, context->numa_nodes, context->numa_mask, buf, page_size, sz);
 #endif
 
 					if (context->mmaphuge_mlock)
@@ -251,6 +252,7 @@ static int stress_mmaphuge(stress_args_t *args)
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	context.numa_mask = NULL;
+	context.numa_nodes = NULL;
 #endif
 	context.sz = 16 * MB;
 	context.fd = -1;
@@ -270,8 +272,9 @@ static int stress_mmaphuge(stress_args_t *args)
 
 	context.bufs = (stress_mmaphuge_buf_t *)calloc(context.mmaphuge_mmaps, sizeof(*context.bufs));
 	if (!context.bufs) {
-		pr_inf_skip("%s: cannot allocate buffer array, skipping stressor\n",
-			args->name);
+		pr_inf_skip("%s: cannot allocate %zu byte buffer array%s, skipping stressor\n",
+			args->name, sizeof(context.mmaphuge_mmaps) * sizeof(*context.bufs),
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -324,22 +327,11 @@ static int stress_mmaphuge(stress_args_t *args)
 
 	if (context.mmaphuge_numa) {
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-		if (stress_numa_nodes() > 1) {
-			context.numa_mask = stress_numa_mask_alloc();
-			if (!context.numa_mask) {
-				pr_inf("%s: cannot allocate NUMA mask, disabling --mmaphuge-numa\n",
-					args->name);
-				context.mmaphuge_numa = false;
-			}
-		} else {
-			if (args->instance == 0) {
-				pr_inf("%s: only 1 NUMA node available, disabling --mmaphuge-numa\n",
-					args->name);
-				context.mmaphuge_numa = false;
-			}
-		}
+		stress_numa_mask_and_node_alloc(args, &context.numa_nodes,
+						&context.numa_mask, "--mmaphuge-numa",
+						&context.mmaphuge_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --mmaphuge-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		context.mmaphuge_numa = false;
@@ -351,6 +343,8 @@ static int stress_mmaphuge(stress_args_t *args)
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	if (context.numa_mask)
 		stress_numa_mask_free(context.numa_mask);
+	if (context.numa_nodes)
+		stress_numa_mask_free(context.numa_nodes);
 #endif
 	free(context.bufs);
 
@@ -364,7 +358,7 @@ static int stress_mmaphuge(stress_args_t *args)
 
 const stressor_info_t stress_mmaphuge_info = {
 	.stressor = stress_mmaphuge,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -374,7 +368,7 @@ const stressor_info_t stress_mmaphuge_info = {
 
 const stressor_info_t stress_mmaphuge_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
