@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2023-2024 Luis Chamberlain <mcgrof@kernel.org>
- * Copyright (C) 2023-2024 Colin Ian King.
+ * Copyright (C) 2023-2025 Luis Chamberlain <mcgrof@kernel.org>
+ * Copyright (C) 2023-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,9 +19,12 @@
  */
 #include "stress-ng.h"
 #include "core-attribute.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-module.h"
 
+#include <ctype.h>
+
 #if defined(HAVE_LINUX_MODULE_H)
 #include <linux/module.h>
 #else
@@ -38,6 +41,10 @@ UNEXPECTED
 UNEXPECTED
 #endif
 
+#if defined(HAVE_LZMA_H)
+#include <lzma.h>
+#endif
+
 #ifndef MODULE_INIT_IGNORE_MODVERSIONS
 #define MODULE_INIT_IGNORE_MODVERSIONS 1
 #endif
@@ -46,6 +53,9 @@ UNEXPECTED
 #define MODULE_INIT_IGNORE_VERMAGIC 2
 #endif
 
+#define MODULE_KO	(1)
+#define MODULE_KO_XZ	(2)
+
 static const stress_help_t help[] = {
 	{ NULL,	"module N",	    "start N workers performing module requests" },
 	{ NULL,	"module-name F",    "use the specified module name F to load." },
@@ -67,32 +77,12 @@ static int stress_module_supported(const char *name)
 	return 0;
 }
 
-static int stress_set_module_no_unload(const char *opt)
-{
-	return stress_set_setting_true("module-no-unload", opt);
-}
-
-static int stress_set_module_no_modver(const char *opt)
-{
-	return stress_set_setting_true("module-no-modver", opt);
-}
-
-static int stress_set_module_no_vermag(const char *opt)
-{
-	return stress_set_setting_true("module-no-vermag", opt);
-}
-
-static int stress_set_module_name(const char *name)
-{
-	return stress_set_setting("module-name", TYPE_ID_STR, name);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_module_name,	stress_set_module_name},
-	{ OPT_module_no_modver,	stress_set_module_no_modver },
-	{ OPT_module_no_vermag,	stress_set_module_no_vermag },
-	{ OPT_module_no_unload,	stress_set_module_no_unload },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_module_name,      "module-name",      TYPE_ID_STR,  0, 0, NULL },
+	{ OPT_module_no_modver, "module-no-modver", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_module_no_vermag, "module-no-vermag", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_module_no_unload, "module-no-unload", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(__linux__)
@@ -161,7 +151,7 @@ static enum parse_line_type parse_get_line_type(
 	if (iscomment(line, line_len))
 		return PARSE_COMMENT;
 
-	(void)snprintf(fmt, sizeof(fmt), "%%%zd[^:]:", module_len - 1);
+	(void)snprintf(fmt, sizeof(fmt), "%%%zu[^:]:", module_len - 1);
 
 	/* should be a "kernel/foo/path.ko: .* */
 	ret = sscanf(line, fmt, module);
@@ -223,9 +213,8 @@ static int get_modpath_name(
 		goto out_close;
 
 	while ((line_len = getline(&line, &len, fp)) != -1) {
-		char *module_pathp;
+		const char *module_pathp, *modulenamep;
 		char *start_postfix;
-		char *modulenamep;
 
 		lineno++;
 		parse_type = parse_get_line_type(line, (size_t)line_len, module, sizeof(module));
@@ -273,7 +262,18 @@ static int get_modpath_name(
 			(void)snprintf(module_path, module_path_size, "%s/%s/%s",
 				 dirname_default_prefix,
 				 u.release, module);
-			ret = 0;
+
+			/* Check for .ko end, can't decompress .zst, .xz etc yet */
+			ret = -1 ;
+			len = strlen(module_path);
+			if (len > 6) {
+				if (strncmp(module_path + len - 6, ".ko.xz", 6) == 0)
+					ret = MODULE_KO_XZ;
+			}
+			if (len > 3) {
+				if (strncmp(module_path + len - 3, ".ko", 3) == 0)
+					ret = MODULE_KO;
+			}
 			goto out_close;
 		case PARSE_INVALID:
 			ret = -1;
@@ -301,6 +301,130 @@ out_close:
 #endif
 }
 
+/*
+ *  stress_module_open()
+ *	either open .ko directly or decompress a .ko.xz and return
+ *	a fd to the temp file containing a decompressed .ko
+ */
+static int stress_module_open(stress_args_t *args, int mod_type)
+{
+#if defined(HAVE_LZMA_H) &&	\
+    defined(HAVE_LIB_LZMA)
+	lzma_stream strm = LZMA_STREAM_INIT;
+	lzma_action action = LZMA_RUN;
+	lzma_ret ret;
+	uint8_t buf_in[1024];
+	uint8_t buf_out[1024];
+	char modname[PATH_MAX];
+	int fd_in, fd_out;
+#endif
+
+	(void)args;
+
+	/* Simple case, a ko, open directly */
+	if (mod_type == MODULE_KO)
+		return open(global_module_path, O_RDONLY | O_CLOEXEC);
+
+	/* Not a ko.xz, fail! */
+	if (mod_type != MODULE_KO_XZ)
+		return -1;
+
+#if defined(HAVE_LZMA_H) &&	\
+    defined(HAVE_LIB_LZMA)
+	/*
+	 *  Decompress..
+	 */
+	ret = lzma_stream_decoder(&strm, UINT64_MAX, LZMA_CONCATENATED);
+	if (ret != LZMA_OK) {
+		pr_inf("%s: lzma_stream_decoder failed, ret=%d\n", args->name, ret);
+		return -1;
+	}
+	(void)stress_temp_filename_args(args,
+		modname, sizeof(modname), stress_mwc32());
+	(void)shim_strlcat(modname, ".ko", sizeof(modname));
+	fd_in = open(global_module_path, O_RDONLY);
+	if (fd_in < 0)
+		return -1;
+	fd_out = open(modname, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
+	if (fd_out < 0) {
+		(void)close(fd_in);
+		return -1;
+	}
+	strm.next_in = NULL;
+	strm.avail_in = 0;
+	strm.next_out = buf_out;
+	strm.avail_out = sizeof(buf_out);
+
+	for (;;) {
+		if (strm.avail_in == 0) {
+			ssize_t rd;
+
+			rd = read(fd_in, buf_in, sizeof(buf_in));
+			if (rd < 0) {
+				pr_inf("%s: decompress read failure on '%s', errno=%d (%s)\n",
+					args->name, global_module_path,
+					errno, strerror(errno));
+				(void)unlink(modname);
+				(void)close(fd_out);
+				(void)close(fd_in);
+				return -1;
+			}
+			strm.next_in = buf_in;
+			strm.avail_in = rd;
+
+			if (rd == 0)
+				action = LZMA_FINISH;
+		}
+		ret = lzma_code(&strm, action);
+
+		if ((strm.avail_out == 0) || (ret == LZMA_STREAM_END)) {
+			size_t n = sizeof(buf_out) - strm.avail_out;
+			ssize_t wr;
+
+			wr = write(fd_out, buf_out, n);
+			if (wr < 0) {
+				pr_inf("%s: decompress write failure, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				(void)unlink(modname);
+				(void)close(fd_out);
+				(void)close(fd_in);
+				return -1;
+			}
+			strm.next_out = buf_out;
+			strm.avail_out = wr;
+		}
+
+		if (ret != LZMA_OK) {
+			if (ret == LZMA_STREAM_END)
+				break;
+			pr_inf("%s: decompress error %d\n", args->name, ret);
+			(void)unlink(modname);
+			(void)close(fd_out);
+			(void)close(fd_in);
+			return -1;
+		}
+	}
+
+	if (lseek(fd_out, (off_t)0, SEEK_SET) < 0) {
+		pr_inf("%s: lseek failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		(void)unlink(modname);
+		(void)close(fd_out);
+		(void)close(fd_in);
+		return -1;
+	}
+
+	(void)close(fd_in);
+	(void)close(fd_out);
+	fd_in = open(modname, O_RDONLY | O_CLOEXEC);
+	(void)unlink(modname);
+
+	return fd_in;
+#else
+	return -1;
+#endif
+}
+
 /*
  *  stress_module
  *	stress by heavy module ops
@@ -310,13 +434,23 @@ static int stress_module(stress_args_t *args)
 	bool module_no_unload = false;
 	bool module_no_vermag = false;
 	bool module_no_modver = false;
-	char *module_name_cli = NULL;
-	char *module_name;
-	static char default_module[] = "test_module";
+	const char *module_name_cli = NULL;
+	const char *module_name;
 	const char *finit_args1 = "";
 	unsigned int kernel_flags = 0;
 	struct stat statbuf;
-	int fd, ret = EXIT_SUCCESS;
+	int fd = -1, ret;
+	static const char * const default_modules[] = {
+		"test_user_copy",
+		"test_bpf",
+		"test_module",
+		"test_static_key_base",
+		"test_firmware"
+	};
+
+	ret = stress_temp_dir_mk_args(args);
+        if (ret < 0)
+                return stress_exit_status((int)-ret);
 
 	(void)stress_get_setting("module-name", &module_name_cli);
 	(void)stress_get_setting("module-no-vermag", &module_no_vermag);
@@ -328,10 +462,21 @@ static int stress_module(stress_args_t *args)
 	if (module_no_modver)
 		kernel_flags |= MODULE_INIT_IGNORE_MODVERSIONS;
 
-	module_name = module_name_cli ? module_name_cli : default_module;
-	ret = get_modpath_name(args, module_name, global_module_path, sizeof(global_module_path));
+	if (module_name_cli) {
+		module_name = module_name_cli;
+		ret = get_modpath_name(args, module_name, global_module_path, sizeof(global_module_path));
+	} else {
+		size_t i;
+
+		for (i = 0; i < SIZEOF_ARRAY(default_modules); i++) {
+			module_name = default_modules[i];
+			ret = get_modpath_name(args, module_name, global_module_path, sizeof(global_module_path));
+			if (ret > 0)
+				break;
+		}
+	}
 	if (ret < 0) {
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			if (module_name_cli) {
 				pr_inf_skip("%s: could not find a module path for "
 					"the specified module '%s', ensure it "
@@ -339,22 +484,32 @@ static int stress_module(stress_args_t *args)
 					"skipping stressor\n",
 					args->name, module_name);
 			} else {
+				size_t i;
+				char buf[SIZEOF_ARRAY(default_modules) * 32];
+
+				(void)shim_memset(buf, 0, sizeof(buf));
+				for (i = 0; i < SIZEOF_ARRAY(default_modules); i++) {
+					(void)shim_strlcat(buf, (i > 0) ? ", " : "", sizeof(buf));
+					(void)shim_strlcat(buf, default_modules[i], sizeof(buf));
+				}
+
 				pr_inf_skip("%s: could not find a module path for "
-					"the default test_module '%s', perhaps "
+					"the default modules '%s', perhaps "
 					"CONFIG_TEST_LKM is disabled in your "
-					"kernel (or alternatively use --module-name "
-					"to specify module), skipping stressor\n",
-					args->name, module_name);
+					"kernel or modules are compressed. Alternatively use --module-name "
+					"to specify module. Skipping stressor\n",
+					args->name, buf);
 			}
 		}
-		return EXIT_NO_RESOURCE;
+		ret = EXIT_NO_RESOURCE;
+		goto out;
 	}
 
 	/*
 	 *  We're exercising modules, so if the open fails chalk this
 	 *  up as a resource failure rather than a module test failure.
 	 */
-	fd = open(global_module_path, O_RDONLY | O_CLOEXEC);
+	fd = stress_module_open(args, ret);
 	if (fd < 0) {
 		pr_inf_skip("%s: cannot open the module file %s, "
 			"errno=%d (%s), skipping stressor\n",
@@ -369,7 +524,7 @@ static int stress_module(stress_args_t *args)
 	 *  time-of-use) race
 	 */
 	if (shim_fstat(fd, &statbuf) < 0) {
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			if (module_name_cli) {
 				pr_inf_skip("%s: could not get fstat() on "
 					"the specified module '%s', "
@@ -382,17 +537,17 @@ static int stress_module(stress_args_t *args)
 					args->name, global_module_path);
 			}
 		}
-		(void)close(fd);
-		return EXIT_NO_RESOURCE;
+		ret = EXIT_NO_RESOURCE;
+		goto out;
 	}
 	if (!S_ISREG(statbuf.st_mode)) {
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			pr_inf_skip("%s: module passed is not a regular file "
 				"'%s', skipping stressor\n",
 				args->name, global_module_path);
 		}
-		(void)close(fd);
-		return EXIT_NO_RESOURCE;
+		ret = EXIT_NO_RESOURCE;
+		goto out;
 	}
 
 	/*
@@ -403,18 +558,23 @@ static int stress_module(stress_args_t *args)
 	if (!module_no_unload)
 		(void)shim_delete_module(module_name, 0);
 
+	if (stress_instance_zero(args))
+		pr_inf("%s: exercising module '%s'\n", args->name, module_name);
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	ret = EXIT_SUCCESS;
 	do {
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		if (shim_finit_module(fd, finit_args1, kernel_flags) == 0) {
+			stress_bogo_inc(args);
 			if (!module_no_unload)
 				(void)shim_delete_module(module_name, 0);
 		}
-
-		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
 out:
@@ -423,22 +583,23 @@ out:
 	if (fd >= 0)
 		(void)close(fd);
 
+	(void)stress_temp_dir_rm_args(args);
 	return ret;
 }
 
-stressor_info_t stress_module_info = {
+const stressor_info_t stress_module_info = {
 	.stressor = stress_module,
-	.class = CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_OS,
+	.opts = opts,
 	.supported = stress_module_supported,
 	.help = help
 };
 
 #else
-stressor_info_t stress_module_info = {
+const stressor_info_t stress_module_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_OS,
+	.opts = opts,
 	.supported = stress_module_supported,
 	.help = help
 };
