@@ -164,7 +164,7 @@ static int stress_mq(stress_args_t *args)
 			return EXIT_NO_RESOURCE;
 		}
 		if (errno == ENOSYS) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: POSIX message queues not implemented, skipping stressor\n",
 					args->name);
 			return EXIT_NOT_IMPLEMENTED;
@@ -528,7 +528,7 @@ finish:
 
 const stressor_info_t stress_mq_info = {
 	.stressor = stress_mq,
-	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
+	.classifier = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -536,7 +536,7 @@ const stressor_info_t stress_mq_info = {
 #else
 const stressor_info_t stress_mq_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
+	.classifier = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
