@@ -260,8 +260,11 @@ typedef struct stress_stressor_info {
 #include "core-parse-opts.h"
 #include "core-perf.h"
 #include "core-setting.h"
+#include "core-signal.h"
+#include "core-stack.h"
 #include "core-log.h"
 #include "core-lock.h"
+#include "core-memory.h"
 #include "core-mwc.h"
 #include "core-rapl.h"
 #include "core-sched.h"
@@ -878,6 +881,7 @@ static inline bool ALWAYS_INLINE stress_instance_zero(stress_args_t *args)
 }
 
 #include "core-helper.h"
+#include "core-filesystem.h"
 
 #define STRESS_METRIC_GEOMETRIC_MEAN	(0x1)
 #define STRESS_METRIC_HARMONIC_MEAN	(0x2)
@@ -904,6 +908,9 @@ extern void stress_metrics_set_const_check(stress_args_t *args,
 int unlink(const char *pathname) __attribute__((deprecated("use shim_unlink")));
 int unlinkat(int dirfd, const char *pathname, int flags) __attribute__((deprecated("use shim_unlinkat")));
 int rmdir(const char *pathname) __attribute__((deprecated("use shim_rmdir")));
+size_t strlcpy(char *dst, const char *src, size_t size) __attribute__((deprecated("shim_strscpy")));
+size_t strlcat(char *dst, const char *src, size_t size) __attribute__((deprecated("shim_strlcat")));
+
 #endif
 
 #endif
