@@ -45,7 +45,7 @@ static const stress_help_t help[] = {
 static const stress_opt_t opts[] = {
 	{ OPT_numa_bytes,        "numa-bytes",        TYPE_ID_SIZE_T_BYTES_VM, MIN_NUMA_MMAP_BYTES, MAX_NUMA_MMAP_BYTES, NULL },
 	{ OPT_numa_shuffle_addr, "numa-shuffle-addr", TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_numa_shuffle_node, "numa-shiffle-node", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_numa_shuffle_node, "numa-shuffle-node", TYPE_ID_BOOL, 0, 1, NULL },
 	END_OPT,
 };
 
@@ -367,7 +367,7 @@ static int stress_numa(stress_args_t *args)
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
-		ret = shim_set_mempolicy(MPOL_PREFERRED, NULL, numa_mask->max_nodes);
+		ret = shim_set_mempolicy(MPOL_PREFERRED, NULL, 0);
 		if (UNLIKELY(ret < 0)) {
 			if (errno != ENOSYS) {
 				pr_fail("%s: set_mempolicy failed, errno=%d (%s)\n",
