@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-attribute.h"
+#include "core-builtin.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"pci N",	 "start N workers that read and mmap PCI regions" },
@@ -114,12 +115,12 @@ static void stress_pci_info_get_by_name(stress_pci_info_t **pci_info_list, const
 
 		(void)snprintf(pci_path, sizeof(pci_path),
 			"%s/%s", sys_pci_devices, name);
-		pci_info->path = strdup(pci_path);
+		pci_info->path = shim_strdup(pci_path);
 		if (UNLIKELY(!pci_info->path)) {
 			free(pci_info);
 			return;
 		}
-		pci_info->name = strdup(name);
+		pci_info->name = shim_strdup(name);
 		if (UNLIKELY(!pci_info->name)) {
 			free(pci_info->path);
 			free(pci_info);
@@ -283,6 +284,7 @@ static void NORETURN MLOCKED_TEXT stress_pci_handler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 static void stress_pci_rate(const stress_metrics_t *metrics, char *str, const size_t len)
@@ -290,7 +292,7 @@ static void stress_pci_rate(const stress_metrics_t *metrics, char *str, const si
 	if (metrics->duration > 0.0)
 		(void)snprintf(str, len, "%8.2f", (metrics->count / metrics->duration) / MB);
 	else
-		(void)snprintf(str, len, "%8s", " unknown");
+		(void)snprintf(str, len, "%8s", "untested");
 }
 
 /*
@@ -363,7 +365,7 @@ static int stress_pci(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_block_begin();
 		pr_inf("%s: PCI space read rates in MB per sec for stressor instance 0:\n", args->name);
 		pr_inf("%s: PCI Device     Config Resource\n", args->name);
@@ -385,14 +387,14 @@ static int stress_pci(stress_args_t *args)
 
 const stressor_info_t stress_pci_info = {
 	.stressor = stress_pci,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.help = help
 };
 #else
 const stressor_info_t stress_pci_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
