@@ -71,12 +71,12 @@ static int stress_personality(stress_args_t *args)
 
 	failed = (bool *)calloc(n, sizeof(*failed));
 	if (!failed) {
-		pr_inf_skip("%s: cannot allocate %zu boolean flags, skipping stressor\n",
-			args->name, n);
+		pr_inf_skip("%s: cannot allocate %zu boolean flags%s, skipping stressor\n",
+			args->name, n, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: exercising %zu personalities\n", args->name, n);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -134,7 +134,7 @@ static int stress_personality(stress_args_t *args)
 
 const stressor_info_t stress_personality_info = {
 	.stressor = stress_personality,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.supported = stress_personality_supported,
 	.help = help
@@ -142,7 +142,7 @@ const stressor_info_t stress_personality_info = {
 #else
 const stressor_info_t stress_personality_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/personality.h or personality() system call"
