@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,10 +17,13 @@
  *
  */
 #include "stress-ng.h"
+#include "core-asm-arm.h"
 #include "core-affinity.h"
 #include "core-arch.h"
+#include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"peterson N",		"start N workers that exercise Peterson's algorithm" },
@@ -28,7 +31,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-#if defined(HAVE_SHIM_MFENCE)
+#if defined(HAVE_SHIM_MFENCE) &&	\
+    defined(HAVE_SIGLONGJMP)
 
 typedef struct {
 	volatile int	turn;
@@ -49,6 +53,61 @@ typedef struct peterson {
 } peterson_t;
 
 static peterson_t *peterson;
+static sigjmp_buf jmp_env;
+
+static inline void ALWAYS_INLINE peterson_mfence(void)
+{
+	shim_mfence();
+}
+
+static inline void ALWAYS_INLINE peterson_mbarrier(void)
+{
+#if defined(HAVE_ASM_ARM_DMB_SY)
+	stress_asm_arm_dmb_sy();
+#endif
+}
+
+static void stress_peterson_sigill_handler(int signum)
+{
+	(void)signum;
+
+	siglongjmp(jmp_env, 1);
+	stress_no_return();
+}
+
+static int stress_peterson_supported(const char *name)
+{
+	static struct sigaction act, oldact;
+	int ret;
+
+	(void)shim_memset(&act, 0, sizeof(act));
+	(void)shim_memset(&oldact, 0, sizeof(oldact));
+
+	ret = sigsetjmp(jmp_env, 1);
+	if (ret == 1) {
+		pr_inf_skip("%s: memory barrier not functional, skipping stressor\n", name);
+		(void)sigaction(SIGILL, &oldact, &act);
+		return -1;
+	}
+
+	act.sa_handler = stress_peterson_sigill_handler;
+        (void)sigemptyset(&act.sa_mask);
+        act.sa_flags = SA_NOCLDSTOP;
+	if (sigaction(SIGILL, &act, &oldact) < 0) {
+		pr_inf_skip("%s: sigaction for SIGILL failed, skipping stressor\n",
+			name);
+		return -1;
+	}
+
+	peterson_mbarrier();
+
+	if (sigaction(SIGILL, &oldact, NULL) < 0) {
+		pr_inf_skip("%s: sigaction for SIGILL failed, skipping stressor\n",
+			name);
+		return -1;
+	}
+	return 0;
+}
 
 static int stress_peterson_p0(stress_args_t *args)
 {
@@ -58,10 +117,11 @@ static int stress_peterson_p0(stress_args_t *args)
 	t = stress_time_now();
 	peterson->m.flag[0] = true;
 	peterson->m.turn = 1;
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 	while (peterson->m.flag[1] && (peterson->m.turn == 1)) {
 #if defined(STRESS_ARCH_RISCV)
-		shim_sched_yield();
+		(void)shim_sched_yield();
 #endif
 	}
 
@@ -70,15 +130,17 @@ static int stress_peterson_p0(stress_args_t *args)
 	peterson->m.check++;
 	check1 = peterson->m.check;
 #if defined(STRESS_ARCH_ARM)
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 #endif
 
 	peterson->m.flag[0] = false;
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 	peterson->p0.duration += stress_time_now() - t;
 	peterson->p0.count += 1.0;
 
-	if (check0 + 1 != check1) {
+	if (UNLIKELY(check0 + 1 != check1)) {
 		pr_fail("%s p0: peterson mutex check failed %d vs %d\n",
 			args->name, check0 + 1, check1);
 		return EXIT_FAILURE;
@@ -94,10 +156,11 @@ static int stress_peterson_p1(stress_args_t *args)
 	t = stress_time_now();
 	peterson->m.flag[1] = true;
 	peterson->m.turn = 0;
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 	while (peterson->m.flag[0] && (peterson->m.turn == 0)) {
 #if defined(STRESS_ARCH_RISCV)
-		shim_sched_yield();
+		(void)shim_sched_yield();
 #endif
 	}
 
@@ -106,16 +169,18 @@ static int stress_peterson_p1(stress_args_t *args)
 	peterson->m.check--;
 	check1 = peterson->m.check;
 #if defined(STRESS_ARCH_ARM)
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 #endif
 	stress_bogo_inc(args);
 
 	peterson->m.flag[1] = false;
-	shim_mfence();
+	peterson_mfence();
+	peterson_mbarrier();
 	peterson->p1.duration += stress_time_now() - t;
 	peterson->p1.count += 1.0;
 
-	if (check0 - 1 != check1) {
+	if (UNLIKELY(check0 - 1 != check1)) {
 		pr_fail("%s p1: peterson mutex check failed %d vs %d\n",
 			args->name, check0 - 1, check1);
 		return EXIT_FAILURE;
@@ -138,10 +203,16 @@ static int stress_peterson(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (peterson == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zd bytes for bekker shared struct, skipping stressor\n",
-			args->name, sz);
+		pr_inf_skip("%s: cannot mmap %zu bytes for peterson shared struct%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, sz, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(peterson, sz, "peterson-lock");
+
+	stress_zero_metrics(&peterson->p0, 1);
+	stress_zero_metrics(&peterson->p1, 1);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
@@ -160,7 +231,7 @@ static int stress_peterson(stress_args_t *args)
 		(void)stress_change_cpu(args, parent_cpu);
 		while (stress_continue(args)) {
 			rc = stress_peterson_p0(args);
-			if (rc != EXIT_SUCCESS)
+			if (UNLIKELY(rc != EXIT_SUCCESS))
 				break;
 		}
 		_exit(rc);
@@ -170,7 +241,7 @@ static int stress_peterson(stress_args_t *args)
 		/* Parent */
 		while (stress_continue(args)) {
 			rc = stress_peterson_p1(args);
-			if (rc != EXIT_SUCCESS)
+			if (UNLIKELY(rc != EXIT_SUCCESS))
 				break;
 		}
 		if (stress_kill_pid_wait(pid, &status) >= 0) {
@@ -194,8 +265,9 @@ static int stress_peterson(stress_args_t *args)
 
 const stressor_info_t stress_peterson_info = {
 	.stressor = stress_peterson,
-	.class = CLASS_CPU_CACHE | CLASS_IPC,
+	.classifier = CLASS_CPU_CACHE | CLASS_IPC,
 	.verify = VERIFY_ALWAYS,
+	.supported = stress_peterson_supported,
 	.help = help
 };
 
@@ -203,10 +275,10 @@ const stressor_info_t stress_peterson_info = {
 
 const stressor_info_t stress_peterson_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_IPC,
+	.classifier = CLASS_CPU | CLASS_IPC,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
-	.unimplemented_reason = "built without user space memory fencing"
+	.unimplemented_reason = "built without user space memory fencing or support for siglongjmp"
 };
 
 #endif
