@@ -90,10 +90,13 @@ static int stress_pipeherd(stress_args_t *args)
 	double t1, t2;
 #endif
 
-	(void)stress_get_setting("pipeherd-yield", &pipeherd_yield);
+	if (!stress_get_setting("pipeherd-yield", &pipeherd_yield)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			pipeherd_yield = true;
+	}
 
 	if (pipe(fd) < 0) {
-		pr_fail("%s: pipe failed: %d (%s)\n",
+		pr_fail("%s: pipe failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -113,11 +116,12 @@ static int stress_pipeherd(stress_args_t *args)
 	}
 #endif
 
+	(void)shim_memset(&data, 0, sizeof(data));	/* stop valgrind complaining */
 	data.counter = 0;
 	data.check = check;
 	sz = write(fd[1], &data, sizeof(data));
 	if (sz < 0) {
-		pr_fail("%s: write to pipe failed: %d (%s)\n",
+		pr_fail("%s: write to pipe failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd[0]);
 		(void)close(fd[1]);
@@ -218,7 +222,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_pipeherd_info = {
 	.stressor = stress_pipeherd,
-	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
+	.classifier = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
