@@ -19,6 +19,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 #include "core-pthread.h"
 #include "core-target-clones.h"
@@ -49,7 +50,7 @@ typedef struct {
 	const char *fs_type;		/* file system type */
 	int fd;				/* file descriptor */
 	bool pseek_rand;		/* random seek option */
-	size_t pseek_io_size;		/* write/read I/O size */
+	uint64_t pseek_io_size;		/* write/read I/O size */
 	pid_t parent_pid;		/* stressor ped */
 } stress_peekio_info_t;
 
@@ -166,7 +167,7 @@ retry:
 	case ENOSPC:
 		return 0;
 	case 0:
-		pr_fail("%s: write of %zu bytes only wrote %zd bytes\n",
+		pr_fail("%s: write of %" PRIu64 " bytes only wrote %zd bytes\n",
 			args->name, info->pseek_io_size, ret);
 		return -1;
 	}
@@ -246,7 +247,7 @@ PRAGMA_UNROLL_N(4)
 	case ENOSPC: /* e.g. on vfat */
 		return 0;
 	case 0:
-		pr_fail("%s: read of %zu bytes only read %zd bytes\n",
+		pr_fail("%s: read of %" PRIu64 " bytes only read %zd bytes\n",
 			args->name, info->pseek_io_size, ret);
 		return -1;
 	}
@@ -373,9 +374,10 @@ static int stress_pseek(stress_args_t *args)
 	procs = stress_mmap_populate(NULL, procs_size, PROT_READ | PROT_WRITE,
 					MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (procs == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap procs array, errno=%d (%s), "
-			"skipping stressor\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zu byte procs array%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, procs_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(procs, procs_size, "process-state");
@@ -385,8 +387,28 @@ static int stress_pseek(stress_args_t *args)
 	info.pseek_io_size = DEFAULT_PSEEKIO_IO_SIZE;
 	info.parent_pid = getpid();
 
-	(void)stress_get_setting("pseek-rand", &info.pseek_rand);
-	(void)stress_get_setting("pseek-io-size", &info.pseek_io_size);
+	if (!stress_get_setting("pseek-rand", &info.pseek_rand)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			info.pseek_rand = true;
+	}
+	if (!stress_get_setting("pseek-io-size", &info.pseek_io_size)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			info.pseek_io_size = MAX_PSEEKIO_IO_SIZE;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			info.pseek_io_size = MIN_PSEEKIO_IO_SIZE;
+	}
+	if (info.pseek_io_size < MIN_PSEEKIO_IO_SIZE) {
+		info.pseek_io_size = MIN_PSEEKIO_IO_SIZE;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --pseek-io-size too small, using %" PRIu64 " instead\n",
+				args->name, info.pseek_io_size);
+	}
+	if (info.pseek_io_size > MAX_PSEEKIO_IO_SIZE) {
+		info.pseek_io_size = MAX_PSEEKIO_IO_SIZE;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --pseek-io-size too large, using %" PRIu64 " instead\n",
+				args->name, info.pseek_io_size);
+	}
 
 	for (i = 0; i < pseek_procs; i++) {
 		procs[i].args = args;
@@ -404,24 +426,24 @@ static int stress_pseek(stress_args_t *args)
 	}
 
 	for (i = 0; i < pseek_procs; i++) {
-		procs[i].buf = (uint8_t *)stress_mmap_populate(NULL, info.pseek_io_size,
+		procs[i].buf = (uint8_t *)stress_mmap_populate(NULL, (size_t)info.pseek_io_size,
 						PROT_READ | PROT_WRITE,
 						MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 		if (procs[i].buf == MAP_FAILED) {
 			size_t j;
 
-			pr_inf_skip("%s: cannot mmap buffer of %zu bytes, "
+			pr_inf_skip("%s: failed to mmap buffer of %" PRIu64 " bytes%s, "
 				"errno=%d (%s), skipping stressor\n",
 				args->name, info.pseek_io_size,
-				errno, strerror(errno));
+				stress_get_memfree_str(), errno, strerror(errno));
 			for (j = 0; j < i; j++) {
-				(void)munmap((void *)procs[j].buf, info.pseek_io_size);
+				(void)munmap((void *)procs[j].buf, (size_t)info.pseek_io_size);
 			}
 			rc = EXIT_NO_RESOURCE;
 			goto tidy_munmap_procs;
 		}
-		stress_set_vma_anon_name(procs[i].buf, info.pseek_io_size, "pseek-buffer");
-		(void)shim_memset(procs[i].buf, stress_mwc8(), info.pseek_io_size);
+		stress_set_vma_anon_name(procs[i].buf, (size_t)info.pseek_io_size, "pseek-buffer");
+		(void)shim_memset(procs[i].buf, stress_mwc8(), (size_t)info.pseek_io_size);
 	}
 
 	ret = stress_temp_dir_mk_args(args);
@@ -497,7 +519,7 @@ tidy_unlink:
 	(void)stress_temp_dir_rm_args(args);
 tidy_munmap_bufs:
 	for (i = 0; i < pseek_procs; i++)
-		(void)munmap((void *)procs[i].buf, info.pseek_io_size);
+		(void)munmap((void *)procs[i].buf, (size_t)info.pseek_io_size);
 tidy_munmap_procs:
 	(void)munmap((void *)procs, procs_size);
 
@@ -506,13 +528,13 @@ tidy_munmap_procs:
 
 static const stress_opt_t opts[] = {
 	{ OPT_pseek_rand,    "pseek-seek-rand",  TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_pseek_io_size, "pseek-io-size",    TYPE_ID_SIZE_T_BYTES_FS, MIN_PSEEKIO_IO_SIZE, MAX_PSEEKIO_IO_SIZE, NULL },
+	{ OPT_pseek_io_size, "pseek-io-size",    TYPE_ID_UINT64_BYTES_FS, MIN_PSEEKIO_IO_SIZE, MAX_PSEEKIO_IO_SIZE, NULL },
 	END_OPT,
 };
 
 const stressor_info_t stress_pseek_info = {
 	.stressor = stress_pseek,
-	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
