@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 #include "core-sort.h"
 #include "core-target-clones.h"
@@ -29,8 +30,10 @@
 #define MAX_QSORT_SIZE		(4 * MB)
 #define DEFAULT_QSORT_SIZE	(256 * KB)
 
+#if defined(HAVE_SIGLONGJMP)
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
+#endif
 
 typedef int (*comp_func_t)(const void *v1, const void *v2);
 typedef void (*qsort_func_t)(void *base, size_t nmemb, size_t size, comp_func_t cmp);
@@ -48,6 +51,7 @@ typedef struct {
 	const qsort_func_t qsort_func;
 } stress_qsort_method_t;
 
+#if defined(HAVE_SIGLONGJMP)
 /*
  *  stress_qsort_handler()
  *	SIGALRM generic handler
@@ -59,8 +63,10 @@ static void MLOCKED_TEXT stress_qsort_handler(int signum)
 	if (do_jmp) {
 		do_jmp = false;
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
 	}
 }
+#endif
 
 typedef uint32_t qsort_swap_type_t;
 
@@ -280,12 +286,14 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 	uint64_t qsort_size = DEFAULT_QSORT_SIZE;
 	int32_t *data;
 	size_t n, data_size, qsort_method = 0;
-	struct sigaction old_action;
-	int ret;
 	double rate;
 	NOCLOBBER double duration = 0.0, count = 0.0, sorted = 0.0;
 	NOCLOBBER int rc = EXIT_SUCCESS;
 	qsort_func_t qsort_func;
+#if defined(HAVE_SIGLONGJMP)
+	struct sigaction old_action;
+	int ret;
+#endif
 
 	stress_catch_sigill();
 
@@ -311,6 +319,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 	(void)stress_madvise_collapse(data, data_size);
 	stress_set_vma_anon_name(data, data_size, "qsort-data");
 
+#if defined(HAVE_SIGLONGJMP)
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
 		/*
@@ -319,12 +328,11 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 		goto tidy;
 	}
-
 	if (stress_sighandler(args->name, SIGALRM, stress_qsort_handler, &old_action) < 0) {
 		free(data);
 		return EXIT_FAILURE;
 	}
-
+#endif
 
 	stress_sort_data_int32_init(data, n);
 
@@ -392,9 +400,11 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
+#if defined(HAVE_SIGLONGJMP)
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
+#endif
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "qsort comparisons per sec",
