@@ -125,7 +125,7 @@ again:
 	case RACE_SCHED_METHOD_ALL:
 		method_index = method_all_index;
 		method_all_index++;
-		if (UNLIKELY(method_all_index > SIZEOF_ARRAY(stress_race_sched_methods)))
+		if (UNLIKELY(method_all_index >= SIZEOF_ARRAY(stress_race_sched_methods)))
 			method_all_index = 1;
 		goto again;
 	case RACE_SCHED_METHOD_NEXT:
@@ -277,7 +277,8 @@ static stress_race_sched_child_t *stress_race_sched_new(void)
 static void stress_race_sched_head_remove(const int options)
 {
 	if (children.head) {
-		int status, ret;
+		pid_t ret;
+		int status;
 		stress_race_sched_child_t *head = children.head;
 
 		ret = waitpid(children.head->pid, &status, options);
@@ -455,7 +456,7 @@ static int stress_race_sched(stress_args_t *args)
 
 const stressor_info_t stress_race_sched_info = {
 	.stressor = stress_race_sched,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -465,7 +466,7 @@ const stressor_info_t stress_race_sched_info = {
 
 const stressor_info_t stress_race_sched_info = {
         .stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.verify = VERIFY_ALWAYS,
