@@ -423,8 +423,7 @@ skip_fsopen:
 
 #endif
 		stress_bogo_inc(args);
-	} while (keep_mounting && stress_continue(args) &&
-		 (!args->bogo.max_ops || (stress_bogo_get(args) < args->bogo.max_ops)));
+	} while (keep_mounting && stress_continue(args));
 
 cleanup:
 	stress_ramfs_umount(args, realpathname);
@@ -502,7 +501,7 @@ finish:
 
 const stressor_info_t stress_ramfs_info = {
 	.stressor = stress_ramfs_mount,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.supported = stress_ramfs_supported,
 	.verify = VERIFY_ALWAYS,
@@ -511,7 +510,7 @@ const stressor_info_t stress_ramfs_info = {
 #else
 const stressor_info_t stress_ramfs_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.supported = stress_ramfs_supported,
 	.verify = VERIFY_ALWAYS,
