@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mmap.h"
 #include "core-vmstat.h"
 
 #if defined(HAVE_SYS_SYSMACROS_H)
@@ -155,7 +156,7 @@ static int stress_rawdev_wiggle(
 			ret = pread(fd, buffer, blksz, offset);
 			if (UNLIKELY(ret < 0)) {
 				if (errno != EINTR) {
-					pr_fail("%s: pread at %" PRIdMAX "failed, errno=%d (%s)\n",
+					pr_fail("%s: pread at %" PRIdMAX " failed, errno=%d (%s)\n",
 						args->name, (intmax_t)offset, errno, strerror(errno));
 					return -1;
 				}
@@ -382,8 +383,10 @@ static int stress_rawdev(stress_args_t *args)
 
 	metrics = (stress_metrics_t *)calloc(SIZEOF_ARRAY(rawdev_methods), sizeof(*metrics));
 	if (!metrics) {
-		pr_inf_skip("%s: cannot allocate metrics table, skipping stressor\n",
-			args->name);
+		pr_inf_skip("%s: cannot allocate %zu byte metrics table%s, "
+			"skipping stressor\n",
+			args->name, SIZEOF_ARRAY(rawdev_methods),
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -408,14 +411,14 @@ static int stress_rawdev(stress_args_t *args)
 
 	fd = open(devpath, O_RDONLY | O_NONBLOCK);
 	if (fd < 0) {
-		pr_inf("%s: cannot open raw block device: errno=%d (%s)\n",
+		pr_inf("%s: cannot open raw block device, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		free(metrics);
 		return EXIT_NO_RESOURCE;
 	}
 	ret = ioctl(fd, BLKGETSIZE, &blks);
 	if (ret < 0) {
-		pr_inf("%s: cannot get block size: errno=%d (%s)\n",
+		pr_inf("%s: cannot get block size, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
 		free(metrics);
@@ -423,7 +426,7 @@ static int stress_rawdev(stress_args_t *args)
 	}
 	ret = ioctl(fd, BLKSSZGET, &blksz);
 	if (ret < 0) {
-		pr_inf("%s: cannot get block size: errno=%d (%s)\n",
+		pr_inf("%s: cannot get block size, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
 		free(metrics);
@@ -440,8 +443,8 @@ static int stress_rawdev(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buffer == MAP_FAILED) {
-		pr_inf("%s: cannot allocate buffer of %zd bytes with %zd allocation\n",
-			args->name, blksz, mmapsz);
+		pr_inf("%s: failed to mmap buffer of %zu bytes with %zu allocations%s\n",
+			args->name, blksz, mmapsz, stress_get_memfree_str());
 		(void)close(fd);
 		free(metrics);
 		return EXIT_NO_RESOURCE;
@@ -451,14 +454,14 @@ static int stress_rawdev(stress_args_t *args)
 	(void)close(fd);
 	fd = open(devpath, O_RDONLY | O_DIRECT);
 	if (fd < 0) {
-		pr_inf("%s: cannot open raw block device: errno=%d (%s)\n",
+		pr_inf("%s: cannot open raw block device, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)munmap((void *)buffer, mmapsz);
 		free(metrics);
 		return EXIT_NO_RESOURCE;
 	}
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: exercising %s (%zd blocks of size %zd bytes)\n",
 			args->name, devpath, blks, blksz);
 
@@ -499,7 +502,7 @@ static int stress_rawdev(stress_args_t *args)
 const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_rawdev,
 	.supported = stress_rawdev_supported,
-	.class = CLASS_IO,
+	.classifier = CLASS_IO,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -507,7 +510,7 @@ const stressor_info_t stress_rawdev_info = {
 #else
 const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_IO,
+	.classifier = CLASS_IO,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
