@@ -99,8 +99,9 @@ static int stress_reboot(stress_args_t *args)
 
 	stack = (char *)malloc(CLONE_STACK_SIZE);
 	if (!stack) {
-		pr_inf_skip("%s: out of memory allocating %zd byte stack, skipping stressor\n",
-			args->name, (size_t)CLONE_STACK_SIZE);
+		pr_inf_skip("%s: cannot allocate %zu byte stack%s, skipping stressor\n",
+			args->name, (size_t)CLONE_STACK_SIZE,
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 #endif
@@ -201,12 +202,12 @@ static int stress_reboot(stress_args_t *args)
 	free(stack);
 #endif
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 const stressor_info_t stress_reboot_info = {
 	.stressor = stress_reboot,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -215,7 +216,7 @@ const stressor_info_t stress_reboot_info = {
 
 const stressor_info_t stress_reboot_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux with reboot() support"
