@@ -105,7 +105,10 @@ static void NORETURN stress_rmap_child(
 	do {
 		register ssize_t i;
 		const uint8_t rnd8 = stress_mwc8();
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 		const int sync_flag = (rnd8 & 0x80) ? MS_ASYNC : MS_SYNC;
+#endif
 
 		switch (rnd8 & 3) {
 		case 0:
@@ -117,7 +120,10 @@ static void NORETURN stress_rmap_child(
 						rc = EXIT_FAILURE;
 						goto fail;
 					}
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 					(void)shim_msync(mappings[i], sz, sync_flag);
+#endif
 				}
 			}
 			break;
@@ -130,7 +136,10 @@ static void NORETURN stress_rmap_child(
 						rc = EXIT_FAILURE;
 						goto fail;
 					}
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 					(void)shim_msync(mappings[i], sz, sync_flag);
+#endif
 				}
 			}
 			break;
@@ -145,7 +154,10 @@ static void NORETURN stress_rmap_child(
 						rc = EXIT_FAILURE;
 						goto fail;
 					}
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 					(void)shim_msync(mappings[j], sz, sync_flag);
+#endif
 				}
 			}
 			break;
@@ -158,7 +170,10 @@ static void NORETURN stress_rmap_child(
 						rc = EXIT_FAILURE;
 						goto fail;
 					}
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 					(void)shim_msync(mappings[i], sz, sync_flag);
+#endif
 				}
 			}
 			break;
@@ -188,16 +203,17 @@ static int stress_rmap(stress_args_t *args)
 	if (stress_sigchld_set_handler(args) < 0)
 		return EXIT_NO_RESOURCE;
 
-	s_pids = stress_s_pids_mmap(RMAP_CHILD_MAX);
+	s_pids = stress_sync_s_pids_mmap(RMAP_CHILD_MAX);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, RMAP_CHILD_MAX);
+		pr_inf_skip("%s: failed to mmap %d PIDs%s, skipping stressor\n",
+			args->name, RMAP_CHILD_MAX, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
 	counter_lock = stress_lock_create("counter");
 	if (!counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
-		(void)stress_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -212,7 +228,7 @@ static int stress_rmap(stress_args_t *args)
 	rc = stress_temp_dir_mk_args(args);
 	if (rc < 0) {
 		(void)stress_lock_destroy(counter_lock);
-		(void)stress_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 		return stress_exit_status((int)-rc);
 	}
 
@@ -226,7 +242,7 @@ static int stress_rmap(stress_args_t *args)
 		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		(void)stress_lock_destroy(counter_lock);
-		(void)stress_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
 		return (int)rc;
 	}
@@ -238,7 +254,7 @@ static int stress_rmap(stress_args_t *args)
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
 		(void)stress_lock_destroy(counter_lock);
-		(void)stress_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
 		return EXIT_NO_RESOURCE;
 	}
@@ -271,7 +287,7 @@ static int stress_rmap(stress_args_t *args)
 
 		s_pids[i].pid = fork();
 		if (s_pids[i].pid < 0) {
-			pr_err("%s: fork failed: errno=%d: (%s)\n",
+			pr_err("%s: fork failed, errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			goto cleanup;
 		} else if (s_pids[i].pid == 0) {
@@ -303,7 +319,7 @@ static int stress_rmap(stress_args_t *args)
 	 *  Wait for SIGINT or SIGALRM
 	 */
 	while (stress_bogo_inc_lock(args, counter_lock, false)) {
-		pause();
+		(void)shim_pause();
 	}
 
 cleanup:
@@ -321,14 +337,14 @@ cleanup:
 	(void)close(fd);
 	(void)stress_temp_dir_rm_args(args);
 	(void)stress_lock_destroy(counter_lock);
-	(void)stress_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+	(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
 	return rc;
 }
 
 const stressor_info_t stress_rmap_info = {
 	.stressor = stress_rmap,
-	.class = CLASS_OS | CLASS_MEMORY,
+	.classifier = CLASS_OS | CLASS_MEMORY,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
