@@ -20,6 +20,7 @@
 #include "core-arch.h"
 #include "core-builtin.h"
 #include "core-helper.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pragma.h"
 
@@ -228,15 +229,16 @@ static int stress_rseq(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (rseq_info == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate shared page, "
+		pr_inf_skip("%s: failed to mmap %zu byte shared page%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, sizeof(*rseq_info),
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(rseq_info, sizeof(*rseq_info), "state");
 
 	rseq_area = stress_rseq_get_area();
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("libc rseq_area @ %p\n", rseq_area);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -259,13 +261,13 @@ static int stress_rseq(stress_args_t *args)
 const stressor_info_t stress_rseq_info = {
 	.stressor = stress_rseq,
 	.supported = stress_rseq_supported,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.help = help
 };
 #else
 const stressor_info_t stress_rseq_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.help = help,
 	.unimplemented_reason = "built without Linux restartable sequences support"
 };
