@@ -21,6 +21,8 @@
 #include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
+#include "core-prime.h"
 #include "core-sched.h"
 
 #include <sched.h>
@@ -257,22 +259,27 @@ redo:
 #if defined(HAVE_SYS_SELECT_H) &&       \
     defined(HAVE_SELECT)
 	case 25:
-		FD_ZERO(&rfds);
-		FD_SET(fdstdin, &rfds);
-		tv.tv_sec = 0;
-		tv.tv_usec = 100;
-		(void)select(fdstdin + 1, &rfds, NULL, NULL, &tv);
+		if (fdstdin < FD_SETSIZE) {
+			FD_ZERO(&rfds);
+			FD_SET(fdstdin, &rfds);
+			tv.tv_sec = 0;
+			tv.tv_usec = 100;
+			(void)select(fdstdin + 1, &rfds, NULL, NULL, &tv);
+		}
 		break;
 #endif
 #if defined(HAVE_SYS_SELECT_H) &&       \
-    defined(HAVE_PSELECT)
+    defined(HAVE_PSELECT) &&		\
+    defined(FD_SETSIZE)
 	case 26:
-		FD_ZERO(&rfds);
-		FD_SET(fdstdin, &rfds);
-		ts.tv_sec = 0;
-		ts.tv_nsec = 100000;
-		(void)sigemptyset(&sigmask);
-		(void)pselect(fdstdin + 1, &rfds, NULL, NULL, &ts, &sigmask);
+		if (fdstdin < FD_SETSIZE) {
+			FD_ZERO(&rfds);
+			FD_SET(fdstdin, &rfds);
+			ts.tv_sec = 0;
+			ts.tv_nsec = 100000;
+			(void)sigemptyset(&sigmask);
+			(void)pselect(fdstdin + 1, &rfds, NULL, NULL, &ts, &sigmask);
+		}
 		break;
 #endif
 	default:
@@ -541,7 +548,7 @@ static int stress_schedmix(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		pause();
+		(void)shim_pause();
 	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
