@@ -320,10 +320,10 @@ static void stress_sctp_sockopts(const int fd)
 	STRESS_SCTP_SOCKOPT(SCTP_EXPOSE_POTENTIALLY_FAILED_STATE, struct sctp_assoc_value)
 #endif
 #if defined(SCTP_REMOTE_UDP_ENCAPS_PORT) && 	\
-    defined(HAVE_SCTP_ASSOCIATION)
-	STRESS_SCTP_SOCKOPT(SCTP_REMOTE_UDP_ENCAPS_PORT, struct sctp_association)
+    defined(HAVE_SCTP_UDPENCAPS)
+	STRESS_SCTP_SOCKOPT(SCTP_REMOTE_UDP_ENCAPS_PORT, struct sctp_udpencaps)
 #else
-	//UNEXPECTED
+	UNEXPECTED
 #endif
 #if defined(SCTP_PLPMTUD_PROBE_INTERVAL) &&	\
     defined(HAVE_SCTP_PROBEINTERVAL)
@@ -367,7 +367,7 @@ retry:
 		fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
 		if (UNLIKELY(fd < 0)) {
 			if (errno == EPROTONOSUPPORT) {
-				if (args->instance == 0)
+				if (stress_instance_zero(args))
 					pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 						args->name);
 				return EXIT_NO_RESOURCE;
@@ -419,7 +419,7 @@ retry:
 #endif
 
 		do {
-			int flags;
+			int flags = 0;
 			struct sctp_sndrcvinfo sndrcvinfo;
 			ssize_t n;
 
@@ -488,7 +488,7 @@ static int OPTIMIZE3 stress_sctp_server(
 	fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
 	if (fd < 0) {
 		if (errno == EPROTONOSUPPORT) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 					args->name);
 			rc = EXIT_NO_RESOURCE;
@@ -669,7 +669,7 @@ again:
 	if (pid < 0) {
 		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -702,7 +702,7 @@ finish:
 
 const stressor_info_t stress_sctp_info = {
 	.stressor = stress_sctp,
-	.class = CLASS_NETWORK,
+	.classifier = CLASS_NETWORK,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -710,7 +710,7 @@ const stressor_info_t stress_sctp_info = {
 #else
 const stressor_info_t stress_sctp_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_NETWORK,
+	.classifier = CLASS_NETWORK,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
