@@ -100,7 +100,9 @@ static int stress_secretmem_child(stress_args_t *args, void *context)
 
 	mappings = (uint8_t **)calloc(MMAP_MAX, sizeof(*mappings));
 	if (UNLIKELY(!mappings)) {
-		pr_fail("%s: calloc failed, out of memory\n", args->name);
+		pr_inf_skip("%s: failed to allocate %zu bytes%s, skipping stressor\n",
+			args->name, (size_t)MMAP_MAX * sizeof(*mappings),
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -196,14 +198,14 @@ static int stress_secretmem(stress_args_t *args)
 
 const stressor_info_t stress_secretmem_info = {
 	.stressor = stress_secretmem,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.help = help,
 	.supported = stress_secretmem_supported
 };
 #else
 const stressor_info_t stress_secretmem_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.help = help,
 	.unimplemented_reason = "built with headers that did not define memfd_secret() system call"
 };
