@@ -135,7 +135,7 @@ static int stress_sendfile(stress_args_t *args)
 		}
 
 		if (errno == ENOSYS) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: skipping stressor, sendfile not implemented\n",
 					args->name);
 			rc = EXIT_NOT_IMPLEMENTED;
@@ -206,7 +206,7 @@ dir_out:
 
 const stressor_info_t stress_sendfile_info = {
 	.stressor = stress_sendfile,
-	.class = CLASS_PIPE_IO | CLASS_OS,
+	.classifier = CLASS_PIPE_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -214,7 +214,7 @@ const stressor_info_t stress_sendfile_info = {
 #else
 const stressor_info_t stress_sendfile_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_PIPE_IO | CLASS_OS,
+	.classifier = CLASS_PIPE_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
