@@ -62,6 +62,8 @@ UNEXPECTED
 #define MAX_SHM_SYSV_SEGMENTS	(128)
 #define DEFAULT_SHM_SYSV_SEGMENTS (8)
 
+#define MAX_SHM_KEYS		(1U << 16)
+
 static const stress_help_t help[] = {
 	{ NULL,	"shm-sysv N",		"start N workers that exercise System V shared memory" },
 	{ NULL,	"shm-sysv-bytes N",	"allocate and free N bytes of shared memory per loop" },
@@ -318,17 +320,13 @@ static int get_bad_shmid(stress_args_t *args)
  *  exercise_shmctl()
  *	exercise shmctl syscall with all possible values of arguments
  */
-static void exercise_shmctl(const size_t sz, stress_args_t *args)
+static void exercise_shmctl(const key_t key, const size_t sz, stress_args_t *args)
 {
-	key_t key;
 	int shm_id;
 #if !defined(STRESS_ARCH_M68K)
 	const int bad_shmid = get_bad_shmid(args);
 #endif
 
-	/* Get a unique random key */
-	key = (key_t)stress_mwc16();
-
 	shm_id = shmget(key, sz, IPC_CREAT);
 	if (shm_id < 0)
 		return;
@@ -354,6 +352,7 @@ static void exercise_shmctl(const size_t sz, stress_args_t *args)
 		struct shmid_ds buf;
 		int ret;
 
+		errno = 0;
 		ret = shmctl(shm_id, IPC_STAT, &buf);
 		if (UNLIKELY((ret >= 0) && (errno == 0)))
 			pr_fail("%s: shmctl IPC_STAT unexpectedly succeeded on non-existent shared "
@@ -368,14 +367,10 @@ static void exercise_shmctl(const size_t sz, stress_args_t *args)
  *  exercise_shmget()
  *	exercise shmget syscall with all possible values of arguments
  */
-static void exercise_shmget(const size_t sz, const char *name)
+static void exercise_shmget(const key_t key, const size_t sz, const char *name)
 {
-	key_t key;
 	int shm_id;
 
-	/* Get a unique random key */
-	key = (key_t)stress_mwc16();
-
 	/* Exercise invalid flags */
 	shm_id = shmget(key, sz, ~0);
 	if (shm_id >= 0)
@@ -388,6 +383,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 		 * Exercise invalid shmget by creating an already
 		 * existing shared memory segment and IPC_EXCL flag
 		 */
+		errno = 0;
 		shm_id2 = shmget(key, sz, IPC_CREAT | IPC_EXCL);
 		if (UNLIKELY((shm_id2 >= 0) && (errno == 0))) {
 			pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded and re-created "
@@ -400,6 +396,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 		 * Exercise invalid shmget by creating an already
 		 * existing shared memory segment but of greater size
 		 */
+		errno = 0;
 		shm_id2 = shmget(key, sz + (1024 * 1024), IPC_CREAT);
 		if (UNLIKELY((shm_id2 >= 0) && (errno == 0))) {
 			pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded and again "
@@ -413,6 +410,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 
     /* Exercise shmget on invalid sizes argument*/
 #if defined(SHMMIN)
+	errno = 0;
 	shm_id = shmget(key, SHMMIN - 1, IPC_CREAT);
 	if (UNLIKELY((SHMMIN > 0) && (shm_id >= 0))) {
 		pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on invalid value of"
@@ -424,6 +422,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 #endif
 
 #if defined(SHMMAX)
+	errno = 0;
 	shm_id = shmget(key, SHMMAX + 1, IPC_CREAT);
 	if (UNLIKELY((SHMMAX < ~(size_t)0) && (shm_id >= 0))) {
 		pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on invalid value of"
@@ -441,6 +440,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 			if (sscanf(buf, "%zu", &shmmax) == 1) {
 				shmmax++;
 				if (shmmax > 0) {
+					errno = 0;
 					shm_id = shmget(key, shmmax, IPC_CREAT);
 					if (UNLIKELY(shm_id >= 0)) {
 						pr_fail("%s: shmget IPC_CREAT unexpectedly succeeded on "
@@ -469,6 +469,7 @@ static void exercise_shmget(const size_t sz, const char *name)
 	UNEXPECTED
 #endif
 
+	errno = 0;
 	shm_id = shmget(key, sz, IPC_EXCL);
 	if (UNLIKELY((shm_id >= 0) && (errno == 0))) {
 		pr_fail("%s: shmget IPC_RMID unexpectedly succeeded on non-existent shared "
@@ -523,7 +524,7 @@ static void stress_shm_sysv_linux_proc_map(const void *addr, const size_t sz)
 	const int len = (int)sizeof(void *);
 	const intptr_t start = (intptr_t)addr, end = start + (intptr_t)sz;
 
-	(void)snprintf(path, sizeof(path), "/proc/%d/map_files/%*.*tx-%*.*tx",
+	(void)snprintf(path, sizeof(path), "/proc/%d/map_files/%*.*" PRIxPTR "-%*.*" PRIxPTR,
 		getpid(), len, len, start, len, len, end);
 
 	/*
@@ -559,6 +560,55 @@ static void stress_shm_sysv_linux_proc_map(const void *addr, const size_t sz)
 	}
 }
 #endif
+
+/*
+ *  stress_shm_sysv_get_key()
+ *	find a 'random' unused key
+ */
+static int stress_shm_sysv_get_key(
+	stress_args_t *args,
+	const size_t segment,
+	const uint32_t keys_per_instance,
+	const uint32_t keys_per_segment,
+	key_t *key,
+	key_t keys[MAX_SHM_SYSV_SEGMENTS])
+{
+	*key = 0;
+
+	for (;;) {
+		key_t new_key;
+		size_t i;
+		uint32_t offset;
+
+retry:
+		if (!stress_continue(args))
+			break;
+
+		/* Get a unique random key */
+		offset = (args->instance * keys_per_instance) +
+		         (segment * keys_per_segment) +
+		         stress_mwc32modn(keys_per_segment);
+		/* Key should never exceed MAX_SHM_KEYS */
+		new_key = offset & (MAX_SHM_KEYS - 1);
+		if (!new_key)
+			goto retry;
+
+		/* Is it already used in our local cache? */
+		for (i = 0; i < MAX_SHM_SYSV_SEGMENTS; i++) {
+			if (new_key == keys[i])
+				goto retry;
+		}
+		/* Is it already used in the entire system? */
+		if (shmget(new_key, args->page_size, 0) < 0) {
+			if (errno == ENOENT) {
+				*key = new_key;
+				return 0;
+			}
+		}
+	}
+	return -1;
+}
+
 /*
  *  stress_shm_sysv_child()
  * 	stress out the shm allocations. This can be killed by
@@ -588,10 +638,20 @@ static int stress_shm_sysv_child(
 	double shmget_duration = 0.0, shmget_count = 0.0;
 	double shmat_duration = 0.0, shmat_count = 0.0;
 	double shmdt_duration = 0.0, shmdt_count = 0.0;
+	uint32_t seg_space = args->instances * shm_sysv_segments;
+	uint32_t max_keys = (uint32_t)MAX_SHM_KEYS / seg_space;
+	const uint32_t keys_per_instance = MAX_SHM_KEYS / args->instances;
+	const uint32_t keys_per_segment = keys_per_instance / shm_sysv_segments;
+
+	max_keys = (max_keys < 1) ? 1 : max_keys;
+
+	if (stress_instance_zero(args))
+		pr_dbg("%s: %" PRIu32 " shm-sysv keys per stressor segment (%zu segments)\n",
+			args->name, max_keys, shm_sysv_segments);
 
 	buffer = (uint64_t *)calloc(buffer_size, sizeof(*buffer));
 	if (!buffer) {
-		pr_inf_skip("%s: faild to allocate %zu byte buffer%s, skipping stressor\n",
+		pr_inf_skip("%s: failed to allocate %zu byte buffer%s, skipping stressor\n",
 			args->name, buffer_size, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
@@ -612,14 +672,20 @@ static int stress_shm_sysv_child(
 	do {
 		size_t sz = max_sz;
 		pid_t pid = -1;
+		key_t key;
 
-		exercise_shmget(sz, args->name);
-		exercise_shmctl(sz, args);
+		/* find key without an identifier associated with it */
+		if (stress_shm_sysv_get_key(args, i,
+					keys_per_instance,
+					keys_per_segment,
+					&key, keys) == 0) {
+			exercise_shmget(key, sz, args->name);
+			exercise_shmctl(key, sz, args);
+		}
 
 		for (i = 0; i < shm_sysv_segments; i++) {
 			int shm_id = -1, count = 0;
 			void *addr;
-			key_t key = 0;
 			size_t shmall, freemem, totalmem, freeswap, totalswap;
 			double t;
 
@@ -635,34 +701,23 @@ static int stress_shm_sysv_child(
 				goto reap;
 
 			for (count = 0; count < KEY_GET_RETRIES; count++) {
-				bool unique;
-				const int rnd = stress_mwc32modn(SIZEOF_ARRAY(shm_flags));
-				const int rnd_flag = shm_flags[rnd] & mask;
+				int rnd, rnd_flag;
+retry:
+				rnd = stress_mwc32modn(SIZEOF_ARRAY(shm_flags));
+				rnd_flag = shm_flags[rnd] & mask;
 
 				if (sz < page_size)
 					goto reap;
 
-				/* Get a unique key */
-				do {
-					size_t j;
-					unique = true;
-
-					if (UNLIKELY(!stress_continue_flag()))
-						goto reap;
-
-					/* Get a unique random key */
-					key = (key_t)stress_mwc16();
-					for (j = 0; j < i; j++) {
-						if (key == keys[j]) {
-							unique = false;
-							break;
-						}
-					}
-					if (UNLIKELY(!stress_continue_flag()))
-						goto reap;
-				} while (!unique);
+				/* find key without an identifier associated with it */
+				if (stress_shm_sysv_get_key(args, i,
+						keys_per_instance,
+						keys_per_segment,
+						&key, keys) < 0)
+					goto reap;
 
 				t = stress_time_now();
+errno = 0;
 				shm_id = shmget(key, sz,
 					IPC_CREAT | IPC_EXCL |
 					S_IRUSR | S_IWUSR | rnd_flag);
@@ -676,6 +731,7 @@ static int stress_shm_sysv_child(
 				if (errno == EPERM) {
 					/* ignore using the flag again */
 					mask &= ~rnd_flag;
+					goto retry;
 				}
 				if ((errno == EINVAL) || (errno == ENOMEM)) {
 					/*
@@ -711,15 +767,19 @@ static int stress_shm_sysv_child(
 			msg.index = (int)i;
 			msg.shm_id = shm_id;
 			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
-				pr_err("%s: write failed, errno=%d: (%s)\n",
-					args->name, errno, strerror(errno));
-				rc = EXIT_FAILURE;
+				if (errno != EINTR) {
+					pr_err("%s: write failed, errno=%d: (%s)\n",
+						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 				goto reap;
 			}
 
 			t = stress_time_now();
 			addr = shmat(shm_id, NULL, 0);
 			if (UNLIKELY(addr == (char *)-1)) {
+				if ((errno == EINVAL) || (errno == EIDRM))
+					goto reap;
 				ok = false;
 				pr_fail("%s: shmat on NULL address failed on id %d, (key=%d, size=%zd), errno=%d (%s)\n",
 					args->name, shm_id, (int)key, sz, errno, strerror(errno));
@@ -741,7 +801,7 @@ static int stress_shm_sysv_child(
 
 			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
-			(void)stress_madvise_random(addr, sz);
+			(void)stress_madvise_randomize(addr, sz);
 
 			if (UNLIKELY(!stress_continue(args)))
 				goto reap;
@@ -888,9 +948,11 @@ reap:
 			msg.index = (int)i;
 			msg.shm_id = -1;
 			if (UNLIKELY(write(fd, &msg, sizeof(msg)) < 0)) {
-				pr_dbg("%s: write failed, errno=%d: (%s)\n",
-					args->name, errno, strerror(errno));
-				ok = false;
+				if (errno != EINTR) {
+					pr_dbg("%s: write failed, errno=%d: (%s)\n",
+						args->name, errno, strerror(errno));
+					ok = false;
+				}
 			}
 			addrs[i] = NULL;
 			shm_ids[i] = -1;
@@ -908,9 +970,11 @@ reap:
 	msg.index = -1;
 	msg.shm_id = -1;
 	if (write(fd, &msg, sizeof(msg)) < 0) {
-		pr_err("%s: write failed, errno=%d: (%s)\n",
-			args->name, errno, strerror(errno));
-		rc = EXIT_FAILURE;
+		if (errno != EINTR) {
+			pr_err("%s: write failed, errno=%d: (%s)\n",
+				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
+		}
 	}
 	free(buffer);
 
@@ -1073,7 +1137,7 @@ fork_again:
 			 * doubles the OOM score
 			 */
 			errno = 0;
-			VOID_RET(int, nice(5));
+			VOID_RET(int, shim_nice(5));
 			if (errno != 0)
 				pr_dbg("%s: nice of child failed, errno=%d (%s) "
 					"(instance %d)\n", args->name,
