@@ -19,14 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
-
-static sigjmp_buf jmp_env;
-#if defined(SA_SIGINFO)
-static volatile void *fault_addr;
-static volatile void *expected_addr;
-static volatile int signo;
-static volatile int code;
-#endif
+#include "core-mmap.h"
 
 /*
  *  currently disabled
@@ -39,6 +32,17 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+#if defined(SA_SIGINFO)
+static volatile void *fault_addr;
+static volatile void *expected_addr;
+static volatile int signo;
+static volatile int code;
+#endif
+
+
 /*
  *  stress_bushandler()
  *	SIGBUS handler
@@ -65,6 +69,7 @@ static void NORETURN MLOCKED_TEXT stress_bushandler(
 		code = info->si_code;
 	}
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #else
 static void NORETURN MLOCKED_TEXT stress_bushandler(int signum)
@@ -79,6 +84,7 @@ static void NORETURN MLOCKED_TEXT stress_bushandler(int signum)
 			     "popf;\n");
 #endif
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #endif
 
@@ -128,9 +134,10 @@ static int stress_sigbus(stress_args_t *args)
 		PROT_READ | PROT_WRITE,
 		MAP_SHARED, fd, 0);
 	if (ptr == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of read only page failed, "
+		pr_inf_skip("%s: failed to mmap %zu byte read only pages%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size * 2,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto tidy_close;
 	}
@@ -290,9 +297,20 @@ tidy_dir:
 
 const stressor_info_t stress_sigbus_info = {
 	.stressor = stress_sigbus,
-	.class = CLASS_SIGNAL | CLASS_OS,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
 #if defined(SA_SIGINFO)
 	.verify = VERIFY_OPTIONAL,
 #endif
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_sigbus_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
