@@ -48,7 +48,8 @@ static const stress_help_t help[] = {
 #if !defined(__UCLIBC__) &&		\
     !defined(STRESS_ARCH_ARC64) &&	\
     defined(HAVE_FENV_H) &&     	\
-    defined(HAVE_FLOAT_H)
+    defined(HAVE_FLOAT_H) &&		\
+    defined(HAVE_SIGLONGJMP)
 
 #define SNG_INTDIV	(0x40000000)
 #define SNG_FLTDIV	(0x80000000)
@@ -73,6 +74,7 @@ static void NORETURN MLOCKED_TEXT stress_fpehandler(int num, siginfo_t *info, vo
 	siginfo = *info;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #else
 static void NORETURN MLOCKED_TEXT stress_fpehandler(int num)
@@ -81,6 +83,7 @@ static void NORETURN MLOCKED_TEXT stress_fpehandler(int num)
 	(void)feclearexcept(FE_ALL_EXCEPT);
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 #endif
 
