@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mmap.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"signest N",	 "start N workers generating nested signals" },
@@ -25,6 +26,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
 #define MAX_SIGNALS	(64)	/* Note: must NOT be larger than 8 * sizeof(signal_info.signalled) */
 
 static bool jmp_env_ok;
@@ -192,14 +195,18 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 
 	if (UNLIKELY(run_time > (double)g_opt_timeout)) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
 	if (UNLIKELY(signal_info.stop)) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
 	if (UNLIKELY(!signal_info.args))
@@ -208,8 +215,10 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 	stress_bogo_inc(signal_info.args);
 	if (UNLIKELY(!stress_continue(signal_info.args))) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
 	signal_info.signalled |= STRESS_BIT_ULL(signal_index);
@@ -218,8 +227,10 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 		goto done;
 
 	if (signal_info.stop || !stress_continue(signal_info.args)) {
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 	(void)shim_raise(signals[signal_index]);
 	raised++;
@@ -381,3 +392,15 @@ const stressor_info_t stress_signest_info = {
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_signest_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
