@@ -128,6 +128,7 @@ static void *stress_pthread_func(void *c)
 	cpu_cstate_t *cstate_list = stress_cpuidle_cstate_list_head();
 
 	while (stress_continue(args) && !thread_terminate) {
+		bool eintr;
 		cpu_cstate_t *cc;
 		struct timespec tv;
 		double delta, expected;
@@ -144,6 +145,7 @@ static void *stress_pthread_func(void *c)
 		 *  to try and get CPU into deeper C states
 		 */
 		expected = 0.0;
+		eintr = false;
 		stress_sleep_time_now(&t1);
 		for (cc = cstate_list; cc; cc = cc->next) {
 			if (cc->residency > 0) {
@@ -152,14 +154,16 @@ static void *stress_pthread_func(void *c)
 				expected += cc->residency;
 				tv.tv_nsec = residency_ns % 1000000000;
 				tv.tv_sec = residency_ns / 1000000000;
-				if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+				if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+					eintr |= (errno == EINTR);
 					break;
+				}
 			}
 		}
 		stress_sleep_time_now(&t2);
 		delta = stress_time_delta(&t1, &t2);
 		/* don't check for clock warping */
-		if ((expected > 0.0) && (delta > 0.0)) {
+		if (!eintr && (expected > 0.0) && (delta > 0.0)) {
 			if (UNLIKELY(delta < expected / STRESS_DBL_NANOSECOND)) {
 				pr_fail("%s: C state nanosleeps for %.f nanosecs took less than %.2f nanosecs to complete\n",
 					args->name, expected, delta * STRESS_DBL_NANOSECOND);
@@ -167,46 +171,57 @@ static void *stress_pthread_func(void *c)
 			}
 		}
 
+		eintr = false;
 		stress_sleep_time_now(&t1);
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		tv.tv_sec = 0;
 		tv.tv_nsec = 1;
-		if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+		if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		tv.tv_sec = 0;
 		tv.tv_nsec = 10;
-		if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+		if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		tv.tv_sec = 0;
 		tv.tv_nsec = 100;
-		if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+		if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		tv.tv_sec = 0;
 		tv.tv_nsec = 1000;
-		if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+		if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		tv.tv_sec = 0;
 		tv.tv_nsec = 10000;
-		if (UNLIKELY(nanosleep(&tv, NULL) < 0))
+		if (UNLIKELY(nanosleep(&tv, NULL) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		stress_sleep_time_now(&t2);
 		delta = stress_time_delta(&t1, &t2);
 		/* don't check for clock warping */
-		if (delta > 0.0) {
+		if (!eintr && (delta > 0.0)) {
 			expected = (1.0 + 10.0 + 100.0 + 1000.0 + 10000.0);
 			if (UNLIKELY(delta < expected / STRESS_DBL_NANOSECOND)) {
 				pr_fail("%s: nanosleeps for %.f nanosecs took less than %.2f nanosecs to complete\n",
@@ -215,36 +230,47 @@ static void *stress_pthread_func(void *c)
 			}
 		}
 
+		eintr = false;
 		stress_sleep_time_now(&t1);
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		if (UNLIKELY(shim_usleep(1) < 0))
+		if (UNLIKELY(shim_usleep(1) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		if (UNLIKELY(shim_usleep(10) < 0))
+		if (UNLIKELY(shim_usleep(10) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		if (UNLIKELY(shim_usleep(100) < 0))
+		if (UNLIKELY(shim_usleep(100) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		if (UNLIKELY(shim_usleep(1000) < 0))
+		if (UNLIKELY(shim_usleep(1000) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
-		if (UNLIKELY(shim_usleep(10000) < 0))
+		if (UNLIKELY(shim_usleep(10000) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		stress_sleep_time_now(&t2);
 		delta = stress_time_delta(&t1, &t2);
 		expected = (1.0 + 10.0 + 100.0 + 1000.0 + 10000.0);
-		if (UNLIKELY(delta < expected / STRESS_DBL_MICROSECOND)) {
+		if (!eintr && UNLIKELY(delta < expected / STRESS_DBL_MICROSECOND)) {
 			pr_fail("%s: nanosleeps for %.f microsecs took less than %.2f microsecs to complete\n",
 				args->name, expected, delta * STRESS_DBL_MICROSECOND);
 			ctxt->underruns++;
@@ -302,39 +328,48 @@ skip_pselect:
 
 #if defined(HAVE_SYS_SELECT_H) &&	\
     defined(HAVE_SELECT)
+		eintr = false;
 		stress_sleep_time_now(&t1);
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		timeout.tv_sec = 0;
 		timeout.tv_usec = 10;
-		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0))
+		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		timeout.tv_sec = 0;
 		timeout.tv_usec = 100;
-		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0))
+		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		timeout.tv_sec = 0;
 		timeout.tv_usec = 1000;
-		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0))
+		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		timeout.tv_sec = 0;
 		timeout.tv_usec = 10000;
-		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0))
+		if (UNLIKELY(select(0, NULL, NULL, NULL, &timeout) < 0)) {
+			eintr |= (errno == EINTR);
 			break;
+		}
 
 		stress_sleep_time_now(&t2);
 		delta = stress_time_delta(&t1, &t2);
 		expected = (1.0 + 10.0 + 100.0 + 1000.0 + 10000.0);
-		if (UNLIKELY(delta < expected / STRESS_DBL_MICROSECOND)) {
+		if (!eintr && UNLIKELY(delta < expected / STRESS_DBL_MICROSECOND)) {
 			pr_fail("%s: selects for %.f microsecs took less than %.2f microsecs to complete\n",
 				args->name, expected, delta * STRESS_DBL_MICROSECOND);
 			ctxt->underruns++;
