@@ -23,6 +23,7 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-net.h"
 
 #include <sys/ioctl.h>
@@ -109,7 +110,9 @@ static const stress_help_t help[] = {
 static const stress_sock_options_t sock_options_opts[] = {
 	{ "random",	SOCKET_OPT_RANDOM },
 	{ "send",	SOCKET_OPT_SEND },
+#if defined(HAVE_SENDMSG)
 	{ "sendmsg",	SOCKET_OPT_SENDMSG },
+#endif
 #if defined(HAVE_SENDMMSG)
 	{ "sendmmsg",	SOCKET_OPT_SENDMMSG },
 #else
@@ -309,8 +312,13 @@ static void stress_sock_ioctl(
 static void stress_sock_invalid_recv(const int fd, const int opt)
 {
 	char ALIGN64 buf[16];
+#if defined(HAVE_RECVMSG) ||	\
+    defined(HAVE_RECVMMSG)
 	struct iovec ALIGN64 vec[1];
+#endif
+#if defined(HAVE_RECVMSG)
 	struct msghdr msg;
+#endif
 #if defined(HAVE_RECVMMSG)
 	struct mmsghdr ALIGN64 msgvec[MSGVEC_SIZE];
 	struct timespec ts;
@@ -324,6 +332,7 @@ static void stress_sock_invalid_recv(const int fd, const int opt)
 		/* exercise invalid fd */
 		VOID_RET(ssize_t, recv(~0, buf, sizeof(buf), 0));
 		break;
+#if defined(HAVE_RECVMSG)
 	case SOCKET_OPT_RECVMSG:
 		vec[0].iov_base = buf;
 		vec[0].iov_len = sizeof(buf);
@@ -337,6 +346,7 @@ static void stress_sock_invalid_recv(const int fd, const int opt)
 		/* exercise invalid fd */
 		VOID_RET(ssize_t, recvmsg(~0, &msg, 0));
 		break;
+#endif
 #if defined(HAVE_RECVMMSG)
 	case SOCKET_OPT_RECVMMSG:
 		(void)shim_memset(msgvec, 0, sizeof(msgvec));
@@ -528,7 +538,9 @@ retry:
 				args->name, errno, strerror(errno));
 			goto free_controls;
 		}
-#if defined(MSG_ZEROCOPY) && defined(SO_ZEROCOPY)
+#if defined(MSG_ZEROCOPY) &&	\
+    defined(SO_ZEROCOPY) &&	\
+    defined(SOL_SOCKET)
 		if (sock_zerocopy) {
 			int so_zerocopy = 1;
 			static bool warned = false;
@@ -611,13 +623,20 @@ retry:
 				&tos, &optlen);
 		}
 #endif
+
+#if defined(SOL_SOCKET)
 		for (i = 0; i < SIZEOF_ARRAY(sol_socket_so_opts); i++) {
 			int val = 0;
 			socklen_t optlen = sizeof(val);
 
 			VOID_RET(int, getsockopt(fd, SOL_SOCKET, sol_socket_so_opts[i], &val, &optlen));
 		}
+#endif
+#if defined(AF_INET6)
 		if ((sock_domain == AF_INET) || (sock_domain == AF_INET6)) {
+#else
+		if (sock_domain == AF_INET) {
+#endif
 
 #if defined(TCP_NODELAY)
 			{
@@ -755,9 +774,14 @@ retry:
 
 		do {
 			ssize_t n = 0;
+#if defined(HAVE_RECVMSG) ||	\
+    defined(HAVE_RECVMMSG)
 			size_t j;
-			struct msghdr ALIGN64 msg;
 			struct iovec ALIGN64 vec[MMAP_IO_SIZE / 16];
+#endif
+#if defined(HAVE_RECVMSG)
+			struct msghdr ALIGN64 msg;
+#endif
 #if defined(HAVE_RECVMMSG)
 			struct mmsghdr ALIGN64 msgvec[MSGVEC_SIZE];
 			const int max_opt = 3;
@@ -804,6 +828,7 @@ retry:
 			case SOCKET_OPT_RECV:
 				n = recv(fd, buf, MMAP_IO_SIZE, recvflag);
 				break;
+#if defined(HAVE_RECVMSG)
 			case SOCKET_OPT_RECVMSG:
 				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 					vec[j].iov_base = buf;
@@ -814,6 +839,7 @@ retry:
 				msg.msg_iovlen = j;
 				n = recvmsg(fd, &msg, recvflag);
 				break;
+#endif
 #if defined(HAVE_RECVMMSG)
 			case SOCKET_OPT_RECVMMSG:
 				(void)shim_memset(msgvec, 0, sizeof(msgvec));
@@ -943,7 +969,9 @@ static int OPTIMIZE3 stress_sock_server(
 		goto die;
 	}
 
-#if defined(MSG_ZEROCOPY) && defined(SO_ZEROCOPY)
+#if defined(MSG_ZEROCOPY) &&	\
+    defined(SO_ZEROCOPY) &&	\
+    defined(SOL_SOCKET)
 	if (sock_zerocopy) {
 		int so_zerocopy = 1;
 		static bool warned = false;
@@ -962,6 +990,7 @@ static int OPTIMIZE3 stress_sock_server(
 #else
 	(void)sock_zerocopy;
 #endif
+#if defined(SOL_SOCKET)
 	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
 		&so_reuseaddr, sizeof(so_reuseaddr)) < 0) {
 		pr_fail("%s: setsockopt failed, errno=%d (%s)\n",
@@ -976,11 +1005,11 @@ static int OPTIMIZE3 stress_sock_server(
 	/* exercise invalid setsockopt fd */
 	(void)setsockopt(-1, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
 
-	/* exercise invalid level */
-	(void)setsockopt(fd, -1, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
-
 	/* exercise invalid optname */
 	(void)setsockopt(fd, SOL_SOCKET, -1, &so_reuseaddr, sizeof(so_reuseaddr));
+#endif
+	/* exercise invalid level */
+	(void)setsockopt(fd, -1, SO_REUSEADDR, &so_reuseaddr, sizeof(so_reuseaddr));
 
 	if (stress_set_sockaddr_if(args->name, args->instance, ppid,
 			sock_domain, sock_port, sock_if,
@@ -1038,12 +1067,21 @@ static int OPTIMIZE3 stress_sock_server(
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
 #endif
 		if (LIKELY(sfd >= 0)) {
-			size_t i, j, k;
+			size_t i, k;
+#if defined(HAVE_SENDMSG) ||	\
+    defined(HAVE_SENDMMSG)
+			size_t j;
+#endif
 			struct sockaddr saddr;
 			socklen_t len;
 			int sndbuf, opt;
-			struct msghdr ALIGN64 msg;
+#if defined(HAVE_SENDMSG) ||	\
+    defined(HAVE_SENDMMSG)
 			struct iovec ALIGN64 vec[MMAP_IO_SIZE / 16];
+#endif
+#if defined(HAVE_SENDMSG)
+			struct msghdr ALIGN64 msg;
+#endif
 #if defined(HAVE_SENDMMSG)
 			struct mmsghdr ALIGN64 msgvec[MSGVEC_SIZE];
 #endif
@@ -1079,12 +1117,15 @@ static int OPTIMIZE3 stress_sock_server(
 			}
 
 			len = sizeof(sndbuf);
+#if defined(SOL_SOCKET) &&	\
+    defined(SO_SNDBUF)
 			if (UNLIKELY(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, &len) < 0)) {
 				pr_fail("%s: getsockopt failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				(void)close(sfd);
 				break;
 			}
+#endif
 #if defined(SO_RESERVE_MEM)
 			{
 				const int mem = 4 * 1024 * 1024;
@@ -1149,6 +1190,7 @@ retry_send:
 						}
 					}
 					break;
+#if defined(HAVE_SENDMSG)
 				case SOCKET_OPT_SENDMSG:
 					for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 						vec[j].iov_base = buf;
@@ -1171,6 +1213,7 @@ retry_sendmsg:
 						msgs += j;
 					}
 					break;
+#endif
 #if defined(HAVE_SENDMMSG)
 				case SOCKET_OPT_SENDMMSG:
 					(void)shim_memset(msgvec, 0, sizeof(msgvec));
