@@ -24,7 +24,10 @@
 #include "core-net.h"
 
 #include <sys/ioctl.h>
+
+#if defined(HAVE_SYS_FILE_H)
 #include <sys/file.h>
+#endif
 
 #if defined(HAVE_SYS_XATTR_H)
 #include <sys/xattr.h>
@@ -256,6 +259,7 @@ static int stress_sockabuse_server(
 				args->name, errno, strerror(errno));
 			continue;
 		}
+#if defined(SOL_SOCKET)
 		if (UNLIKELY(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
 			&so_reuseaddr, sizeof(so_reuseaddr)) < 0)) {
 			rc = stress_exit_status(errno);
@@ -264,6 +268,7 @@ static int stress_sockabuse_server(
 			(void)close(fd);
 			continue;
 		}
+#endif
 
 		if (UNLIKELY(stress_set_sockaddr(args->name, args->instance, mypid,
 				AF_INET, sockabuse_port,
@@ -311,12 +316,15 @@ static int stress_sockabuse_server(
 					break;
 				}
 				len = sizeof(sndbuf);
+#if defined(SOL_SOCKET) &&	\
+    defined(SO_SNDBUF)
 				if (UNLIKELY(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, &len) < 0)) {
 					pr_fail("%s: getsockopt failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					(void)close(sfd);
 					break;
 				}
+#endif
 				(void)shim_memset(buf, stress_ascii64[stress_bogo_get(args) & 63], sizeof(buf));
 
 				n = send(sfd, buf, sizeof(buf), 0);
@@ -429,7 +437,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_sockabuse_info = {
 	.stressor = stress_sockabuse,
-	.class = CLASS_NETWORK | CLASS_OS,
+	.classifier = CLASS_NETWORK | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
