@@ -148,6 +148,7 @@ static void MLOCKED_TEXT NORETURN stress_rlimit_handler(int signum)
 
 	stress_continue_set_flag(false);
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
 
 /*
@@ -368,7 +369,7 @@ again:
 
 	softlockup_start = true;
 
-	(void)pause();
+	(void)shim_pause();
 	rc = stress_kill_and_wait_many(args, s_pids, (size_t)cpus_online, SIGALRM, false);
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
