@@ -18,7 +18,9 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
+#include "core-prime.h"
 
 #if defined(HAVE_SYS_TREE_H)
 #include <sys/tree.h>
