@@ -19,7 +19,9 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-cpu-cache.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pragma.h"
 #include "core-put.h"
@@ -34,8 +36,6 @@ typedef struct stress_stack_check {
 	struct stress_stack_check *self_addr;	/* Address of this struct to check */
 } stress_stack_check_t;
 
-static sigjmp_buf jmp_env;
-
 static const stress_help_t help[] = {
 	{ NULL,	"stack N",	"start N workers generating stack overflows" },
 	{ NULL,	"stack-fill",	"fill stack, touches all new pages " },
@@ -54,6 +54,10 @@ static const stress_opt_t opts[] = {
 	END_OPT,
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+
 /*
  *  stress_segvhandler()
  *	SEGV handler
@@ -63,6 +67,7 @@ static void MLOCKED_TEXT NORETURN stress_segvhandler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 
 /*
@@ -92,7 +97,7 @@ static bool OPTIMIZE3 stress_stack_alloc(
 		return true;
 
 	if (stack_fill) {
-		(void)shim_memset(data, 0, STRESS_DATA_SIZE);
+		(void)shim_memset(data, stress_mwc8(), STRESS_DATA_SIZE);
 	} else {
 		register size_t i;
 
@@ -108,6 +113,8 @@ static bool OPTIMIZE3 stress_stack_alloc(
 			*(ptr + 1) = stress_mwc32() | 1;
 		}
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush(data, STRESS_DATA_SIZE);
 #if defined(HAVE_MLOCK)
 	if (stack_mlock) {
 		intptr_t ptr = ((intptr_t)data) + ((intptr_t)page_size - 1);
@@ -188,13 +195,25 @@ static int stress_stack_child(stress_args_t *args, void *context)
 
 	(void)context;
 
-	(void)stress_get_setting("stack-fill", &stack_fill);
-	(void)stress_get_setting("stack-mlock", &stack_mlock);
-	(void)stress_get_setting("stack-pageout", &stack_pageout);
-	(void)stress_get_setting("stack-unmap", &stack_unmap);
+	if (!stress_get_setting("stack-fill", &stack_fill)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			stack_fill = true;
+	}
+	if (!stress_get_setting("stack-mlock", &stack_mlock)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			stack_mlock = true;
+	}
+	if (!stress_get_setting("stack-pageout", &stack_pageout)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			stack_pageout = true;
+	}
+	if (!stress_get_setting("stack-unmap", &stack_unmap)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			stack_unmap = true;
+	}
 
 #if !defined(MADV_PAGEOUT)
-	if (stack_pageout && (args->instance == 0)) {
+	if (stack_pageout && (stress_instance_zero(args))) {
 		pr_inf("%s: stack-pageout not supported on this system\n", args->name);
 		stack_pageout = false;
 	}
@@ -210,9 +229,10 @@ static int stress_stack_child(stress_args_t *args, void *context)
 		PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (altstack == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate %zd byte signal stack: "
+		pr_inf_skip("%s: failed to mmap %zd byte signal stack%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, (size_t)STRESS_SIGSTKSZ, errno, strerror(errno));
+			args->name, (size_t)STRESS_SIGSTKSZ,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(altstack, STRESS_SIGSTKSZ, "altstack");
@@ -233,7 +253,7 @@ static int stress_stack_child(stress_args_t *args, void *context)
 	stress_parent_died_alarm();
 
 	if (start_ptr == (void *) -1) {
-		pr_err("%s: sbrk(0) failed: errno=%d (%s)\n",
+		pr_err("%s: sbrk(0) failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -308,8 +328,21 @@ static int stress_stack(stress_args_t *args)
 
 const stressor_info_t stress_stack_info = {
 	.stressor = stress_stack,
-	.class = CLASS_VM | CLASS_MEMORY,
+	.classifier = CLASS_VM | CLASS_MEMORY,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_stack_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_VM | CLASS_MEMORY,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
