@@ -24,9 +24,9 @@
 #endif
 
 static const stress_help_t help[] = {
-	{ NULL,	"mount N",	"start N workers exercising statmount and listmount" },
-	{ NULL,	"mount-ops N", 	"stop after N bogo statmount and listmount operations" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	"statmount N",	   "start N workers exercising statmount and listmount" },
+	{ NULL,	"statmount-ops N", "stop after N bogo statmount and listmount operations" },
+	{ NULL,	NULL,		   NULL }
 };
 
 #if defined(__linux__) &&		\
@@ -199,7 +199,7 @@ static int stress_statmount(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_inf("%s: %d mount points exercised by statmount\n", args->name, max_mounts);
 
 	rate = (duration > 0.0) ? count / duration  : 0.0;
@@ -212,14 +212,14 @@ static int stress_statmount(stress_args_t *args)
 
 const stressor_info_t stress_statmount_info = {
 	.stressor = stress_statmount,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_statmount_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without Linux statmount or listmount"
