@@ -21,6 +21,7 @@
 #include "core-affinity.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 
 #if defined(HAVE_MQUEUE_H)
 #include <mqueue.h>
@@ -165,8 +166,9 @@ static int stress_switch_pipe(
 			PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (UNLIKELY(buf == MAP_FAILED)) {
-		pr_fail("%s: pipe read/write buffer allocation failed\n",
-			args->name);
+		pr_fail("%s: failed to mmap %zu byte pipe read/write buffer%s, errno=%d (%s)\n",
+			args->name, buf_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
 		return EXIT_FAILURE;
@@ -197,7 +199,7 @@ again:
 			goto again;
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -300,7 +302,7 @@ static int stress_switch_sem_sysv(
 			break;
 	}
 	if (sem_id < 0) {
-		pr_err("%s: semaphore init (SYSV) failed: errno=%d (%s)\n",
+		pr_err("%s: semaphore init (SYSV) failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
                 return EXIT_FAILURE;
         }
@@ -312,7 +314,7 @@ again:
 	if (pid < 0) {
 		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -360,7 +362,7 @@ again:
 			if (UNLIKELY(switch_freq))
 				stress_switch_delay(args, switch_delay, threshold, t_start, &delay);
 
-			if (!stress_continue(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 			sem.sem_num = 0;
 			sem.sem_op = -1;
@@ -415,7 +417,7 @@ static int stress_switch_mq(
 	attr.mq_curmsgs = 0;
 	mq = mq_open(mq_name, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR, &attr);
 	if (mq < 0) {
-		pr_err("%s: message queue open failed: errno=%d (%s)\n",
+		pr_err("%s: message queue open failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
                 return EXIT_FAILURE;
 	}
@@ -428,7 +430,7 @@ again:
 	if (pid < 0) {
 		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -475,7 +477,7 @@ finish:
 }
 #endif
 
-static stress_switch_method_t stress_switch_methods[] = {
+static const stress_switch_method_t stress_switch_methods[] = {
 #if defined(HAVE_MQUEUE_H) &&   \
     defined(HAVE_LIB_RT) &&     \
     defined(HAVE_MQ_POSIX)
@@ -526,7 +528,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_switch_info = {
 	.stressor = stress_switch,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
