@@ -19,9 +19,11 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-capabilities.h"
 #include "core-cpu-cache.h"
 #include "core-killpid.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 
 #include <sys/ioctl.h>
@@ -34,6 +36,12 @@
 #include <termios.h>
 #endif
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#else
+UNEXPECTED
+#endif
+
 #if defined(HAVE_SYS_STATFS_H)
 #include <sys/statfs.h>
 #else
@@ -306,7 +314,7 @@ static void bad_bind(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_cacheflush(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(int, shim_cacheflush(ba->addr, 4096, SHIM_DCACHE));
+	stress_cpu_data_cache_flush(ba->addr, 4096);
 }
 #endif
 
@@ -700,6 +708,7 @@ static void bad_getdomainname(stress_bad_addr_t *ba, volatile uint64_t *counter)
 }
 #endif
 
+#if defined(HAVE_GETGROUPS)
 static void bad_getgroups(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unwriteable) {
@@ -707,6 +716,7 @@ static void bad_getgroups(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		VOID_RET(int, getgroups(8192, (gid_t *)ba->addr));
 	}
 }
+#endif
 
 #if defined(HAVE_GETHOSTNAME)
 static void bad_gethostname(stress_bad_addr_t *ba, volatile uint64_t *counter)
@@ -922,10 +932,10 @@ static void bad_getsockname3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 
 static void bad_gettimeofday1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
-	struct timezone *tz = (struct timezone *)inc_addr(ba->addr, 1);
+	shim_timezone_t *tz = (shim_timezone_t *)inc_addr(ba->addr, 1);
 
 	(*counter)++;
-	VOID_RET(int, gettimeofday((struct timeval *)ba->addr, tz));
+	VOID_RET(int, shim_gettimeofday((struct timeval *)ba->addr, tz));
 }
 
 static void bad_gettimeofday2(stress_bad_addr_t *ba, volatile uint64_t *counter)
@@ -933,16 +943,16 @@ static void bad_gettimeofday2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	struct timeval tv;
 
 	(*counter)++;
-	VOID_RET(int, gettimeofday(&tv, (struct timezone *)ba->addr));
+	VOID_RET(int, shim_gettimeofday(&tv, (shim_timezone_t *)ba->addr));
 }
 
 static void bad_gettimeofday3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unwriteable) {
-		struct timezone tz;
+		shim_timezone_t tz;
 
 		(*counter)++;
-		VOID_RET(int, gettimeofday((struct timeval *)ba->addr, &tz));
+		VOID_RET(int, shim_gettimeofday((struct timeval *)ba->addr, &tz));
 	}
 }
 
@@ -1464,7 +1474,7 @@ static void bad_preadv(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		fd = open("/dev/zero", O_RDONLY);
 		if (fd > -1) {
 			(*counter)++;
-			VOID_RET(ssize_t, readv(fd, (struct iovec *)ba->addr, 1));
+			VOID_RET(ssize_t, preadv(fd, (struct iovec *)ba->addr, 1, 0));
 			VOID_RET(int, close(fd));
 		}
 	}
@@ -1652,7 +1662,7 @@ static void bad_readlink3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	}
 }
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_READV)
 static void bad_readv(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unwriteable) {
@@ -1988,6 +1998,7 @@ static void bad_utime(stress_bad_addr_t *ba, volatile uint64_t *counter)
 }
 #endif
 
+#if defined(HAVE_UTIMES)
 static void bad_utimes1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
@@ -1995,7 +2006,9 @@ static void bad_utimes1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		VOID_RET(int, utimes(ba->addr, (const struct timeval *)inc_addr(ba->addr, 1)));
 	}
 }
+#endif
 
+#if defined(HAVE_UTIMES)
 static void bad_utimes2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
@@ -2003,7 +2016,9 @@ static void bad_utimes2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		VOID_RET(int, utimes(stress_get_temp_path(), (const struct timeval *)ba->addr));
 	}
 }
+#endif
 
+#if defined(HAVE_UTIMES)
 static void bad_utimes3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
@@ -2011,6 +2026,7 @@ static void bad_utimes3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		VOID_RET(int, utimes(ba->addr, NULL));
 	}
 }
+#endif
 
 static void bad_wait(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
@@ -2054,7 +2070,7 @@ static void bad_write(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	}
 }
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_WRITEV)
 static void bad_writev(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
@@ -2072,7 +2088,7 @@ static void bad_writev(stress_bad_addr_t *ba, volatile uint64_t *counter)
 UNEXPECTED
 #endif
 
-static stress_bad_syscall_t bad_syscalls[] = {
+static const stress_bad_syscall_t bad_syscalls[] = {
 	bad_access,
 /*
 	bad_acct,
@@ -2144,7 +2160,9 @@ static stress_bad_syscall_t bad_syscalls[] = {
 #if defined(HAVE_GETDOMAINNAME)
 	bad_getdomainname,
 #endif
+#if defined(HAVE_GETGROUPS)
 	bad_getgroups,
+#endif
 	bad_get_mempolicy1,
 	bad_get_mempolicy2,
 	bad_get_mempolicy3,
@@ -2310,7 +2328,7 @@ static stress_bad_syscall_t bad_syscalls[] = {
 	bad_readlink1,
 	bad_readlink2,
 	bad_readlink3,
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_READV)
 	bad_readv,
 #endif
 #if defined(HAVE_REMOVEXATTR) &&	\
@@ -2362,16 +2380,22 @@ static stress_bad_syscall_t bad_syscalls[] = {
 #if defined(HAVE_UTIME_H)
 	bad_utime,
 #endif
+#if defined(HAVE_UTIMES)
 	bad_utimes1,
+#endif
+#if defined(HAVE_UTIMES)
 	bad_utimes2,
+#endif
+#if defined(HAVE_UTIMES)
 	bad_utimes3,
+#endif
 	bad_wait,
 	bad_waitpid,
 #if defined(HAVE_WAITID)
 	bad_waitid,
 #endif
 	bad_write,
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_WRITEV)
 	bad_writev,
 #endif
 };
@@ -2450,13 +2474,14 @@ static inline int stress_do_syscall(stress_args_t *args)
 		}
 		_exit(EXIT_SUCCESS);
 	} else {
-		int ret, status;
+		pid_t ret;
+		int status;
 
 		ret = shim_waitpid(pid, &status, 0);
 		if (ret < 0) {
 			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_dbg("%s: waitpid() on PID %" PRIdMAX" failed, errno=%d (%s)\n",
+					args->name, (intmax_t)pid, errno, strerror(errno));
 			(void)stress_kill_pid_wait(pid, &status);
 		}
 		rc = WEXITSTATUS(status);
@@ -2516,9 +2541,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 		sizeof(*state), PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (state == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous state structure: "
+		pr_inf_skip("%s: failed to mmap %zu byte anonymous state structure%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, sizeof(*state),
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2527,9 +2553,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 	ro_page = stress_mmap_populate(NULL, page_size,
 		PROT_READ, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (ro_page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous read-only page: "
+		pr_inf_skip("%s: failed to mmap %zu byte anonymous read-only page%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2540,9 +2567,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 		PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (rw_page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous read-write page: "
+		pr_inf_skip("%s: failed to mmap %zu byte anonymous read-write page%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size << 1,
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2553,9 +2581,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 		PROT_EXEC | PROT_READ,
 		MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (rx_page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous execute-only page: "
+		pr_inf_skip("%s: failed to mmap %zu byte anonymous execute-only page%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2565,9 +2594,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 	no_page = stress_mmap_populate(NULL, page_size,
 		PROT_NONE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (no_page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous prot-none page: "
+		pr_inf_skip("%s: failed to mmap %zu anonymous prot-none page%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2576,9 +2606,10 @@ static int stress_sysbadaddr(stress_args_t *args)
 	wo_page = stress_mmap_populate(NULL, page_size,
 		PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (wo_page == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap anonymous write-only page: "
+		pr_inf_skip("%s: failed to mmap %zu byte anonymous write-only page%s, "
 		       "errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
@@ -2627,6 +2658,6 @@ cleanup:
 
 const stressor_info_t stress_sysbadaddr_info = {
 	.stressor = stress_sysbadaddr,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.help = help
 };
