@@ -22,7 +22,9 @@
 #include "core-capabilities.h"
 #include "core-hash.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-pthread.h"
+#include "core-prime.h"
 #include "core-put.h"
 #include "core-try-open.h"
 
@@ -90,7 +92,7 @@ static void stress_sysfs_sys_power_disk(const char *path)
 	(void)stress_system_write(path, "test_resume", 11);
 }
 
-static stress_sysfs_wr_func_t stress_sysfs_wr_funcs[] = {
+static const stress_sysfs_wr_func_t stress_sysfs_wr_funcs[] = {
 #if defined(__linux__)
 	{ "/sys/power/disk",	stress_sysfs_sys_power_disk },
 #endif
@@ -725,7 +727,7 @@ static int stress_sysfs(stress_args_t *args)
 #endif
 	sysfs_hash_table = stress_hash_create(1021);
 	if (!sysfs_hash_table) {
-		pr_err("%s: cannot create sysfs hash table: %d (%s))\n",
+		pr_err("%s: cannot create sysfs hash table, errno=%d (%s))\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto exit_free;
@@ -793,14 +795,15 @@ again:
 				goto finish;
 			}
 		} else if (pid > 0) {
+			pid_t wret;
 			int status;
 
 			/* Parent, wait for child */
-			ret = waitpid(pid, &status, 0);
-			if (ret < 0) {
+			wret = waitpid(pid, &status, 0);
+			if (wret < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 				/* Ring ring, time to die */
 				stress_kill_and_wait(args, pid, SIGALRM, true);
 			} else {
@@ -902,7 +905,7 @@ exit_free:
 	return rc;
 
 exit_no_sysfs_entries:
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_inf_skip("%s: no /sys entries found, skipping stressor\n", args->name);
 	rc = EXIT_NO_RESOURCE;
 	goto exit_free;
@@ -910,14 +913,14 @@ exit_no_sysfs_entries:
 
 const stressor_info_t stress_sysfs_info = {
 	.stressor = stress_sysfs,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
 const stressor_info_t stress_sysfs_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "not Linux or built without pthread support"
