@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-capabilities.h"
 #include "core-killpid.h"
 #include "core-out-of-memory.h"
 #include <sys/socket.h>
@@ -2407,6 +2408,7 @@ static void MLOCKED_TEXT stress_syscall_itimer_handler(int sig)
 		do_jmp = false;
 		current_context->type = SYSCALL_TIMED_OUT;
 		siglongjmp(jmpbuf, 1);
+		stress_no_return();
 	}
 }
 
