@@ -64,8 +64,8 @@ static int stress_tsearch(stress_args_t *args)
 	n = (size_t)tsearch_size;
 
 	if ((data = (int32_t *)calloc(n, sizeof(*data))) == NULL) {
-		pr_fail("%s: calloc failed allocating %zd integers, skipping stressor\n",
-			args->name, n);
+		pr_fail("%s: failed to allocating %zu integers%s, skipping stressor\n",
+			args->name, n, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -152,7 +152,7 @@ abort:
 
 const stressor_info_t stress_tsearch_info = {
 	.stressor = stress_tsearch,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -162,7 +162,7 @@ const stressor_info_t stress_tsearch_info = {
 
 const stressor_info_t stress_tsearch_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
