@@ -19,6 +19,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 
 #define UNLINK_PROCS		(3)
 #define UNLINK_FILES		(1024)	/* must be power of 2 and less than 65536 */
@@ -92,7 +93,7 @@ static void stress_unlink_exercise(
 		for (i = 0; i < UNLINK_FILES; i++)
 			fds[i] = -1;
 
-		for (i = 0; stress_continue(args) && (i < UNLINK_FILES); i++) {
+		for (i = 0; LIKELY(stress_continue(args) && (i < UNLINK_FILES)); i++) {
 			int mode, retries = 0;
 
 			if (UNLIKELY((i & 7) == 7)) {
@@ -203,8 +204,10 @@ static int stress_unlink(stress_args_t *args)
 					PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (metrics == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap %zu bytes for metrics, skipping stressor\n",
-			args->name, metrics_sz);
+		pr_inf_skip("%s: failed to mmap %zu bytes for metrics%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, metrics_sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(metrics, metrics_sz, "metrics");
@@ -228,10 +231,11 @@ static int stress_unlink(stress_args_t *args)
 			'a' + stress_mwc8modn(26),
 			'a' + stress_mwc8modn(26), i);
 
-		filenames[i] = strdup(filename);
+		filenames[i] = shim_strdup(filename);
 		if (!filenames[i]) {
-			pr_inf_skip("%s: failed to allocate filenames, "
-				"skipping stressor\n", args->name);
+			pr_inf_skip("%s: failed to allocate filenames%s, "
+				"skipping stressor\n", args->name,
+				stress_get_memfree_str());
 			goto filenames_free;
 		}
 	}
@@ -294,7 +298,7 @@ metrics_free:
 
 const stressor_info_t stress_unlink_info = {
 	.stressor = stress_unlink,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_NONE,
 	.help = help
 };
