@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,8 @@
  */
 #include "stress-ng.h"
 
+#include <time.h>
+
 #if defined(HAVE_SYS_AUXV_H)
 #include <sys/auxv.h>
 #else
@@ -92,7 +94,7 @@ static int OPTIMIZE3 wrap_getcpu(void *vdso_func)
  */
 static int OPTIMIZE3 wrap_gettimeofday(void *vdso_func)
 {
-	int (*vdso_gettimeofday)(struct timeval *tv, struct timezone *tz);
+	int (*vdso_gettimeofday)(struct timeval *tv, shim_timezone_t *tz);
 	struct timeval tv;
 
 	*(void **)(&vdso_gettimeofday) = vdso_func;
@@ -230,8 +232,8 @@ static int dl_wrapback(struct dl_phdr_info* info, size_t info_size, void *vdso)
 			ElfW(Word *) hash = NULL;
 			ElfW(Word) j;
 			ElfW(Word) buckets = 0;
-			ElfW(Word *) bucket = NULL;
-			ElfW(Word *) chain = NULL;
+			const ElfW(Word *) bucket = NULL;
+			const ElfW(Word *) chain = NULL;
 			char *strtab = NULL;
 
 			if (!load_offset)
@@ -298,7 +300,7 @@ static int dl_wrapback(struct dl_phdr_info* info, size_t info_size, void *vdso)
 							/*
 							 *  Add to list of wrapable vDSO functions
 							 */
-							vdso_sym = calloc(1, sizeof(*vdso_sym));
+							vdso_sym = (stress_vdso_sym_t *)calloc(1, sizeof(*vdso_sym));
 							if (vdso_sym == NULL)
 								return -1;
 
@@ -415,7 +417,7 @@ static void vdso_sym_list_remove_duplicates(stress_vdso_sym_t **list)
  */
 static int stress_vdso_supported(const char *name)
 {
-	unsigned long vdso = getauxval(AT_SYSINFO_EHDR);
+	unsigned long int vdso = getauxval(AT_SYSINFO_EHDR);
 
 	if (vdso == 0) {
 		pr_inf_skip("%s stressor will be skipped, failed to find vDSO address\n", name);
@@ -484,7 +486,7 @@ static int stress_vdso(stress_args_t *args)
 
 	if (!vdso_sym_list) {
 		/* Should not fail, but worth checking to avoid breakage */
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: could not find any vDSO functions, skipping\n",
 				args->name);
 		return EXIT_NOT_IMPLEMENTED;
@@ -494,7 +496,7 @@ static int stress_vdso(stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		char *str;
 
 		str = vdso_sym_list_str();
@@ -558,14 +560,14 @@ static int stress_vdso(stress_args_t *args)
 const stressor_info_t stress_vdso_info = {
 	.stressor = stress_vdso,
 	.supported = stress_vdso_supported,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.help = help
 };
 #else
 const stressor_info_t stress_vdso_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without sys/auxv.h, link.h, getauxval() or AT_SYSINFO_EHDR defined"
