@@ -94,8 +94,9 @@ static int OPTIMIZE3 stress_vm_child(void *arg)
 		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
 		rc = stress_exit_status(errno);
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: failed to mmap %zu bytes%s, errno=%d (%s)\n",
+			args->name, ctxt->sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		goto cleanup;
 	}
 	stress_set_vma_anon_name(buf, ctxt->sz, "context");
@@ -186,8 +187,9 @@ static int OPTIMIZE3 stress_vm_parent(stress_context_t *ctxt)
 	localbuf = mmap(NULL, ctxt->sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (localbuf == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: failed to mmap %zu bytes%s, errno=%d (%s)\n",
+			args->name, ctxt->sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)close(ctxt->pipe_wr[0]);
 		(void)close(ctxt->pipe_wr[1]);
 		(void)close(ctxt->pipe_rd[0]);
@@ -364,20 +366,22 @@ static int stress_vm_rw(stress_args_t *args)
 	stress_context_t ctxt;
 	uint8_t stack[64*1024];
 	uint8_t *stack_top = (uint8_t *)stress_get_stack_top((void *)stack, STACK_SIZE);
-	size_t vm_rw_bytes = DEFAULT_VM_RW_BYTES;
+	size_t vm_rw_bytes, vm_rw_bytes_total = DEFAULT_VM_RW_BYTES;
 	int rc;
 
-	if (!stress_get_setting("vm-rw-bytes", &vm_rw_bytes)) {
+	if (!stress_get_setting("vm-rw-bytes", &vm_rw_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			vm_rw_bytes = MAX_32;
+			vm_rw_bytes_total = MAX_32;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			vm_rw_bytes = MIN_VM_RW_BYTES;
+			vm_rw_bytes_total = MIN_VM_RW_BYTES;
 	}
-	vm_rw_bytes /= args->instances;
+	vm_rw_bytes = vm_rw_bytes_total / args->instances;
 	if (vm_rw_bytes < MIN_VM_RW_BYTES)
 		vm_rw_bytes = MIN_VM_RW_BYTES;
 	if (vm_rw_bytes < args->page_size)
 		vm_rw_bytes = args->page_size;
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, vm_rw_bytes, vm_rw_bytes * args->instances);
 	ctxt.args = args;
 	ctxt.sz = vm_rw_bytes & ~(args->page_size - 1);
 	ctxt.iov_count = (ctxt.sz + CHUNK_SIZE - 1) / CHUNK_SIZE;
@@ -423,7 +427,7 @@ again:
 
 const stressor_info_t stress_vm_rw_info = {
 	.stressor = stress_vm_rw,
-	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -431,7 +435,7 @@ const stressor_info_t stress_vm_rw_info = {
 #else
 const stressor_info_t stress_vm_rw_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
