@@ -19,6 +19,8 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-prime.h"
+#include "core-mmap.h"
 
 #define MIN_VM_SPLICE_BYTES	(4 * KB)
 #define MAX_VM_SPLICE_BYTES	(64 * MB)
@@ -48,39 +50,43 @@ static int stress_vm_splice(stress_args_t *args)
 	int fd, fds[2], rc = EXIT_SUCCESS;
 	uint8_t *buf;
 	const size_t page_size = args->page_size;
-	size_t sz, vm_splice_bytes = DEFAULT_VM_SPLICE_BYTES;
+	size_t sz, vm_splice_bytes, vm_splice_bytes_total = DEFAULT_VM_SPLICE_BYTES;
 	char *data;
 	double duration = 0.0, bytes = 0.0, vm_splices = 0.0, rate;
 	int metrics_counter = 0;
 	uint64_t checkval = stress_mwc64();
 	uint64_t prime;
 
-	if (!stress_get_setting("vm-splice-bytes", &vm_splice_bytes)) {
+	if (!stress_get_setting("vm-splice-bytes", &vm_splice_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			vm_splice_bytes = MAX_VM_SPLICE_BYTES;
+			vm_splice_bytes_total = MAX_VM_SPLICE_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			vm_splice_bytes = MIN_VM_SPLICE_BYTES;
+			vm_splice_bytes_total = MIN_VM_SPLICE_BYTES;
 	}
-	vm_splice_bytes /= args->num_instances;
+	vm_splice_bytes = vm_splice_bytes_total / args->instances;
 	if (vm_splice_bytes < MIN_VM_SPLICE_BYTES)
 		vm_splice_bytes = MIN_VM_SPLICE_BYTES;
 	if (vm_splice_bytes < page_size)
 		vm_splice_bytes = page_size;
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, vm_splice_bytes, vm_splice_bytes * args->instances);
 	sz = vm_splice_bytes & ~(page_size - 1);
 
 	buf = stress_mmap_populate(NULL, sz, PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of %zd sized buffer failed, errno=%d (%s), skipping stressor\n",
-			args->name, sz, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zu bytes buffer%s, errno=%d (%s), skipping stressor\n",
+			args->name, sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(buf, sz, "iov-buffer");
 	data = stress_mmap_populate(NULL, page_size, PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (data == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of %zd sized buffer failed, errno=%d (%s), skipping stressor\n",
-			args->name, page_size, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zu sized buffer%s, errno=%d (%s), skipping stressor\n",
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)buf, sz);
 		return EXIT_NO_RESOURCE;
 	}
@@ -200,7 +206,7 @@ static int stress_vm_splice(stress_args_t *args)
 
 const stressor_info_t stress_vm_splice_info = {
 	.stressor = stress_vm_splice,
-	.class = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -208,7 +214,7 @@ const stressor_info_t stress_vm_splice_info = {
 #else
 const stressor_info_t stress_vm_splice_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_PIPE_IO | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
