@@ -26,11 +26,13 @@
 #include "core-target-clones.h"
 #include "core-madvise.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 #include "core-nt-load.h"
 #include "core-nt-store.h"
 #include "core-numa.h"
 #include "core-out-of-memory.h"
 #include "core-pragma.h"
+#include "core-prime.h"
 #include "core-vecmath.h"
 
 #define MIN_VM_BYTES		(4 * KB)
@@ -3614,7 +3616,7 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 #endif
 
 			if (advice < 0)
-				(void)stress_madvise_random(buf, buf_sz);
+				(void)stress_madvise_randomize(buf, buf_sz);
 			else
 				(void)shim_madvise(buf, buf_sz, advice);
 #if defined(HAVE_LINUX_MEMPOLICY_H)
@@ -3636,7 +3638,7 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 		}
 
 		if (!vm_keep) {
-			(void)stress_madvise_random(buf, buf_sz);
+			(void)stress_madvise_randomize(buf, buf_sz);
 #if defined(HAVE_MPROTECT) &&	\
     defined(PROT_NONE)
 			(void)stress_munmap_retry_enomem(buf, buf_sz + page_size);
@@ -3776,7 +3778,9 @@ static int stress_vm(stress_args_t *args)
 
 	ret = stress_oomable_child(args, &context, stress_vm_child, STRESS_OOMABLE_NORMAL);
 
+#if defined(MS_SYNC)
 	(void)shim_msync(context.bit_error_count, page_size, MS_SYNC);
+#endif
 	if (*context.bit_error_count > 0) {
 		pr_fail("%s: detected %" PRIu64 " bit errors while "
 			"stressing memory\n",
