@@ -20,7 +20,9 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-cpu-cache.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
 #include "core-put.h"
@@ -95,11 +97,11 @@ static bool stress_vma_continue_flag;
 
 static bool stress_vma_continue(stress_args_t *args)
 {
-	if (UNLIKELY(!g_stress_continue_flag))
+	if (UNLIKELY(!stress_continue_flag()))
 		return false;
-	if (LIKELY(args->max_ops == 0))
+	if (LIKELY(args->bogo.max_ops == 0))
 		return true;
-        return stress_vma_metrics->s.metrics[STRESS_VMA_MMAP] < args->max_ops;
+        return stress_vma_metrics->s.metrics[STRESS_VMA_MMAP] < args->bogo.max_ops;
 }
 
 #if defined(__linux__) &&		\
@@ -338,7 +340,7 @@ static void *stress_vma_munlock(void *ptr)
 		const size_t offset = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
 		const size_t len = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
 
-		if (LIKELY(munlock((void *)(data + offset), len) == 0))
+		if (LIKELY(shim_munlock((void *)(data + offset), len) == 0))
 			stress_vma_metrics->s.metrics[STRESS_VMA_MUNLOCK]++;
 	}
 	return NULL;
@@ -355,6 +357,7 @@ static void *stress_vma_madvise(void *ptr)
 	stress_args_t *args = (stress_args_t *)ctxt->args;
 	const uintptr_t data = (uintptr_t)ctxt->data;
 	const size_t page_size = args->page_size;
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 
 	static const int advice[] = {
 #if defined(MADV_NORMAL)
@@ -402,6 +405,8 @@ static void *stress_vma_madvise(void *ptr)
 
 		if (madvise((void *)(data + offset), len, advice[i]) == 0)
 			stress_vma_metrics->s.metrics[STRESS_VMA_MADVISE]++;
+		if (aggressive)
+			stress_cpu_data_cache_flush((void *)ptr, page_size);
 	}
 	return NULL;
 }
@@ -498,7 +503,7 @@ static void *stress_vma_msync(void *ptr)
 		const size_t offset = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
 		const size_t len = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
 
-		if (LIKELY(msync((void *)(data + offset), len, flags[i]) == 0))
+		if (LIKELY(shim_msync((void *)(data + offset), len, flags[i]) == 0))
 			stress_vma_metrics->s.metrics[STRESS_VMA_MSYNC]++;
 	}
 	return NULL;
@@ -542,6 +547,7 @@ static void *stress_vma_access(void *ptr)
 	stress_args_t *args = (stress_args_t *)ctxt->args;
 	const uintptr_t data = (uintptr_t)ctxt->data;
 	const size_t page_size = args->page_size;
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 
 	while (stress_vma_continue_flag && stress_vma_continue(args)) {
 		const size_t offset = page_size * stress_mwc8modn(STRESS_VMA_PAGES);
@@ -549,6 +555,8 @@ static void *stress_vma_access(void *ptr)
 
 		stress_vma_metrics->s.metrics[STRESS_VMA_ACCESS]++;
 		stress_uint8_put(*ptr8);
+		if (aggressive)
+			stress_cpu_data_cache_flush((void *)ptr, page_size);
 	}
 	return NULL;
 }
@@ -652,8 +660,7 @@ static void stress_vma_loop(
 
 		(void)sleep(10);
 		stress_vma_continue_flag = false;
-		VOID_RET(int, kill(pid, SIGKILL));
-		VOID_RET(int, shim_waitpid(pid, &status, 0));
+		stress_kill_pid_wait(pid, &status);
 	} while (stress_vma_continue(args));
 }
 
@@ -664,7 +671,7 @@ static int stress_vma_child(stress_args_t *args, void *void_ctxt)
 	stress_vma_context_t *ctxt = (stress_vma_context_t *)void_ctxt;
 	int ret;
 
-	s_pids = stress_s_pids_mmap(STRESS_VMA_PROCS);
+	s_pids = stress_sync_s_pids_mmap(STRESS_VMA_PROCS);
 	if (s_pids == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, STRESS_VMA_PROCS);
 		return EXIT_NO_RESOURCE;
@@ -700,7 +707,7 @@ static int stress_vma_child(stress_args_t *args, void *void_ctxt)
 	} while (stress_continue(args));
 
 	ret = stress_kill_and_wait_many(args, s_pids, i, SIGALRM, false);
-	(void)stress_s_pids_munmap(s_pids, STRESS_VMA_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, STRESS_VMA_PROCS);
 
 	return ret;
 }
@@ -762,13 +769,13 @@ static int stress_vma(stress_args_t *args)
 
 const stressor_info_t stress_vma_info = {
 	.stressor = stress_vma,
-	.class = CLASS_VM,
+	.classifier = CLASS_VM,
 	.help = help
 };
 #else
 const stressor_info_t stress_vma_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM,
+	.classifier = CLASS_VM,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
 };
