@@ -188,6 +188,28 @@ static void stress_waitcpu_ppc64_mdoom(void)
 }
 #endif
 
+#if defined(STRESS_ARCH_PPC)
+static bool stress_waitcpu_ppc_supported(void)
+{
+	return true;
+}
+
+static void stress_waitcpu_ppc_yield(void)
+{
+	stress_asm_ppc_yield();
+}
+
+static void stress_waitcpu_ppc_mdoio(void)
+{
+	stress_asm_ppc_mdoio();
+}
+
+static void stress_waitcpu_ppc_mdoom(void)
+{
+	stress_asm_ppc_mdoom();
+}
+#endif
+
 #if defined(STRESS_ARCH_RISCV)
 static bool stress_waitcpu_riscv_pause_supported(void)
 {
@@ -243,6 +265,11 @@ stress_waitcpu_method_t stress_waitcpu_method[] = {
 	{ "mdoom",	stress_waitcpu_ppc64_mdoom,	stress_waitcpu_ppc64_supported,		false, 0.0, 0.0, 0.0 },
 	{ "yield",	stress_waitcpu_ppc64_yield,	stress_waitcpu_ppc64_supported,		false, 0.0, 0.0, 0.0 },
 #endif
+#if defined(STRESS_ARCH_PPC)
+	{ "mdoio",	stress_waitcpu_ppc_mdoio,	stress_waitcpu_ppc_supported,		false, 0.0, 0.0, 0.0 },
+	{ "mdoom",	stress_waitcpu_ppc_mdoom,	stress_waitcpu_ppc_supported,		false, 0.0, 0.0, 0.0 },
+	{ "yield",	stress_waitcpu_ppc_yield,	stress_waitcpu_ppc_supported,		false, 0.0, 0.0, 0.0 },
+#endif
 #if defined(STRESS_ARCH_LOONG64)
 #if defined(HAVE_ASM_LOONG64_DBAR)
 	{ "dbar",	stress_waitcpu_loong64_dbar,	stress_waitcpu_loong64_dbar_supported,	false, 0.0, 0.0, 0.0 },
@@ -277,12 +304,12 @@ static int stress_waitcpu(stress_args_t *args)
 		stress_waitcpu_method[i].count = 0.0;
 	}
 	if (!supported) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: no CPU wait/pause instructions available, skipping stressor\n",
 				args->name);
 		return EXIT_NO_RESOURCE;
 	}
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_inf("%s: exercising instruction%s:%s\n", args->name,
 			SIZEOF_ARRAY(stress_waitcpu_method) > 1 ? "s" : "",
 			str);
@@ -381,7 +408,7 @@ static int stress_waitcpu(stress_args_t *args)
 
 const stressor_info_t stress_waitcpu_info = {
 	.stressor = stress_waitcpu,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
