@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-helper.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 
 #include <sys/ioctl.h>
@@ -84,8 +85,10 @@ static int stress_zero(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (rd_buffer == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate page sized read buffer, skipping test\n",
-			args->name);
+		pr_inf_skip("%s: failed to mmap %zu byte read buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(rd_buffer, page_size, "read-buffer");
@@ -95,8 +98,10 @@ static int stress_zero(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (wr_buffer == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate page sized write buffer, skipping test\n",
-			args->name);
+		pr_inf_skip("%s: failed to mmap %zu byte write buffer%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap(rd_buffer, page_size);
 		return EXIT_NO_RESOURCE;
 	}
@@ -119,7 +124,7 @@ static int stress_zero(stress_args_t *args)
 		double t;
 		ssize_t ret = 0;
 
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: exercising /dev/zero with just reads\n", args->name);
 
 		t = stress_time_now();
@@ -151,7 +156,7 @@ static int stress_zero(stress_args_t *args)
 		size_t mmap_index = 0;
 #endif
 
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: exercising /dev/zero with reads, mmap, lseek, and ioctl; for just read benchmarking use --zero-read\n",
 				args->name);
 		do {
@@ -285,7 +290,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_zero_info = {
 	.stressor = stress_zero,
-	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.opts = opts,
 	.help = help
