@@ -24,6 +24,7 @@
 #include "core-builtin.h"
 #include "core-cpu.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-target-clones.h"
 
 #include <ctype.h>
@@ -41,13 +42,20 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-#if defined(HAVE_LIB_Z)
+#if defined(HAVE_LIB_Z) &&	\
+    defined(HAVE_SIGLONGJMP)
 
 #include "zlib.h"
 
 #define DATA_SIZE_64K 	(KB * 64)	/* Must be a multiple of 64 bytes */
 #define DATA_SIZE DATA_SIZE_64K
 
+#define ZLIB_MIN_COMPRESSION	(0)
+#define ZLIB_MAX_COMPRESSION	(Z_BEST_COMPRESSION)
+
+#define ZLIB_MIN_MEM_LEVEL	(1)
+#define ZLIB_MAX_MEM_LEVEL	(9)
+
 typedef void (*stress_zlib_rand_data_func)(stress_args_t *args,
 	uint64_t *RESTRICT data, uint64_t *RESTRICT data_end);
 
@@ -186,6 +194,7 @@ static void NORETURN MLOCKED_TEXT stress_bad_read_handler(int signum)
 	(void)signum;
 
 	siglongjmp(jmpbuf, 1);
+	stress_no_return();
 }
 
 /*
@@ -1349,9 +1358,10 @@ static const char *stress_zlib_method(const size_t i)
 	return (i < SIZEOF_ARRAY(zlib_rand_data_methods)) ? zlib_rand_data_methods[i].name : NULL;
 }
 
+
 static const stress_opt_t opts[] = {
-	{ OPT_zlib_level,        "zlib-level",        TYPE_ID_UINT32, 0, Z_BEST_COMPRESSION, NULL },
-	{ OPT_zlib_mem_level,    "zlib-mem-level",    TYPE_ID_UINT32, 1, 9, NULL },
+	{ OPT_zlib_level,        "zlib-level",        TYPE_ID_UINT32, ZLIB_MIN_COMPRESSION, ZLIB_MAX_COMPRESSION, NULL },
+	{ OPT_zlib_mem_level,    "zlib-mem-level",    TYPE_ID_UINT32, ZLIB_MIN_MEM_LEVEL, ZLIB_MAX_MEM_LEVEL, NULL },
 	{ OPT_zlib_method,       "zlib-method",       TYPE_ID_SIZE_T_METHOD, 0, 0, stress_zlib_method },
 	{ OPT_zlib_window_bits,  "zlib-window-bits",  TYPE_ID_CALLBACK, 0, 0, stress_zlib_window_bits },
 	{ OPT_zlib_stream_bytes, "zlib-stream-bytes", TYPE_ID_UINT64_BYTES_VM, 0, MAX_MEM_LIMIT, NULL },
@@ -1393,15 +1403,25 @@ static const char *stress_zlib_err(const int zlib_err)
  *	get all zlib arguments at once
  */
 static void stress_zlib_get_args(stress_zlib_args_t *params) {
-	params->level = Z_BEST_COMPRESSION;
+	params->level = ZLIB_MAX_COMPRESSION;
 	params->mem_level = 8;
 	params->method = 0;
 	params->window_bits = 15;
 	params->stream_bytes = 0;
 	params->strategy = Z_DEFAULT_STRATEGY;
 
-	(void)stress_get_setting("zlib-level", &params->level);
-	(void)stress_get_setting("zlib-mem-level", &params->mem_level);
+	if (!stress_get_setting("zlib-level", &params->level)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			params->level = Z_BEST_COMPRESSION;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			params->level = 0;
+	}
+	if (!stress_get_setting("zlib-mem-level", &params->mem_level)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			params->level = ZLIB_MAX_MEM_LEVEL;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			params->level = ZLIB_MIN_MEM_LEVEL;
+	}
 	(void)stress_get_setting("zlib-method", &params->method);
 	(void)stress_get_setting("zlib-window-bits", &params->window_bits);
 	(void)stress_get_setting("zlib-stream-bytes", &params->stream_bytes);
@@ -1440,7 +1460,7 @@ static int stress_zlib_inflate(
 	stream_inf.avail_in = 0;
 	stream_inf.next_in = Z_NULL;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("INF: lvl=%d mem-lvl=%d wbits=%d strategy=%d\n",
 			zlib_args.level, zlib_args.mem_level, zlib_args.window_bits,
 			zlib_args.strategy);
@@ -1448,7 +1468,7 @@ static int stress_zlib_inflate(
 	do {
 		ret = inflateInit2(&stream_inf, zlib_args.window_bits);
 		if (UNLIKELY(ret != Z_OK)) {
-			pr_fail("%s: zlib inflateInit error: %s\n",
+			pr_fail("%s: zlib inflateInit error, %s\n",
 				args->name, stress_zlib_err(ret));
 			zlib_checksum->error = true;
 			goto zlib_checksum_error;
@@ -1464,7 +1484,7 @@ static int stress_zlib_inflate(
 			} else if ((sz != sizeof(def_size)) || (sz < 0)) {
 				(void)inflateEnd(&stream_inf);
 				if (UNLIKELY((errno != EINTR) && (errno != EPIPE))) {
-					pr_fail("%s: zlib pipe read size error: %s (ret=%zd errno=%d)\n",
+					pr_fail("%s: zlib pipe read size error, %s (ret=%zd errno=%d)\n",
 						args->name, strerror(errno), sz, errno);
 					zlib_checksum->error = true;
 					goto zlib_checksum_error;
@@ -1483,7 +1503,7 @@ static int stress_zlib_inflate(
 			} else if ((sz != def_size) || (sz < 0)) {
 				(void)inflateEnd(&stream_inf);
 				if (UNLIKELY((errno != EINTR) && (errno != EPIPE))) {
-					pr_fail("%s: zlib pipe read buffer error: %s (ret=%zd errno=%d)\n",
+					pr_fail("%s: zlib pipe read buffer error, %s (ret=%zd errno=%d)\n",
 						args->name, strerror(errno), sz, errno);
 					zlib_checksum->error = true;
 					goto zlib_checksum_error;
@@ -1509,7 +1529,7 @@ static int stress_zlib_inflate(
 				case Z_NEED_DICT:
 				case Z_DATA_ERROR:
 				case Z_MEM_ERROR:
-					pr_fail("%s: zlib inflate error: %s\n",
+					pr_fail("%s: zlib inflate error, %s\n",
 						args->name, stress_zlib_err(ret));
 					(void)inflateEnd(&stream_inf);
 					goto zlib_checksum_error;
@@ -1580,7 +1600,7 @@ static int stress_zlib_deflate(
 	if (zlib_args.window_bits > 31)
 		zlib_args.window_bits = zlib_args.window_bits - 32;
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		pr_dbg("DEF: lvl=%d mem-lvl=%d wbits=%d strategy=%d stream-bytes=%llu\n",
 			zlib_args.level, zlib_args.mem_level, zlib_args.window_bits,
 			zlib_args.strategy, (unsigned long long int)zlib_args.stream_bytes);
@@ -1590,7 +1610,7 @@ static int stress_zlib_deflate(
 				Z_DEFLATED, zlib_args.window_bits,
 				zlib_args.mem_level, zlib_args.strategy);
 		if (UNLIKELY(ret != Z_OK)) {
-			pr_fail("%s: zlib deflateInit error: %s\n",
+			pr_fail("%s: zlib deflateInit error, %s\n",
 				args->name, stress_zlib_err(ret));
 			zlib_checksum->error = true;
 			(void)deflateEnd(&stream_def);
@@ -1645,7 +1665,7 @@ static int stress_zlib_deflate(
 
 				ret = deflate(&stream_def, flush);
 				if (UNLIKELY(ret == Z_STREAM_ERROR)) {
-					pr_fail("%s: zlib deflate error: %s\n",
+					pr_fail("%s: zlib deflate error, %s\n",
 						args->name, stress_zlib_err(ret));
 					(void)deflateEnd(&stream_def);
 					ret = EXIT_FAILURE;
@@ -1666,7 +1686,7 @@ static int stress_zlib_deflate(
 				} else if (sz != sizeof(def_size)) {
 					(void)deflateEnd(&stream_def);
 					if (UNLIKELY((errno != EINTR) && (errno != EPIPE) && (errno != 0))) {
-						pr_fail("%s: zlib pipe write size error: %s (ret=%zd errno=%d)\n",
+						pr_fail("%s: zlib pipe write size error, %s (ret=%zd errno=%d)\n",
 							args->name, strerror(errno), sz, errno);
 						ret = EXIT_FAILURE;
 						goto zlib_checksum_error;
@@ -1685,7 +1705,7 @@ static int stress_zlib_deflate(
 				} else if (sz != def_size) {
 					(void)deflateEnd(&stream_def);
 					if (UNLIKELY((errno != EINTR) && (errno != EPIPE) && (errno != 0))) {
-						pr_fail("%s: zlib pipe write buffer error: %s (ret=%zd errno=%d)\n",
+						pr_fail("%s: zlib pipe write buffer error, %s (ret=%zd errno=%d)\n",
 							args->name, strerror(errno), sz, errno);
 						ret = EXIT_FAILURE;
 						goto zlib_checksum_error;
@@ -1750,8 +1770,10 @@ static int stress_zlib(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (shared_checksums == MAP_FAILED) {
-		pr_inf("%s: failed to mmap %zd bytes, skipping stressor\n",
-			args->name, sizeof(*shared_checksums));
+		pr_inf("%s: failed to mmap %zu bytes%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, sizeof(*shared_checksums),
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 	stress_set_vma_anon_name(shared_checksums, sizeof(*shared_checksums), "zlib-checksums");
@@ -1852,7 +1874,7 @@ again:
 
 const stressor_info_t stress_zlib_info = {
 	.stressor = stress_zlib,
-	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -1860,9 +1882,9 @@ const stressor_info_t stress_zlib_info = {
 #else
 const stressor_info_t stress_zlib_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
-	.unimplemented_reason = "built without zlib library support"
+	.unimplemented_reason = "built without zlib library support or siglongjmp support"
 };
 #endif
